﻿--[[
	Original portions of this script are Copyright 2013-2014 DeathByNukes (http://deathbynukes.com/)
	Based on Audiosurf 2's "neon" skin.

	You may freely reuse any or all of this code for your own Audiosurf 2 skin as long as you
	credit DeathByNukes in your skin's lua source file or an enclosed txt file.

	The above text only applies to this lua source file. See DBN_assets/COPYING.txt for
	information on the other files. If you have any questions, contact dbn@deathbynukes.com.

	This skin is distributed without any warranty whatsoever. DeathByNukes is not responsible
	for anything done with it or any problems that arise from its use.
]]

--@define AZURE_CLASSIC

print([[Azure 194 loading]])

local function THE_ENTIRE_SCRIPT()

---------------------------------------------------------------------------------------------------------------------------
-- Options
---------------------------------------------------------------------------------------------------------------------------

--psst, let us have a UI panel for skin settings
options = {
	debug = false,
	spectrum_fps = 60, -- framerate limit of spectrum visualizer effects. comment this out to not limit it at all
	debug_map_camera = true, -- show the camera view frustum on the debug map. eats fps.
	debug_camera = false, -- lifts up the wakeboard crouch camera and the vehicle camera
	--screen_resolution = {1920, 1080}, -- TODO

	deterministic = false, -- if you play the same song twice, the skin will have the exact same appearance
	shuffle = false, -- randomly changes these options. you probably want to switch off "deterministic" when using this
	shuffle_extra = false, -- randomly changes other parts of the skin that options don't control
	
	
	invert = false, -- black on white color scheme
	skywires = 1/32, -- opacity
	
	glow = false,
	glow_foreground = true, -- glow for some objects on top of the track
	glow_foreground_passes = 4,
	glow_foreground_radius = 2.5,
	glow_foreground_lastpass_radius = 1.5, -- on the last pass do a smaller size to smooth out the artifacts from the previous passes
	troll = false, -- modifies glow_foreground
	
	water = true,
	water_transparency = true,
	water_wake = true,
	force_water_wake = false, -- show wakes even if it's not a wakeboard mod
	
	enable_structures = true, -- random buildings in the background
	struct_hex = {
		vertices = 6, -- bump this up to 12 or so for actual circles (higher = smoother and more lag)
		start_health = 1, -- waves lose 1 HP per second when the speed scale is 1
		fade_health = 0.5, -- the point when waves start fading out
		wave_brightness = 1/4,
		wave_size_min = 1/10,
		-- power ^ exponent * scale + add
		speed_exponent = 4,
		speed_scale = 6,
		speed_add = 1,
		upscale = "uphill", -- scale up the audio intensity: uphill, downhill, never, always
		-- for now the structure placement system assumes "uphill". "downhill" and "always" may hurt gameplay.
	},
	
	track_colors = {
		{255, 160,   0}, -- orange (low intensity)
		{248,  38,   0}, -- red
		{174,   0, 255}, -- purple
		{  0, 127, 255}, -- blue
		{134, 201, 255}, -- bright blue (high intensity)
	},
	
	block_colors = { -- puzzle colors. (I left these at the values from Neon)
		{93,    8,      132   },
		{1.0,   0.27,   0.2   },
		{0.047, 0.5098, 0.94  },
		{1.0,   0.85,   0.4157},
		{0,     0.29,   0.08  },
		{1,     0,      0     },
	},
	
	track_surface = true, -- ignored if water is on
	
	track_shoulders = true, -- shoulders may be automatically deleted if they're right on the edge of the track surface
	track_lanes = true,
	track_lanes_minimum = 1, -- mod must have at least this many lane dividers for them to be shown
	
	track_rings = true,
	air_rings = true, -- disabling this makes the graphics less confusing while jumping. I was tempted to release the skin with them disabled.
	tunnel = false, -- featured in my "Tunnel" skin. included here because why not
	
	track_aurora_ceiling = true,
	track_aurora_edge = false, -- made for playing with no track surface
	
	
	overrides = {
		flat = {
			water = false,
		},
		flight = {
			water = false,
			force_water_wake = false, -- the wakes look pretty good in flight but I think most people using this version prefer simplicity
			track_surface = false,
			track_aurora_ceiling = false,
			track_aurora_edge = true,
		},
		white = {
			invert = true,
			track_colors = {
				{255, 160,   0}, -- orange (low intensity)
				{248,  38,   0}, -- red
				{174,   0, 255}, -- purple
				{  0, 127, 255}, -- blue
				{ 58, 156, 255}, -- light blue (high intensity)
			},
			force_water_wake = false,
			water = false, -- can't set custom water shader and can't customize it well enough to match it with this theme
			
		},
		shuffle = {
			deterministic = false,
			shuffle = true,
			shuffle_extra = true,
		},
	},
	-- "override" can be set to the name of one of the sub-tables in "overrides", a function that returns a name, or nil
	-- options present in the specified table will override normal options
	-- the function will be evaluated after the utility functions have been defined and after options.deterministic has been applied
	-- the override can still change options.deterministic though. that will be detected and the setting will be re-applied
	override = "flight",
	--override = "flat",
	--override = nil,
	
	--override = function() return pick(nil, unpack(tableKeys(options.overrides))) end, -- pick one of the overrides randomly
	--override = function() return pick(nil, 'flat', 'flight') end,
	--override = function() return pick('flat', 'flight') end,
}

-- TODO
--[[
LoadSounds{
	hit="hit.wav",
	hitgrey="hitgrey.wav",
	trickfail="hitgrey.wav",
	matchsmall="matchsmall.wav",
	matchmedium="matchmedium.wav",
	matchlarge="matchlarge.wav",
	matchhuge="matchhuge.wav",
	overfillwarning="overfillwarning.wav"
}
--]]

---------------------------------------------------------------------------------------------------------------------------
-- Shuffle
---------------------------------------------------------------------------------------------------------------------------

-- if options.shuffle is true, settings present in this block will override the settings above
-- you can remove or comment out some of them if you wish for only partial randomness
-- it is applied after options.override is applied and override cannot be shuffled here
function doShuffle()
	local new_options = {
		invert = chance(4),
		

		troll = chance(20),
		
		water = chance(3),
		water_transparency = coinflip(),
		water_wake = coinflip(),
		force_water_wake = coinflip(),
		
		enable_structures = highchance(16),
		
		track_shoulders = highchance(3),
		track_lanes = highchance(3),
		track_lanes_minimum = pick(3,1),
		
		track_rings = highchance(8),
		air_rings = coinflip(),
		tunnel = chance(10),
		
		track_aurora_edge = coinflip(),
		
		track_surface = highchance(3),
		
		track_aurora_ceiling = highchance(4),
	}
	if chance(3) then
		new_options.track_colors = shallowCopy(options.track_colors)
		flipArray(new_options.track_colors)
	elseif chance(5) then
		new_options.track_colors = shallowCopy(options.track_colors)
		randomSortArray(new_options.track_colors)
	end
	if chance(3) then
		new_options.block_colors = shallowCopy(options.block_colors)
		flipArray(new_options.block_colors)
	elseif chance(5) then
		new_options.block_colors = shallowCopy(options.block_colors)
		randomSortArray(new_options.block_colors)
	end
	return new_options
end

---------------------------------------------------------------------------------------------------------------------------
-- Debug Hooks
---------------------------------------------------------------------------------------------------------------------------
if options.debug then
	local function layerCheck(table, level)
		if table.layer == nil then
			error("Specify a layer!", level or 3)
		end
	end
	local function gameobjectCheck(go, level)
		if go == nil then return end
		level = (level or 3) + 1
		layerCheck(go, level)
		local children = go.childobjects
		if children ~= nil then
			for i=1,#children do gameobjectCheck(children[i], level) end
		end
	end
	
	local _CreateRail = CreateRail
	function CreateRail(table)
		layerCheck(table)
		if table.renderqueue == nil and (not table.allowfullmaterialoptions or table.material == nil) then
			error("Specify a renderqueue!", 3)
		end
		return _CreateRail(table)
	end

	local _CreateObject = CreateObject
	function CreateObject(table)
		gameobjectCheck(table.gameobject)
		return _CreateObject(table)
	end

	local _SetPlayer = SetPlayer
	function SetPlayer(table)
		gameobjectCheck(table.vehicle)
		gameobjectCheck(table.airvehicle)
		gameobjectCheck(table.watervehicle)
		local surfer = table.surfer
		if surfer ~= nil then
			gameobjectCheck(surfer.arms)
			gameobjectCheck(surfer.leg)
			gameobjectCheck(surfer.body)
			gameobjectCheck(surfer.board)
		end
		return _SetPlayer(table)
	end

	local _CreateRepeatedMeshRail = CreateRepeatedMeshRail
	function CreateRepeatedMeshRail(table)
		gameobjectCheck(table.gameobject)
		return _CreateRepeatedMeshRail(table)
	end
	
	local _AddForest = AddForest
	function AddForest(table)
		local trees = table.trees
		if trees ~= nil then
			for i=1,#trees do gameobjectCheck(trees[i].gameobject) end
		end
		return _AddForest(table)
	end

	local _BatchRenderEveryFrame = BatchRenderEveryFrame
	function BatchRenderEveryFrame(s)
		local n = #s.locations
		if s.offsets and #s.offsets ~= 3 and #s.offsets ~= n then error("incorrect number of offsets") end
		if s.scales and #s.scales ~= n then error("incorrect number of scales") end
		if s.colors and #s.colors ~= n then error("incorrect number of colors") end
		return _BatchRenderEveryFrame(s)
	end
end

---------------------------------------------------------------------------------------------------------------------------
-- Headers
---------------------------------------------------------------------------------------------------------------------------

do -- common.luainc
local pairs, ipairs, next, type = pairs, ipairs, next, type
local insert, concat = table.insert, table.concat

math.tau = math.pi * 2

function fif(test, if_true, if_false) if test then return if_true else return if_false end end



	-- ASSERT_TYPE(variable_or_expression,string) -- DO NOT pad the arguments with whitespace!
	


--ASSERT_TYPE("foo",string)
--ASSERT_TYPE(1 + 1,string)

function sign(num)
	if num < 0 then return -1 end
	return 1
end

-- returns add(listener), fire(...)
function createEvent()
	local listeners = {}
	return function(listener)
		insert(listeners, listener)
	end,
	function(...)
		for i=1,#listeners do listeners[i](...) end
	end
end

-- FFT_BINS == #GetSpectrum()


-- reverse sorts a number indexed table
function flipArray(t)
	local L = 1
	local R = #t
	while L < R do
		t[L], t[R] = t[R], t[L]
		L = L + 1
		R = R - 1
	end
	return t
end
-- shallow copies a number indexed table in reverse order
function copyFlipArray(t)
	local n = {}
	for i=#t,1,-1 do insert(n, t[i]) end
	return n
end

function concatTable(t, concat)
	for k,v in pairs(concat) do t[k] = v end
	return t
end

-- copy everything from the base table to the overrides table except for things that already exist there
-- works recursively for sub-tables
do
	local _Inherit
	function Inherit(base, overrides)
		for k,v in pairs(base) do
			over_v = overrides[k]
			if over_v == nil then
				overrides[k] = v
			elseif type(over_v) == "table" and type(v) == "table" then
				overrides[k] = _Inherit(v, over_v)
			end
		end
		return overrides
	end
	_Inherit = Inherit
end

function clones(value, count)
	local tbl = {}
	for i=1,count do tbl[i] = value end
	return unpack(tbl)
end
function clonesArray(value, count)
	local tbl = {}
	for i=1,count do tbl[i] = value end
	return tbl
end

white = {1, 1, 1}
black = {0, 0, 0}
function greyscale(amount, alpha) return {amount,amount,amount,alpha} end -- can omit alpha
function normalizeColor(c)
	if c[1] and c[2] and c[3] then return c end
	return {c.r, c.g, c.b, c.a}
end
function colorAlpha(c, a)
	c = {c[1] or c.r, c[2] or c.g, c[3] or c.b, a}
	if a ~= 0 and (c[1] > 1 or c[2] > 1 or c[3] > 1) then
		c[1], c[2], c[3] = c[1]/255, c[2]/255, c[3]/255
	end
	return c
end

-- for iterating over a table with the keys sorted alphabetically
-- http://www.lua.org/pil/19.3.html
do
	local sort = table.sort
	function pairsByKeys (t, f)
		local a = {}
		for n in pairs(t) do insert(a, n) end
		sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
		return iter
	end
end

-- http://lua-users.org/wiki/TableUtils
do
	local string, tostring = string, tostring
	local _tableValToStr, _tableKeyToStr, _tableToString
	function tableValToStr ( v )
		if "string" == type( v ) then
			v = string.gsub( v, "\n", "\\n" )
			if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
				return "'" .. v .. "'"
			end
			return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
		else
			return "table" == type( v ) and _tableToString( v ) or
				tostring( v )
		end
	end
	function tableKeyToStr ( k )
		if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
			return k
		else
			return "[" .. _tableValToStr( k ) .. "]"
		end
	end
	function tableToString( tbl )
		local result, done = {}, {}
		for k, v in ipairs( tbl ) do
			insert( result, _tableValToStr( v ) )
			done[ k ] = true
		end
		for k, v in pairs( tbl ) do
			if not done[ k ] then
				insert( result,
					_tableKeyToStr( k ) .. "=" .. _tableValToStr( v ) )
			end
		end
		return "{" .. concat( result, "," ) .. "}"
	end
	_tableValToStr, _tableKeyToStr, _tableToString = tableValToStr, tableKeyToStr, tableToString

	local pairsByKeys = pairsByKeys
	-- modified version of tableToString by DBN
	function tableToPrettyString( tbl )
		local result, done = {}, {}
		for k, v in ipairs( tbl ) do
			insert( result, "" )
			insert( result, _tableValToStr( v ) )
			done[ k ] = true
		end
		local indent = 0
		for k, v in pairsByKeys( tbl, function(a,b) return tostring(a) < tostring(b) end ) do
			if not done[ k ] then
				local key = _tableKeyToStr( k )
				if string.len(key) > indent then indent = string.len(key) end
				insert( result, key )
				insert( result, " = " .. _tableValToStr( v ) )
			end
		end
		--if indent ~= 0 then
			local last = #result - 1
			for i=1,last,2 do
				result[i] = result[i] .. string.rep(" ", indent - string.len(result[i]))
				if i ~= last then result[i+1] = result[i+1] .. ",\n\t" end
			end
		--end
		return "{\n\t" .. concat( result ) .. "\n}"
	end
end

do
	local setmetatable, getmetatable = setmetatable, getmetatable
	local _deepCopy
	function deepCopy(orig) -- a pretty standard lua function for making a copy of a table
		local orig_type = type(orig)
		local copy
		if orig_type == 'table' then
			copy = {}
			for orig_key, orig_value in next, orig, nil do
				copy[_deepCopy(orig_key)] = _deepCopy(orig_value)
			end
			setmetatable(copy, _deepCopy(getmetatable(orig)))
		else -- number, string, boolean, etc
			copy = orig
		end
		return copy
	end
	_deepCopy = deepCopy
end

function shallowCopy(orig)
	local copy = {}
	for orig_key, orig_value in next, orig, nil do
		copy[orig_key] = orig_value
	end
	return copy
end

function tableKeys(table)
	local keys = {}
	for key in pairs(table) do insert(keys, key) end
	return keys
end

-- AS2 API functions are supposed to handle lua objects with "using (obj) {}" blocks or at least call .Dispose() on them but it doesn't. (for skins, anyways. some mod functions are getting patched)
-- we can use these functions to try to stave off "out of memory" errors as long as possible
function destroy(tbl)
	for k in pairs(tbl) do tbl[k] = nil end
end
function destroyRecursive(tbl)
	for k, v in pairs(tbl) do
		if type(v) == "table" then destroyRecursive(v) end
		tbl[k] = nil
	end
end


function makeCylinderThruster(radius, points, inside_out)
	local min, max, step
	if inside_out then min, max, step = 0, points-1, 1
	else min, max, step = points-1, 0, -1 end
	
	local shape = {}
	local tau = math.tau
	for i=min,max,step do
		local angle = tau * (i/points)
		insert(shape, {radius * math.cos(angle), radius * math.sin(angle), 0})
	end
	return shape
end

end -- common.luainc
-- Gaussian random, aka normal distribution: https://en.wikipedia.org/wiki/Normal_distribution
-- Generates random numbers that are more likely to be a specified number
-- takes two arguments, mean and std_dev
-- mean is the number that is most likely to be generated
-- std_dev (standard deviation) is the distance from the mean which should contain about 68.27% of the values (https://en.wikipedia.org/wiki/68-95-99.7_rule)

function gaussRand(mean, std_dev)
	return gaussRandRaw() * std_dev + mean
end

-- Returned number will not be less than mean
function gaussRandHalf(mean, std_dev)
	return math.abs(gaussRandRaw()) * std_dev + mean
end
-- Returned number will not be greater than mean
function gaussRandLowerHalf(mean, std_dev)
	return -math.abs(gaussRandRaw()) * std_dev + mean
end

-- Returned number will not be negative
function gaussRandPositive(mean, std_dev)
	return math.max(0, gaussRandRaw() * std_dev + mean)
end

-- Retries until it gets a number in the desired range. Make sure that min is lower than max or you will get an infinite loop. :)
function gaussRandRange(mean, std_dev, min, max)
	local r
	repeat
		r = gaussRandRaw() * std_dev + mean
	until r >= min and r <= max
	return r
end

-- careful not to specify a max that is below mean
function gaussRandHalfRange(mean, std_dev, min, max)
	local r
	repeat
		r = math.abs(gaussRandRaw()) * std_dev + mean
	until r >= min and r <= max
	return r
end

do
	local next_rand = 0
	local next_rand_ready = false
	
	-- Returns a random number with a mean of 0 and a std dev of 1.
	function gaussRandRaw()
		-- Implemented with the Box–Muller polar transform (Marsaglia polar method)
		if next_rand_ready then
			next_rand_ready = false
			return next_rand
		end
		local r1, r2, w
		repeat
			r1 = math.random() * 2 - 1
			r2 = math.random() * 2 - 1
			w = r1*r1 + r2*r2
		until w < 1 and w ~= 0
		w = math.sqrt( -2 * math.log( w ) / w )
		next_rand_ready = true
		next_rand = r2 * w
		return r1 * w
	end
end


-- picks a random argument
function pick(...)
	return select(math.random(select('#', ...)), ...)
end
-- picks a random item from an array
function pickArray(array)
	return array[math.random(#array)]
end

-- an array of items with a number before them in the array to indicate their chances. items with a higher chance get more entries in the lottery
function pickArrayWeighted(array)
	local lottery = {}
	local value, chance
	for i=1,#array,2 do
		chance = array[i]
		value = array[i+1]
		for n=1,chance do table.insert(lottery, value) end
	end
	return lottery[math.random(#lottery)]
end

function coinflip()
	if math.random(2) == 1 then return true end
	return false
end

-- a one in X chance of returning true
function chance(odds)
	if math.random(odds) == 1 then return true end
	return false
end
-- a one in X chance of returning false
function highchance(odds)
	if math.random(odds) == 1 then return false end
	return true
end

-- randomly sorts a table (modifies the original)
function randomSortArray(tbl)
	local new = {}
	while #tbl > 0 do
		local i = math.random(#tbl)
		table.insert(new, tbl[i])
		table.remove(tbl, i)
	end
	for i=1,#new do tbl[i] = new[i] end
	return tbl
end


do
	local original_randomseed, modf = math.randomseed, math.modf
	function math.randomseed(entropy)
		-- math.randomseed ignores decimals, converts the number to a C "int", then casts it to an unsigned int...
		local fraction
		entropy, fraction = modf(entropy)
		fraction = string.sub(string.gsub(fraction, "%.", ""), -16) + 0 -- remove the decimal and take up to 16 digits
		entropy = (entropy % 2^52) + (fraction % 2^52) -- 2^53 is the largest integer that can be decreased by 1. also avoid producing infinity as the result

		local INT_MAX = 2147483647
		while entropy > INT_MAX do entropy = entropy - INT_MAX + 1 end
		while entropy < 0 do entropy = entropy + INT_MAX - 1 end
		
		print("rng seed: " .. entropy)
		original_randomseed(entropy)
	end
end

function makeNondeterministic()
	--[[ "Sandboxify" code that runs before us:
		luanet = nil
		package = nil
		require = nil
		dofile = nil
		load = nil
		loadfile = nil
		os = nil
		io = nil
	]]
	
	local entropy = math.random(9000000) -- keep the song-based entropy AS2 is giving us
	local function addEntropyRaw(seed)
		entropy = entropy + seed + math.random() * seed -- throwing in "math.random() * seed" ensures that the order that numbers are added to the entropy is also a part of the entropy
	end
	local function addEntropy(seed)
		if seed == nil then
			seed = math.random(9000000)
		elseif type(seed) == "number" then
			addEntropyRaw(seed)
		else
			if type(seed) ~= "string" then seed = tostring(seed) end -- for functions and tables this will use their raw memory address for entropy
			if seed == "" then seed = "12" end
			seed = {string.byte(seed, 1, #seed)}
			for i=1,#seed do addEntropyRaw(seed[i]) end
		end
	end
	
	-- nope
	if type(os) == "table" and type(os.time) == "function" then
		addEntropy(os.time())
	end
	
	addEntropy(GetMillisecondsSinceStartup())
	--addEntropy(collectgarbage("count") * 1024)
	addEntropy(options)
	addEntropy({})
	addEntropy(addEntropy)
	addEntropy(BatchRenderEveryFrame)

	math.randomseed(entropy)
end

-- feed it the table returned from GetTrack()
function makeDeterministic(track)
	math.randomseed(track[#track].seconds + #track)
end

vec2 = {}
do
	function vec2.add(a, b)       return {a[1] + b[1], a[2] + b[2]} end
	function vec2.sub(a, b)       return {a[1] - b[1], a[2] - b[2]} end
	function vec2.mul(a, b)       return {a[1] * b[1], a[2] * b[2]} end
	function vec2.div(a, b)       return {a[1] / b[1], a[2] / b[2]} end
	function vec2.mulSingle(a, b) return {a[1] * b, a[2] * b} end
	function vec2.divSingle(a, b) return {a[1] / b, a[2] / b} end
	function vec2.assign(a, dest) dest[1], dest[2] = a[1], a[2] end
	function vec2.copy(a)         return { a[1], a[2] } end
	function vec2.fromApi(a)      return { a.x, a.y } end

	local min, max = math.min, math.max
	function vec2.min(a, b)    return { min(a[1], b[1]), min(a[2], b[2]) } end
	function vec2.max(a, b)    return { max(a[1], b[1]), max(a[2], b[2]) } end

	-- returns 2 arrays of numbers: x and y
	function vec2.transposeArray(vec2_array)
		local x, y = {}, {}
		local v
		for i=1,#vec2_array do
			v = vec2_array[i]
			x[i], y[i] = v[1], v[2]
		end
		return x, y
	end

	function vec2.avg(a, b)         return {(a[1] + b[1]) / 2, (a[2] + b[2]) / 2} end

	local sqrt = math.sqrt
	function vec2.len(a)            return sqrt(a[1]^2 + a[2]^2) end
	function vec2.lenSquared(a)     return a[1]^2 + a[2]^2 end
	function vec2.dist(a, b)        return sqrt((a[1] - b[1])^2 + (a[2] - b[2])^2) end
	function vec2.distSquared(a, b) return (a[1] - b[1])^2 + (a[2] - b[2])^2 end

	local divSingle, len = vec2.divSingle, vec2.len
	function vec2.norm(a)           return divSingle(a, len(a)) end
	function vec2.setLen(a, b)      return divSingle(a, len(a) / b) end

	function vec2.right(a)          return {a[2], -a[1]} end
	function vec2.left(a)           return {-a[2], a[1]} end
	function vec2.neg(a)            return {-a[1], -a[2]} end

	local sin, cos = math.sin, math.cos
	function vec2.getRotMatrix(angle, dest)
		local s, c = sin(angle), cos(angle)
		dest[1], dest[2],
		dest[3], dest[4]
		=
		c, -s,
		s,  c
	end
	function vec2.mulMatrix(a, m)
		return {
			a[1]*m[1] + a[2]*m[2],
			a[1]*m[3] + a[2]*m[4],
		}
	end
	local getRotMatrix, mulMatrix = vec2.getRotMatrix, vec2.mulMatrix
	function vec2.rot(a, angle)
		local m = {}
		getRotMatrix(angle, m)
		return mulMatrix(a, m)
	end
end

vec3 = {}
do
	function vec3.add(a, b)       return {a[1] + b[1], a[2] + b[2], a[3] + b[3]} end
	function vec3.sub(a, b)       return {a[1] - b[1], a[2] - b[2], a[3] - b[3]} end
	function vec3.mul(a, b)       return {a[1] * b[1], a[2] * b[2], a[3] * b[3]} end
	function vec3.div(a, b)       return {a[1] / b[1], a[2] / b[2], a[3] / b[3]} end
	function vec3.mulSingle(a, b) return {a[1] * b, a[2] * b, a[3] * b} end
	function vec3.divSingle(a, b) return {a[1] / b, a[2] / b, a[3] / b} end
	function vec3.assign(a, dest) dest[1], dest[2], dest[3] = a[1], a[2], a[3] end
	function vec3.copy(a)         return { a[1], a[2], a[3] } end
	function vec3.fromApi(a)      return { a.x, a.y, a.z } end

	local min, max = math.min, math.max
	function vec3.min(a, b)    return { min(a[1], b[1]), min(a[2], b[2]), min(a[3], b[3]) } end
	function vec3.max(a, b)    return { max(a[1], b[1]), max(a[2], b[2]), max(a[3], b[3]) } end

	-- returns 3 arrays of numbers: x, y, and z
	function vec3.transposeArray(vec3_array)
		local x, y, z = {}, {}, {}
		local v
		for i=1,#vec3_array do
			v = vec3_array[i]
			x[i], y[i], z[i] = v[1], v[2], v[3]
		end
		return x, y, z
	end

	function vec3.avg(a, b)         return {(a[1] + b[1]) / 2, (a[2] + b[2]) / 2, (a[3] + b[3]) / 2} end

	local sqrt = math.sqrt
	function vec3.len(a)            return sqrt(a[1]^2 + a[2]^2 + a[3]^2) end
	function vec3.lenSquared(a)     return a[1]^2 + a[2]^2 + a[3]^2 end
	function vec3.dist(a, b)        return sqrt((a[1] - b[1])^2 + (a[2] - b[2])^2 + (a[3] - b[3])^2) end
	function vec3.distSquared(a, b) return (a[1] - b[1])^2 + (a[2] - b[2])^2 + (a[3] - b[3])^2 end

	local divSingle, len = vec3.divSingle, vec3.len
	function vec3.norm(a)           return divSingle(a, len(a)) end
	function vec3.setLen(a, b)      return divSingle(a, len(a) / b) end

	function vec3.neg(a)            return {-a[1], -a[2], -a[3]} end
end

-- todo: merge this into vec3
do
	local sin, cos, tan = math.sin, math.cos, math.tan
	
	function makeXRotationMatrix(angle, dest)
		local s, c = sin(angle), cos(angle)
		dest[1], dest[2], dest[3],
		dest[4], dest[5], dest[6],
		dest[7], dest[8], dest[9]
		=
		1, 0,  0,
		0, c, -s,
		0, s,  c
		return dest
	end
	function makeYRotationMatrix(angle, dest)
		local s, c = sin(angle), cos(angle)
		dest[1], dest[2], dest[3],
		dest[4], dest[5], dest[6],
		dest[7], dest[8], dest[9]
		=
		 c, 0, s,
		 0, 1, 0,
		-s, 0, c
		return dest
	end
	function makeZRotationMatrix(angle, dest)
		local s, c = sin(angle), cos(angle)
		dest[1], dest[2], dest[3],
		dest[4], dest[5], dest[6],
		dest[7], dest[8], dest[9]
		=
		c, -s, 0,
		s,  c, 0,
		0,  0, 1
		return dest
	end
	function makeRotationMatrix(x, y, z, dest)
		-- adapted from http://www.flipcode.com/documents/matrfaq.html#Q36
		local sx, cx, sy, cy, sz, cz = sin(x), cos(x), sin(y), cos(y), sin(z), cos(z)
		local cx_sy, sx_sy = cx*sy, sx*sy
		
		dest[1], dest[2], dest[3],
		dest[4], dest[5], dest[6],
		dest[7], dest[8], dest[9]
		=
		cy*cz,             -cy*sz,            -sy,
		-sx_sy*cz + cx*sz, sx_sy*sz + cx*cz,  -sx*cy,
		cx_sy*cz + sx*sz,  -cx_sy*sz + sx*cz, cx*cy
		return dest
	end
	
	function matrixMult(v, m, dest)
		dest[1], dest[2], dest[3] =
		v[1]*m[1] + v[2]*m[2] + v[3]*m[3],
		v[1]*m[4] + v[2]*m[5] + v[3]*m[6],
		v[1]*m[7] + v[2]*m[8] + v[3]*m[9]
		return dest
	end
	function matrixMultYZ(v, m, dest)
		dest[2], dest[3] =
		v[1]*m[4] + v[2]*m[5] + v[3]*m[6],
		v[1]*m[7] + v[2]*m[8] + v[3]*m[9]
		return dest
	end
	function matrixMultXZ(v, m, dest)
		dest[1], dest[3] =
		v[1]*m[1] + v[2]*m[2] + v[3]*m[3],
		v[1]*m[7] + v[2]*m[8] + v[3]*m[9]
		return dest
	end
	function matrixMultXY(v, m, dest)
		dest[1], dest[2] =
		v[1]*m[1] + v[2]*m[2] + v[3]*m[3],
		v[1]*m[4] + v[2]*m[5] + v[3]*m[6]
		return dest
	end
end
do
	local insert = table.insert
	function addQuadIndices(t, index0, index1, index2, index3)
		insert(t, index0)
		insert(t, index1)
		insert(t, index2)
		insert(t, index1)
		insert(t, index3)
		insert(t, index2)
	end
	local addQuadIndices = addQuadIndices

	-- These Build* functions return a table that can be passed to BuildMesh. Based on the example in Neon.

	-- mesh = BuildMesh(BuildRect{ min = {0,0,0}, max = {1,1,1}, calculatenormals = false }) -- a 1x1x1 cube with its origin in a bottom corner
	function buildRect(t)
		local L, H = t.min, t.max
		local verts = {
			{L[1], H[2], H[3]},
			{H[1], H[2], H[3]},
			{L[1], H[2], L[3]},
			{H[1], H[2], L[3]},
			{L[1], L[2], H[3]},
			{H[1], L[2], H[3]},
			{L[1], L[2], L[3]},
			{H[1], L[2], L[3]}
		}
		local uvs = {
			{0,1},
			{1,1},
			{0,1},
			{1,1},
			{0,0},
			{1,0},
			{0,0},
			{1,0}
		}
		local indices = {}
		addQuadIndices(indices, 0, 1, 2, 3)
		addQuadIndices(indices, 1, 0, 5, 4)
		addQuadIndices(indices, 3, 1, 7, 5)
		addQuadIndices(indices, 2, 3, 6, 7)
		addQuadIndices(indices, 0, 2, 4, 6)
		addQuadIndices(indices, 5, 4, 7, 6)

		return {
			vertextable=verts,
			indextable=indices,
			uvtable = uvs,
			calculatenormals = t.calculatenormals
		}
	end
	-- mesh = BuildMesh(BuildCube(0.5, false)) -- a 1x1x1 cube with its origin in the center
	function buildCube(half_width, calculatenormals)
		local L, H = -half_width, half_width
		return buildRect{
			min = {L,L,L},
			max = {H,H,H},
			calculatenormals = calculatenormals
		}
	end

	-- BuildXQuad: A quad with its width along the X axis. Top, Bottom, Left, Right, Z offset (Z is zero if omitted)
	-- mesh = BuildMesh(BuildXQuad(0.5, -0.5, -0.5, 0.5, 0)) -- a 1x1 quad with its origin in the middle
	function buildXQuad(T, B, L, R, Z)
		if Z == nil then Z = 0 end
		return buildQuad{
			{L, T, Z},
			{R, T, Z},
			{L, B, Z},
			{R, B, Z}
		}
	end
	function buildZQuad(T, B, L, R, X)
		if X == nil then X = 0 end
		return buildQuad{
			{X, T, L},
			{X, T, R},
			{X, B, L},
			{X, B, R}
		}
	end
	function buildYQuad(T, B, L, R, Y)
		if Y == nil then Y = 0 end
		return buildQuad{
			{L, Y, T},
			{R, Y, T},
			{L, Y, B},
			{R, Y, B}
		}
	end
	function buildQuad(verts)
		local uvs = {
			{0,1},
			{1,1},
			{0,0},
			{1,0}
		}
		local indices = {}
		addQuadIndices(indices, 0, 1, 2, 3)

		return {
			vertextable=verts,
			indextable=indices,
			uvtable = uvs
		}
	end
	function buildMerged(main, add)
		local offset = #main.vertextable
		for tbl_name,tbl in pairs(add) do
			if tbl_name == "indextable" then
				for n=1,#tbl do insert(main[tbl_name], tbl[n] + offset) end
			else
				for n=1,#tbl do insert(main[tbl_name], tbl[n]) end
			end
		end
	end
end


function makeBarGraph(settings)
	local count = settings.count
	local BuildMesh = BuildMesh
	
	-- background
	local panel_width = settings.size[1] + settings.border * 2
	local panel_height = settings.size[2] + settings.border * 2
	local panel_border = settings.border
	
	-- everything
	if not settings.offset then settings.offset = {0,0} end
	local offset_x = settings.offset[1]
	local offset_y = settings.offset[2]
	
	-- bars
	local width = settings.size[1] / count * settings.bar_scale
	local width_spaced = (settings.size[1] - width*count) / (count - 1) + width -- the width of a bar plus the width of the space between it and the previous bar
	local height = settings.size[2]
	
	local left = offset_x + panel_border
	local bottom = offset_y + panel_border
	local top = bottom + (settings.default_height or 0) * height
	
	-- "solid", "shared", "vertical", "full", "slide"
	local uv_mode = settings.uv_mode or "solid"
	
	uv_antistretch = settings.uv_antistretch --or nil
	
	local depth, panel_depth = settings.bars_depth or 0, settings.background_depth or 0
	
	local table = table -- speeds up table.insert
	
	-- build UVs
	local uvs = {}
	local updateUvs = nil -- will be set to a function to update the uvs if uv_antistretch or "slide" is used
	if uv_mode == "solid" then
		local uv = {0,0}
		local uvcount = count * 4
		for i=1,uvcount do table.insert(uvs, uv) end
		--if uv_antistretch then uv_antistretch = nil end
		
	elseif uv_mode == "shared" then
		local base_uvs = {
			{0,1},
			{1,1},
			{0,0},
			{1,0}
		}
		if not uv_antistretch then
			for i=1,count do
				for u=1,4 do table.insert(uvs, base_uvs[u]) end
			end
		else
			if uv_antistretch == "bottom" then
				for i=1,count do
					table.insert(uvs, {0, 1}) -- new table so they can be set to independent values later
					table.insert(uvs, {1, 1})
					table.insert(uvs, base_uvs[3])
					table.insert(uvs, base_uvs[4])
				end
				updateUvs = function(heights)
					local u = 1
					for i=1,count do
						uvs[u][2] = heights[i]
						u = u + 1
						uvs[u][2] = heights[i]
						u = u + 3
					end
				end
			else -- top
				for i=1,count do
					table.insert(uvs, base_uvs[1])
					table.insert(uvs, base_uvs[2])
					table.insert(uvs, {0, 0})
					table.insert(uvs, {1, 0})
				end
				updateUvs = function(heights)
					local u = 3
					for i=1,count do
						uvs[u][2] = 1 - heights[i]
						u = u + 1
						uvs[u][2] = 1 - heights[i]
						u = u + 3
					end
				end
			end
		end
		
	elseif uv_mode == "vertical" then
		for i=1,count do
			local x = (i-1) / (count-1)
			local t, b = {x, 1}, {x, 0}
			table.insert(uvs, t)
			table.insert(uvs, t)
			table.insert(uvs, b)
			table.insert(uvs, b)
		end
		if uv_antistretch then
			if uv_antistretch == "bottom" then
				updateUvs = function(heights)
					local u = 1
					for i=1,count do
						uvs[u][2] = heights[i]
						u = u + 4
					end
				end
			else -- top
				updateUvs = function(heights)
					local u = 3
					for i=1,count do
						uvs[u][2] = 1 - heights[i]
						u = u + 4
					end
				end
			end
		end
		
	elseif uv_mode == "full" then
		for i=1,count do
			local L = (i-1) / count
			local R = i / count
			table.insert(uvs, {L, 1})
			table.insert(uvs, {R, 1})
			table.insert(uvs, {L, 0})
			table.insert(uvs, {R, 0})
		end
		if uv_antistretch then
			if uv_antistretch == "bottom" then
				updateUvs = function(heights)
					local u = 1
					for i=1,count do
						uvs[u][2] = heights[i]
						u = u + 1
						uvs[u][2] = heights[i]
						u = u + 3
					end
				end
			else
				updateUvs = function(heights)
					local u = 3
					for i=1,count do
						uvs[u][2] = 1 - heights[i]
						u = u + 1
						uvs[u][2] = 1 - heights[i]
						u = u + 3
					end
				end
			end
		end
		
	elseif uv_mode == "slide" then
		for i=1,count do
			local t, b = {0, 1}, {0, 0}
			table.insert(uvs, t)
			table.insert(uvs, t)
			table.insert(uvs, b)
			table.insert(uvs, b)
		end
		if not uv_antistretch then
			updateUvs = function(heights)
				local u = 1
				for i=1,count do
					uvs[u][1] = heights[i]
					u = u + 2
					uvs[u][1] = heights[i]
					u = u + 2
				end
			end
		elseif uv_antistretch == "bottom" then
			updateUvs = function(heights)
				local u = 1
				for i=1,count do
					uvs[u][1] = heights[i]
					uvs[u][2] = heights[i]
					u = u + 2
					uvs[u][1] = heights[i]
					u = u + 2
				end
			end
		else -- top
			updateUvs = function(heights)
				local u = 1
				for i=1,count do
					uvs[u][1] = heights[i]
					u = u + 2
					uvs[u][1] = heights[i]
					uvs[u][2] = 1 - heights[i]
					u = u + 2
				end
			end
		end
	end
	
	-- build the mesh
	local mesh
	local verts = {}
	do
		-- build verts and triangles
		local indices = {}
		local index = 1
		for i=1,count do
			local x = width_spaced * (i - 1) + left
			local tl, tr, bl, br = {x, top, depth}, {x + width, top, depth}, {x, bottom, depth}, {x + width, bottom, depth}
			table.insert(verts, tl)
			table.insert(verts, tr)
			table.insert(verts, bl)
			table.insert(verts, br)
			addQuadIndices(indices, index, index+1, index+2, index+3)
			index = index + 4
		end
		
		local orig_mesh = BuildMesh{
			vertextable=verts,
			indextable=indices,
			uvtable = uvs,
			calculatenormals = false
		}
		mesh = { -- when updating the mesh it's best to only include the tables that actually changed
			mesh = orig_mesh,
			vertextable = verts
		}
	end
	if updateUvs then mesh.uvtable = uvs
	else uvs = nil end
	
	
	local function updateGraph(heights)
		local top
		local v = 1
		for i=1,count do
			top = heights[i] * height + bottom
			verts[v][2] = top
			v = v + 1
			verts[v][2] = top
			v = v + 3
		end
		if updateUvs then updateUvs(heights) end
		BuildMesh(mesh)
	end
	
	local background = BuildMesh(buildXQuad(
		offset_y + panel_height, -- top
		offset_y,                -- bottom
		offset_x,                -- left
		offset_x + panel_width,  -- right
		panel_depth
	))
	
	return mesh.mesh, background, updateGraph
end
--[[
	The Audiosurf 2 function "GetSpectrum()" returns a Lua array of 16 decimal numbers.
	Here is what happens when you call that function:
	1. The "Kiss FFT" library is used to do a FFT of the current point in the song. I can't tell what it does here because it's compiled C++.
	2. The library returns an array of 512 float numbers.
	   They seem to be frequency bins containing the magnitude squared. (a^2 + b^2)
	   The magnitude seems to be in the range 0 to 2.
	3. The bins are broken up into groups of 32 and the average of each group is calculated, resulting in 16 bins.
	
	To get the magnitude of a bin: magnitude = math.sqrt(x)
	To convert the above magnitude to decibels: db = 20 * math.log10(magnitude / 2)
	If you only want decibels, you can simplify it: db = 10 * math.log10(x / 4)
	
	Getting the above values in a useful 0 to 1 range:
		norm = magnitude / 2
		norm = math.min(0, (db + 60) / 60) -- cut off values less than -60db
	
	Best reference I've seen online: http://stackoverflow.com/questions/4675457/how-to-generate-the-audio-spectrum-using-fft-in-c
]]
do
	local sqrt, log, log10 = math.sqrt, math.log, math.log10
	
	---- "Normalize" functions, to get the spectrum into a 0 to 1 range ----
	
	-- clamp: just don't let anything go out of range
	function spectrumNormClamp(spectrum)
		for i=1,#spectrum do
			if     spectrum[i] < 0 then spectrum[i] = 0
			elseif spectrum[i] > 1 then spectrum[i] = 1 end
		end
	end
	-- scale clamp: scale and translate the numbers before clamping them
	function spectrumNormScaleClamp(spectrum, mult, add)
		for i=1,#spectrum do
			local bin = spectrum[i] * mult + add
			if     bin < 0 then spectrum[i] = 0
			elseif bin > 1 then spectrum[i] = 1
			else                spectrum[i] = bin end
		end
	end
	-- remap: change values from a known range to be in 0 to 1 range
	function spectrumNormRemap(spectrum, in_low, in_high)
		local range = in_high - in_low
		for i=1,#spectrum do
			spectrum[i] = (spectrum[i] - in_low) / range
		end
	end
	-- dynamic: find the highest/lowest values of the current frame and remap based on that range
	-- optional argument "min_range" can keep the range from getting too small
	-- optional argument "ceiling" can be used to keep the min_range window from going higher than a certain level
	function spectrumNormDynamic(spectrum, min_range, ceiling)
		if min_range == nil then min_range = 0 end
		local min, max = spectrum[1], spectrum[1]
		for i=2,#spectrum do
			if     spectrum[i] < min then min = spectrum[i]
			elseif spectrum[i] > max then max = spectrum[i] end
		end
		local range = max - min
		if range == 0 then -- special case: there's absolutely no variation. probably because it's silent.
			for i=1,#spectrum do spectrum[i] = 0 end
			return
		end
		if range < min_range then
			-- expand the range upward (creates empty space above a graph)
			max = max + min_range - range
			range = min_range
			if ceiling and max > ceiling then
				-- expanding upward hit the ceiling. go as high as possible and expand downward for the rest
				min, max = min - (max-ceiling), ceiling
			end
		end
		for i=1,#spectrum do spectrum[i] = (spectrum[i] - min) / range end
	end
	
	-- returns a function
	-- the point of update_interval is to keep the smoothing behavior time-based rather than frame-based
	-- it actually samples every frame, but those samples will not be used until the next update
	-- during an update, the queued per-frame samples are averaged together and that becomes the actual per-update sample which will be used
	-- this means that a change in FPS WON'T change the time window that is used for smoothing but it WILL affect the accuracy
	
	-- a strong spike in frame rate could cause a particular part of the song to be over-represented in the queue of per-frame samples
	-- any such inaccuracy will be confined to its corresponding per-update sample
	-- it wouldn't be any worse than the kind of inaccuracy that happens with consistently low frame rates
	function spectrumNormSmooth_Generate(min, min_range, max_range, sample_count, update_interval, initial_range)
	
	
	
	
	
	
	
	
	
		error("fixme")
		
		
		
		
		
		
		
		
		
		if sample_count < 1
		or update_interval <= 0
		or min_range > max_range
		or min_range < 0
		then error("invalid arguments supplied to spectrumNormSmooth_Generate")
		end
		
		local samples = {}
		if not initial_range then initial_range = max_range end
		for i=1,sample_count do table.insert(samples, initial_range) end
		local range = initial_range
		local last_time
		local next_update = 0
		local sample_queue = {}
		local GetSongCompletionPercentage = GetSongCompletionPercentage
		return function(spectrum)
			local time = GetSongCompletionPercentage()
			local time_until = next_update - time
			
			-- store this frame's intensity
			if time ~= last_time then -- allowing this to run during a pause screen would be dangerous and incorrect
				local max = spectrum[1]
				local bin
				for i=2,#spectrum do
					bin = spectrum[i]
					if bin > max then max = bin end
				end
				table.insert(sample_queue, max)
			end
			
			-- update averages
			if time_until <= 0 then
				-- the below complications are to deal with the possibility of a frame lasting slightly or significantly longer than update_interval
				-- in the event that we failed to collect enough data to satisfy the demanded rate, the last collected data will be repeated
				-- (because AS2 doesn't allow us to get the spectrum of any time other than the present)
				-- DON'T demand a ridiculously small update_interval. the performance hit of this function increases asymptotically as update_interval approaches zero. (zero causes an infinite loop)
				
				-- average the stored intensities
				local new_sample
				if #sample_queue == 0 then
					-- no data :(
					new_sample = samples[sample_count]
				else
					new_sample = sample_queue[1]
					for i=2,#sample_queue do new_sample = new_sample + sample_queue[i] end
					new_sample = new_sample / #sample_queue
					sample_queue = {}
				end
				
				repeat
					table.remove(samples, 1)
					table.insert(samples, new_sample)
					time_until = time_until + update_interval
				until time_until > 0
				next_update = time + update_interval - time_until
				
				range = 0
				for i=1,sample_count do range = range + samples[i] end
				range = range / sample_count
			end
			
			for i=1,#spectrum do
				spectrum[i] = (spectrum[i] - min) / range + min
			end
			
			return range
		end
	end
	
	function spectrumNormSmoothPeaks_Generate(min, min_range, max_range, fall_rate)
		if min_range < 0 or fall_rate < 0
		then error("invalid arguments supplied to spectrumNormSmoothPeaks_Generate")
		end
		
		local range = min_range
		local min_height = min + min_range
		return function(spectrum, delta_time)
			
			if delta_time > 0 then
				range = range - delta_time*fall_rate
				
				local max = min_height
				local bin
				for i=1,#spectrum do
					bin = spectrum[i]
					if bin > max then max = bin end
				end
				max = max - min
				
				if max > range then range = max end
				if range > max_range then range = max_range end
			end
			
			for i=1,#spectrum do
				spectrum[i] = (spectrum[i] - min) / range + min
			end
			
			return range
		end
	end
	
	
	---- Functions to convert the raw spectrum to a desired scale ----
	
	-- basic linear spectrum. (0 to 1) expect to see occasional spikes in a collection of otherwise low values
	function spectrumLinear(spectrum)
		for i=1,#spectrum do
			spectrum[i] = sqrt(spectrum[i]) / 2
		end
	end
	-- decibels, in the standard -100 to 100 scale. the values should never go above 0
	-- decibels are a logarithmic scale
	function spectrumDecibel(spectrum)
		for i=1,#spectrum do
			spectrum[i] = 10 * log10(spectrum[i] / 4)
		end
	end
	
	
	---- example/helper functions ----
	
	local GetSpectrum = GetSpectrum
	
	do
		-- store a reference to the function in a closure. this makes it faster
		local spectrumLinear = spectrumLinear
		local spectrumNormClamp = spectrumNormClamp
		
		-- basic linear
		function spectLinear()
			local spectrum = GetSpectrum()
			spectrumLinear(spectrum)
			spectrumNormClamp(spectrum) -- probably unnecessary
			return spectrum
		end
	end
	
	do
		local spectrumLinear = spectrumLinear
		local spectrumNormDynamic = spectrumNormDynamic
		
		function spectLinearDynamic()
			local spectrum = GetSpectrum()
			spectrumLinear(spectrum)
			spectrumNormDynamic(spectrum, 0.25, 1)
			return spectrum
		end
	end
	
	do
		local spectrumDecibel = spectrumDecibel
		local spectrumNormScaleClamp = spectrumNormScaleClamp
		
		-- decibels from -55 to -5 (normalized, of course)
		function spectDecibel()
			local spectrum = GetSpectrum()
			spectrumDecibel(spectrum)
			spectrumNormScaleClamp(spectrum, 0.02, 1.1)
			return spectrum
		end
	end
	
	do
		local spectrumDecibel = spectrumDecibel
		local spectrumNormDynamic = spectrumNormDynamic
		
		-- in making this one I tried to imitate the spectrum meter from foobar2000's ColumnsUI plugin
		function spectLogarithmic()
			local spectrum = GetSpectrum()
			spectrumDecibel(spectrum)
			spectrumNormDynamic(spectrum, 25, 0)
			return spectrum
		end
	end
end

if false then -- example spectrum visualizer:

	local graph_mesh, background_mesh, updateGraph = makeBarGraph{
		count = 16, -- number of bars (GetSpectrum returns 16 numbers)
		size = {2, 1}, -- width height
		border = 0.05, -- the background will be this much bigger than "size"
		offset = {1, 0}, -- by default the "center" of the mesh is the bottom left corner. this setting moves the mesh away from that point.
		bar_scale = 0.9, -- scales the width of the individual bars. at a scale of 1 there are no spaces between the bars
		default_height = 0, -- the height that all the bars are set to before the first time you call graphUpdateFunction. defaults to 0.
		
		uv_mode = "slide", -- defaults to "solid"
		-- solid makes every point of the bars colored according to the top left of the texture. use this if you're making them all the same solid color
		-- shared makes each bar span the entire image
		-- full divides the image up evenly among all the bars. if all the bars are at 100% then you will see the original image
		-- vertical is like full, but the bars have zero UV width. (there will be gaps of unused texture between them) if you use this with a horizontal gradient texture the bars will each have a different solid color
		-- slide is like vertical but all bars start at the left and get shifted toward the right as their height increases (uses more fps like uv_antistretch does)
		
		uv_antistretch = "top", -- defaults to nil. choices: nil, "bottom", "top"
		-- if not nil, prevents the texture from being stretched vertically by adjusting the UVs to match the bar size. top/bottom specifies which edge will be adjusted. costs more fps.
		-- if "bottom" is used with "shared" and a vertical gradient texture then the bars would reveal different colors at the bottom when they get taller
		-- obviously this is incompatible with "solid"
		
		bars_depth = 0, -- defaults to 0
		background_depth = 0.01, -- defaults to 0. to avoid z-fighting (flickering), you should adjust this to a larger value if you put the object further away from the camera
	}
	
	--updateGraph is a function that takes an array of numbers between 0 and 1
	local spectLogarithmic = spectLogarithmic
	-- call this function from your Update() function or rename it to Update if you don't have one
	function UpdateSpectrum(deltatime)
		if deltatime <= 0 then return end -- don't update when the game is paused
		local spectrum = spectLogarithmic()
		updateGraph(spectrum)
		-- structured this way so it's easy to do more than one thing with the
		-- spectrum here or call UpdateSpectrumVisualizer from a different place
	end
	
	-- position of bars and background
	local transform = {
		pos = {4, 1, 0},
		rot = {0, 0, 0}, -- flat against the track: {90,-90,-90}
		scale = {1,1,1} -- generally you should be adjusting the width and height up at the top rather than changing this, but you might use this if you're using it in more than one place
	}
	-- bars
	CreateObject{
		railoffset = 0, -- attach it to the center of the track where the player is. a higher offset will put it on track further ahead of the player
		floatonwaterwaves = false,
		gameobject={
			transform = transform,
			mesh = graph_mesh,
			
			--texture = "graph/bars_vertical.png", -- defaults to white
			--texture = "graph/bars_vertical_top.png",
			--texture = "graph/bars_horizontal.png",
			texture = "graph/bars_horizontal_flip.png",
			--texture = "graph/bars_both.png",
			--texture = "graph/bars_both_flip.png",
			--texturefilter = "point" -- choices: bilinear, trilinear, point. bilinear is the default. point filtering makes the texture look blocky rather than blurry when stretched
			texturewrap = "clamp", -- prevents artifacts at the edge of textures not designed to be tiled
			shader = "VertexColorUnlitTinted", -- use VertexColorUnlitTintedAlpha if you add transparency
			shadercolors={
				_Color={255,255,255,0}
			}
		}
	}
	-- background
	CreateObject{
		railoffset = 0,
		floatonwaterwaves = false,
		gameobject={
			transform = transform,
			mesh = background_mesh,
			
			--texture = "example.png",
			texturewrap = "clamp",
			shader = "VertexColorUnlitTinted",
			shadercolors = {
				_Color={0,0,0}
			}
		}
	}
	
	-- you could reuse the mesh for multiple objects and/or use use the objects in BatchRenderEveryFrame.
end

function getTrackBounds(track)
	local vec3 = vec3
	local track_vec3s = { vec3.fromApi(track[1].pos) }

	local minx,miny,minz = unpack(track_vec3s[1])
	local maxx,maxy,maxz = minx,miny,minz
	
	local pos,x,y,z
	for i=2,#track do
		pos = track[i].pos
		x,y,z = pos.x, pos.y, pos.z
		track_vec3s[i] = {x,y,z}
		
		if minx > x then minx = x elseif maxx < x then maxx = x end
		if miny > y then miny = y elseif maxy < y then maxy = y end
		if minz > z then minz = z elseif maxz < z then maxz = z end
	end
	
	local track_bounds = {
		min = {minx,miny,minz}, -- this is the bounds that AS2 uses internally
		max = {maxx,maxy,maxz},
	}
	track_bounds.center = vec3.avg(track_bounds.min, track_bounds.max)
	
	return track_bounds, track_vec3s
end

function addNodeData(track_bounds, track_vec3s, side, up, down)
	local vec3 = vec3
	local node_bounds = {
		-- this assumes nodes never tilt or are mostly square
		side = side,
		up = up,
		down = down,
		min = {-side, -down, -side},
		max = { side,  up,    side},
	}
	track_bounds.node_min = vec3.add(track_bounds.min, node_bounds.min)
	track_bounds.node_max = vec3.add(track_bounds.max, node_bounds.max)
	track_bounds.node_center = vec3.avg(track_bounds.node_min, track_bounds.node_max)
	return node_bounds
end

local function getHorizontalRadius(track_vec3s, center)
	local cx,cz = center[1], center[3]
	local pos,dist
	local max = 0
	for i=1,#track do
		pos = track_vec3s[i]
		dist = (pos[1] - cx)^2 + (pos[3] - cz)^2
		if max < dist then max = dist end
	end
	return math.sqrt(max)
end


































---------------------------------------------------------------------------------------------------------------------------
-- Init
---------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------
-- Read track data (1 of 2)
---------------------------------------------------------------------------------------------------------------------------

--[[
	apply a modified version of the track colors where the alpha ranges from 0 to 1
	this lets us know the track's base slope by reading from the alpha values in GetTrack
	later on before creating the rails, track_colors will be applied again without alpha modifications
	so this only affects the table created here by GetTrack
]]
do
	local orig_colors = options.track_colors
	local colors, count = {}, #orig_colors
	if count == 1 then
		orig_colors = {orig_colors[1], orig_colors[1]}
		count = 2
	end
	for i=1,count do
		local color = orig_colors[i]
		if color.r ~= nil then
			color = {color.r, color.g, color.b, color.a}
		else
			color = shallowCopy(color)
		end
		if math.max(unpack(color)) > 1 then
			for j=1,3 do color[j] = color[j] / 255 end
		end
		color[4] = (i-1) / (count-1)

		colors[i] = color
	end

	SetTrackColors(colors)

	-- while we're on the subject,
	options.color_downhill = orig_colors[count]
	options.color_uphill = orig_colors[1]
end

track = GetTrack()
track_bounds, track_vec3s = getTrackBounds(track)
print("Track node count: " .. #track)

---------------------------------------------------------------------------------------------------------------------------
-- Update function
---------------------------------------------------------------------------------------------------------------------------

local onUpdate
do
	local addListener, fire = createEvent()
	function onUpdate(listener)
		Update = fire -- we only make the Update function when we know it's needed

		addListener(listener)
		onUpdate = addListener -- this function only needs to run for the first listener, so pass the rest directly
	end
end

if UpdateSpectrum then onUpdate(UpdateSpectrum) end

local onUpdateSpectrum
if options.spectrum_fps ~= nil then
	local GetSpectrum, fire = GetSpectrum
	local min_delta = 1/options.spectrum_fps
	local delta_sum, frame_count, spectrum_avg = 0, 0, {}
	for i=1,16 do spectrum_avg[i] = 0 end

	local function updateSpectrum(delta_time, ...)
		if delta_time == 0 then return end
		local spectrum = GetSpectrum()

		for i=1,16 do spectrum_avg[i] = spectrum_avg[i] + spectrum[i] end
		frame_count, delta_sum = frame_count + 1, delta_sum + delta_time
		if delta_sum < min_delta then return end

		for i=1,16 do
			spectrum[i] = spectrum_avg[i] / frame_count
			spectrum_avg[i] = 0
		end
		fire(spectrum, delta_sum, ...)
		delta_sum, frame_count = 0, 0
	end
	do
		local addListener
		addListener, fire = createEvent()
		function onUpdateSpectrum(listener)
			onUpdate(updateSpectrum)

			addListener(listener)
			onUpdateSpectrum = addListener
		end
	end
else
	local GetSpectrum, fire = GetSpectrum
	local function updateSpectrum(delta_time, ...)
		if delta_time == 0 then return end
		fire(GetSpectrum(), delta_time, ...)
	end
	do
		local addListener
		addListener, fire = createEvent()
		function onUpdateSpectrum(listener)
			onUpdate(updateSpectrum)

			addListener(listener)
			onUpdateSpectrum = addListener
		end
	end
end

do
	if not options.deterministic then makeNondeterministic() end

	---------------------------------------------------------------------------------------------------------------------------
	-- Options Logic
	---------------------------------------------------------------------------------------------------------------------------

	do
		local options = options -- faster
		local skinvars = GetSkinProperties()
		print("skinvars:\n" .. tableToPrettyString(skinvars))

		options.isjumping = PlayerCanJump()
		options.ispuzzle = skinvars.colorcount > 1
		options.ismono = not (options.isjumping or options.ispuzzle)
		options.quality = math.min(3, math.max(1, GetQualityLevel())) -- GetQualityLevel returns 1,2,or 3. clamp it to this expected range so we can safely use equality comparisons later
		
		if     options.quality == 1 then function ifquality(H,M,L) return L end
		elseif options.quality == 2 then function ifquality(H,M,L) return M end
		else                             function ifquality(H,M,L) return H end end
		
		if     options.quality == 1 then function qualityClamp(H,M,L) return math.min(L,H) end
		elseif options.quality == 2 then function qualityClamp(H,M,L) return math.min(M,H) end
		else                             function qualityClamp(H,M,L) return H end end
		
		--hifi = (quality == 3)
		--function ifhifi(if_true, if_false) if hifi then return if_true else return if_false end end
		if options.quality == 3 then function ifhifi(H,L) return H end
		else                         function ifhifi(H,L) return L end end
		
		---- overrides ----

		do
			local override = options.override
			options.override = nil
			if type(override) == "function" then
				override = override()
				print("options.override:" .. (override or "<nil>"))
			end
			if override then
				override = options.overrides[override]
				if not override then
					error('Unknown name "'..tostring(override)..'" in options.override. The following names are acceptable: '..table.concat(tableKeys(options.overrides), ", "))
				end
				concatTable(options, override)
			end
		end

		---- shuffle ----
		
		if options.shuffle then
			local shuffle = doShuffle()
			print("Shuffle results:\n" .. tableToPrettyString(shuffle))
			concatTable(options, shuffle)
			doShuffle = nil
		end
		
		---- invert ----
		
		if options.invert then function ifinvert(I,N) return I end
		else                   function ifinvert(I,N) return N end end
		
		if options.invert then concatTable(options, options.invert_options) end
		options.invert_options = nil

		---- custom mod properties ----
		
		-- set disableWater to a nonzero number to disable water
		if type(skinvars.disableWater) == "number" and skinvars.disableWater ~= 0 then options.water = false end
		
		-- set forceLaneDividers to a nonzero number to keep the skin from ignoring them if there aren't many of them
		if type(skinvars.forceLaneDividers) == "number" and skinvars.forceLaneDividers ~= 0 then options.track_lanes_minimum = 1 end
		
		-- set trackheight to a number to ask for extra vertical empty space above the track floor
		options.track_height = 1.5 -- approximate top edge of standard blocks
		if type(skinvars.trackheight) == "number" then options.track_height = skinvars.trackheight end
		
		-- in mods that don't do GameplaySettings{automatic_traffic_collisions = false} the grey hitboxes are skinnier than normal blocks
		-- the convention established by the stock skins is to only do automatic_traffic_collisions in wakeboard mods
		-- if your mod breaks this convention, set skinnyGreyBlocks to a number. 0 is false and any other number is true
		if type(skinvars.skinnyGreyBlocks) == "number" then
			options.skinny_greys = fif(skinvars.skinnyGreyBlocks == 0, false, true)
		else
			options.skinny_greys = options.isjumping
		end
		
		---- option translation ----
		
		options.water_transparency_limiter = options.water and options.water_transparency
		
		options.trackless = not (options.water or options.track_surface)

		---- basic layout ----
		
		-- these two numbers reflect currently unchangeable AS2 appearance. (changing water_width won't actually change the width of the water, it will just make the skin misbehave)
		options.water_width = 11.5
		options.water_wake_start = 5
		
		options.shoulder_lines = shallowCopy(skinvars.shoulderlines)
		options.lane_dividers = shallowCopy(skinvars.lanedividers)
		options.track_width = skinvars.trackwidth
		
		options.water_wake = options.water_wake and (options.isjumping or (options.force_water_wake and options.track_width < options.water_wake_start + 0.3)) -- 0.3 to allow a small amount of overlap
		
		if options.track_width < options.water_width
		and (options.water or options.water_wake)
		then options.track_width = options.water_width end
		
		---- calculate the size of lanes ----
		
		if #options.shoulder_lines + #options.lane_dividers == 0 then
			-- single lane
			options.lane_width = skinvars.trackwidth * 2
		elseif #options.shoulder_lines + #options.lane_dividers == 1 then
			-- divide the track in half
			options.lane_width = skinvars.trackwidth
		else
			local lines = {}
			for i=1,#options.shoulder_lines do table.insert(lines, options.shoulder_lines[i]) end
			for i=1,#options.lane_dividers do table.insert(lines, options.lane_dividers[i]) end
			table.sort(lines)
			
			-- average the lane widths together
			options.lane_width = 0
			for i=1,#lines-1 do
				options.lane_width = options.lane_width + lines[i + 1] - lines[i]
			end
			options.lane_width = options.lane_width / (#lines - 1)
		end
		
		---- lane/shoulder lines ----
		
		if options.track_shoulders then
			-- add extra shoulder lines if the ones provided by the mod don't mark the actual track edge and we are making the track wider than requested
			-- this search assumes the mod is sane and doesn't put shoulder lines way past trackwidth
			local find = options.track_width - skinvars.trackwidth > options.lane_width * 0.5
			local find_area = skinvars.trackwidth - options.lane_width * 0.25
			local missing_L = find
			local missing_R = find
			
			if #options.shoulder_lines > 0 then
				-- while we're in this loop we will also delete shoulder lines that are close to the edge of the track
				local has_edge = options.water or options.track_surface
				local edge = options.track_width - options.lane_width * 0.25
				local i = 1
				while i <= #options.shoulder_lines do
					if has_edge and math.abs(options.shoulder_lines[i]) > edge then
						table.remove(options.shoulder_lines, i)
					else
						if find then
							if options.shoulder_lines[i] > find_area then missing_R = false
							elseif options.shoulder_lines[i] < -find_area then missing_L = false end
						end
						i = i + 1
					end
				end
			end
			
			if missing_L then table.insert(options.shoulder_lines, -skinvars.trackwidth) end
			if missing_R then table.insert(options.shoulder_lines, skinvars.trackwidth) end
		else
			options.shoulder_lines = {}
		end
		options.track_shoulders = nil
		
		if not options.track_lanes
		or #options.lane_dividers < options.track_lanes_minimum
		then options.lane_dividers = {} end
		options.track_lanes = nil
		options.track_lanes_minimum = nil
		
		---- option relationships ----
		
		options.track_surface = options.track_surface and not options.water
		
		
		if options.quality ~= 3 or options.invert then
		
			options.glow = false
			options.glow_foreground = false
		elseif options.trackless and options.glow then
			options.glow_foreground = false -- can't change the layer that blocks are on so can't turn off the stock glow for them
		end
		options.glowing_layer = fif(options.glow, 11, 13)
		--options.glowing_suffix = fif(options.glow_foreground, "_StencilGlow.shader", ".shader")
		if options.glow_foreground then
			options.glowing_suffix = "_AlphaGlow.shader"
			options.glowing_common_tint = greyscale(2/3, 1/2)
		else
			options.glowing_suffix = ".shader"
			options.glowing_common_tint = white
		end

		-- haven't bothered to make the shaders for mirroring without alphaglow
		options.mirror = options.quality == 3 and options.glow_foreground
		options.mirrored_suffix = fif(options.mirror, "_Mirror", "")
	end

	print("--Skin Options--\n" .. tableToPrettyString(options))

	foreground = ifinvert(black, white)
	background = ifinvert(white, black)


	---------------------------------------------------------------------------------------------------------------------------
	-- options.extra_shuffle
	---------------------------------------------------------------------------------------------------------------------------
	if options.shuffle_extra then
		-- we want to output these shuffled values to the log just like normal shuffle
		shuffle_extra_record = {}
		function declareSection(section_name)
			current_section = section_name
			if shuffle_extra_record[current_section] == nil then
				shuffle_extra_record[current_section] = {}
			end
		end
		
		function ifShuffle(tbl)
			local result = tbl.func(unpack(tbl.func_args))
			local clamp = true
			if tbl.retry then -- must supply min and max for this
				while result > tbl.max or result < tbl.min do
					result = tbl.func(unpack(tbl.func_args))
				end
				clamp = tbl.int -- the reasoning here is to avoid retries when the result was very close to being in bounds. think of it like we're rounding toward the bounds
			end
			if tbl.int then result = math.floor(result + 0.5) end -- round
			if clamp then 
				if tbl.max then result = math.min(tbl.max, result) end
				if tbl.min then result = math.max(tbl.min, result) end
			end
			shuffle_extra_record[current_section][tbl.name] = result
			return result
		end
	else
		function declareSection()
		end
		function ifShuffle(tbl)
			local result = tbl.standard
			if tbl.max then result = math.min(tbl.max, result) end
			if tbl.min then result = math.max(tbl.min, result) end
			return result
		end
	end
end

---------------------------------------------------------------------------------------------------------------------------
-- Read track data (2 of 2)
---------------------------------------------------------------------------------------------------------------------------

-- approximate the size of the rail geometry
-- TODO: put CreateRail logic earlier in the file so it can declare exact dimensions for us to use here
--       maybe make a hook for the CreateRail function that records bounds automatically
node_bounds = addNodeData(track_bounds, track_vec3s, options.track_width + 85, options.track_height + 35, 40)
track_bounds.node_center_horizontal_radius = getHorizontalRadius(track_vec3s, track_bounds.node_center) + node_bounds.side

print("track bounds:\n" .. tableToPrettyString(track_bounds))


local onScriptComplete, scriptCompleted = createEvent()

---------------------------------------------------------------------------------------------------------------------------
--  SCENE SETUP
---------------------------------------------------------------------------------------------------------------------------
do

	SetTrackColors(options.track_colors)
	options.track_colors = nil

	SetBlockColors(options.block_colors)
	options.block_colors = nil

	declareSection("SetScene")

	local current_scene = {
		dynamicFOV = false, -- if this is true, the "fov" option in SetPlayer will be ignored.
		camStaticZone = 0,
		radialblur_strength = 0,
		usedefaultrails=false,
		fogenabled = false,
		--fogcolor = {255,255,255},
		--fogstart = 0,
		--fogend = 1000,

		--ambientlight = {255,255,255}

		water = options.water,
		usecubewater = options.water,
		watertype = fif(options.water_transparency, 1, 2), -- type 2 has shader GpuOffsetCubesNoLines and all others have GpuOffsetCubes
		watertexture = ifinvert("White.png", "Audiosurf_assets/WaterCubes_DBN-edit_Tintable.png"),
		minwaterrandomdarkness = ifShuffle{ standard=fif(options.invert,0.9,0.5), name="minwaterrandomdarkness", func=gaussRandRange, func_args={0.5, 0.1, 0, 1} },
		numvisiblewaterchunks = -1,
		watertint_highway = true,
		watertint = {1,1,1,0.9}, -- only the alpha is used when watertint_highway is enabled

		--minimap_colormode = "track",
		towropes = options.isjumping,
		--pullermodel = "",

		
		useblackgrid = options.invert,
		

		glowpasses = 0,
		glowspread = 0,
		airdebris_count = 0,
		airdebris_density = 0,

		use_intro_swoop_cam = not options.debug,
	}
	if options.glow then
		current_scene.glowpasses = 4
		current_scene.glowspread = 0.5
	end
	if not options.invert and not options.glow_foreground then
		current_scene.airdebris_count        = ifShuffle{ standard=2000, func=ifhifi(gaussRand,gaussRandLowerHalf), func_args={ifquality(500,200,100), 50}, min=0, max=ifquality(30000,200,100), int=true, retry=true, name="airdebris_count"        }
		current_scene.airdebris_density      = ifShuffle{ standard=15,   func=ifhifi(gaussRand,gaussRandLowerHalf), func_args={ifquality(15,4,2),      1 }, min=1, max=ifquality(17,4,2),        int=true, retry=true, name="airdebris_density"      }
		current_scene.airdebris_fieldsize    = ifShuffle{ standard=300,  func=ifhifi(gaussRand,gaussRandLowerHalf), func_args={300, 20},                    min=0, max=500,                      int=true, retry=true, name="airdebris_fieldsize"    }
		if options.jumping then
		current_scene.airdebris_particlesize = ifShuffle{ standard=0.2,  func=gaussRand, func_args={0.2,  0.1}, name="airdebris_particlesize" }
		else
		current_scene.airdebris_particlesize = ifShuffle{ standard=0.09, func=gaussRand, func_args={0.09, 0.1}, name="airdebris_particlesize" }
		end
	end

	SetScene(current_scene)
	-- variables that need to be provided every time we call SetScene
	current_scene = {
		water                 = current_scene.water,
		numvisiblewaterchunks = current_scene.numvisiblewaterchunks,
		waterclones           = current_scene.waterclones,
		minimap_colormode     = current_scene.minimap_colormode,
		watertype             = current_scene.watertype,
		watertint_highway     = current_scene.watertint_highway,
		fogenable             = current_scene.fogenable,
		ambientlight          = current_scene.ambientlight,
	}

	---------------------------------------------------------------------------------------------------------------------------
	-- Track effects
	---------------------------------------------------------------------------------------------------------------------------

	if options.glow_foreground then
		SetBlockFlashes{
			--[[
				the block flash fragment shader simply outputs the alpha verbatim and doesn't multiply the colors by the alpha
				the blend mode doesn't do anything different for the alpha, it adds it just like the colors
				the only thing it actually does with alpha is AlphaTest
			]]
			texture = "Audiosurf_assets/hit2_AlphaGlow.png",
			sizescaler = 2/3,
		}
	end

	do
		local ring
		if     options.invert          then ring = "_solid"
		elseif options.glow_foreground then ring = "_AlphaGlow"
		elseif options.glow            then ring = "_noglow"
		else ring = "" end
		ring = "Audiosurf_assets/ring" .. ring .. ".png"

SetRings{
	texture=ring,
	--layer = L_MAIN, -- no glow
	shader=ifinvert("VertexColorUnlitTintedAlpha2", "VertexColorUnlitTintedAddSmooth"),
	size=fif(options.track_rings, options.track_width * (44/23), 0),
	percentringed=0.2,
	useairrings = options.air_rings and not options.invert, -- TODO
	airtexture="Audiosurf_assets/Bits.png",
	airshader="VertexColorUnlitTintedAddSmoothNoDepth",
	airsize=16
}
	end

	
	if not options.water_wake then
		SetWake{height=0}
	else
		-- setwake is retarded and doesn't always listen to our changes if it thinks they are the same as the previous settings
		SetWake{
			height = 99,
			fallrate = 99,
			layer = 1,
			topcolor = black,
			bottomcolor = black,
			extraspray = true,
			shader = "VertexColorUnlitTinted",
		}
		SetWake{
			height = fif(options.quality == 3 and options.isjumping, 4, 2),
			fallrate = 0.95,
			topcolor = nil, -- color changing
			bottomcolor = ifinvert(white, nil),
			extraspray = false,
			
			--shader = "VertexColorUnlitTintedAlpha"
			--shader = ifinvert("VertexColorUnlitTinted", "VertexColorUnlitTintedAddSmooth"),
			shader = ifinvert("VertexColorUnlitTinted", "DBN_assets/FG/Wake" .. options.glowing_suffix),
			layer = 13,
			shadercolors = {
				-- apparently SetWake forces _Color to be {1,1,1,1}
				_Colour = fif(options.glow_foreground, greyscale(0.25,0.25), nil), -- THIS IS STUPID
			},
		}
	end

	---------------------------------------------------------------------------------------------------------------------------
	-- Blocks
	---------------------------------------------------------------------------------------------------------------------------
	do
		local base = {
			mesh = "DBN_assets/FG/block" .. ifinvert("_noborder", "") .. fif(options.quality == 1, "_LQ.obj", ".obj"),
			
			shader = "DBN_assets/FG/Block" .. options.mirrored_suffix .. options.glowing_suffix,
			renderqueue = 2000 - 3,
		}
		
		local blocks = {
			colorblocks={
				height = 0.65,
				float_on_water = false, --options.water, -- hurts visibility too much
				shadercolors = {
					_AlphaGlowTint = greyscale(2/3, 1/2),
					_RimColor = greyscale(ifinvert(1/8,-1/8)),
					_BorderAdd = greyscale(fif(options.trackless, 3/4, 1/4)), -- higher contrast in trackless mode
					_BorderMult = greyscale(fif(options.trackless, 1/4, 1)),
				},
				scale = {1,1,1}, -- this is required. the default scale is something like {1,2/3,1}
			},
			greyblocks={
				shadercolors = {
					_AlphaGlowTint = greyscale(1, 0),
					_Color = greyscale(6/10),
					_RimColor = greyscale(ifinvert(1/4,-1/4)),
					_BorderAdd = greyscale(0.85),
					_BorderMult = greyscale(0),
				},
				--scale = {1,1,1}, -- greyblocks can't be scaled
			},
			chainspans = {
				hide = false,
				shader = "DBN_assets/FG/ColoredRail" .. options.glowing_suffix,
				shadercolors = {
					_Color = options.glowing_common_tint,
				},
			},
		}
		concatTable(blocks.colorblocks, base)
		if options.skinny_greys then blocks.greyblocks = nil -- use the stock greyblocks
		else concatTable(blocks.greyblocks, base) end
		
		SetBlocks(blocks)
	end

	---------------------------------------------------------------------------------------------------------------------------
	-- Puzzle
	---------------------------------------------------------------------------------------------------------------------------
	SetPuzzleGraphics{
--[[
		puzzlematerial = {
			texture = "DBN_assets/FG/Puzzle.png",
			shader = "DBN_assets/FG/Puzzle.shader",
			--renderqueue = 2999, -- DOESN'T WORK
			texturewrap = "clamp",
			usemipmaps = false,
			shadercolors = {
				_Color = greyscale(0.85), -- tone down the colors a bit
			}
		},
		puzzlematchmaterial = {
			texture = "Audiosurf_assets/TileMatching.png",
			shader = "ParticleAddNoVertexColor.shader", -- fixes the multicolor/missing glitch
			texturewrap = "clamp",
			usemipmaps = false,
		},
		puzzleflyupmaterial = {
			texture = "Audiosurf_assets/TileMatching.png",
			shader = "Particles/Additive",
			texturewrap = "clamp",
			usemipmaps = false,
		},
]]
		usesublayerclone = false,
	}
	
	---------------------------------------------------------------------------------------------------------------------------
	-- Post-Processing Effects
	---------------------------------------------------------------------------------------------------------------------------
	
	--[[
	The trick here is to write a vertex shader that DOESN'T do that standard object space -> camera space
	transformation; it should ignore the object's position/rotation and place it on the screen in an unchanging
	position. From there, all we need to do is create an object that will always be generally in visible range
	so the engine won't cull it for optimization purposes. railoffset=0 binds it to the player's current
	position on the highway, which should be reliable since it's the same technique used by the skywires in
	stock skins. In this case, we put each vertex on the screen according to its UV position; 0,0 is the bottom
	left of the screen and 1,1 is the top right. DBN_assets/PP/full.obj is just like quad.obj (see quad-readme.png) except
	the UVs run from -0.01 to 1.01 instead of 0 to 1. This is done because, in practice, rendering precisely on
	the screen edges caused slight seams.
]]
local makeDummyObject
do
	local mesh
	function makeDummyObject(tbl)
		if mesh == nil then mesh = BuildMesh{mesh="DBN_assets/PP/full.obj"} end
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject = concatTable({
				mesh = mesh,
				--mesh = "DBN_assets/PP/right.obj",
				--mesh = "DBN_assets/PP/bottom.obj",
				renderqueue = 4000,
				layer = 13, -- don't render on the glow camera
				--layer = 15, -- render on top of the vehicle
				-- note: probably need to go to layer 14 if you want a postprocess effect to go on top of the surfer
				-- https://dl.dropboxusercontent.com/u/610214/~Post/as2-doc.html#layers
			}, tbl)
		}
	end
end

if not options.glow then
	-- prevent anything from being drawn in the glow layer because apparently it doesn't get disabled...
	makeDummyObject{
		renderqueue = 1,
		shader = "DBN_assets/PP/SetZ.shader",
		layer = 18,
		shadersettings = {
			_Z = 0,
		},
	}
end
if options.glow_foreground then
	assert(type(options.glow_foreground_passes) == [[number]], [[Debug Assert failed, (options.glow_foreground_passes) is not a number]])
	local glow_passes = options.glow_foreground_passes
	if glow_passes == 1 then
		fif(options.shuffle, print, error)("sorry, glow_foreground_passes does not support 1 pass")
		glow_passes = 0
	end
	if glow_passes > 0 then
		-- set the background alpha to 0
		makeDummyObject{
			renderqueue = 1000 + 1,
			shader = "DBN_assets/PP/SetAlpha.shader",
			shadersettings = {
				_Alpha = 0,
			},
		}
		--[ [
		-- do the impossible
		local glow_renderqueue = 4000
		local glow_blur_size = options.glow_foreground_radius
		makeDummyObject{
			renderqueue = glow_renderqueue,
			shader = "DBN_assets/PP/AlphaGlowStart.shader",
			shadersettings = {
				-- a ton of things write 100% alpha and some of them don't allow shader overrides. (water, clouds, beat flash)
				-- the blocks write 0% alpha and we will invert it. the track surface and side rails write a light alpha so we will only use the 0.5 - 0.0 range
				_AlphaMult = -2,
				_AlphaAdd = 1,
				_BlurSize = glow_blur_size,
			},
		}

		if glow_passes > 2 then
			-- BuildMaterial to avoid duplicating the shader several times in memory which would probably needlessly complicate the draw call process
			local pass = {
				material = BuildMaterial{
					shader = "DBN_assets/PP/AlphaGlowPass.shader",
					renderqueue = 4000 + 1,
					shadersettings = {
						_BlurSize = glow_blur_size,
					},
				},
			}
			for i=1,glow_passes-2 do makeDummyObject(pass) end
		end

		if not options.troll then
			makeDummyObject{
				shader = "DBN_assets/PP/AlphaGlowEnd.shader",
				renderqueue = 4000 + 2,
				shadersettings = {
					_BlurSize = options.glow_foreground_lastpass_radius,
				},
			}
		end
	end
end

-- see the invisible
--[[
makeDummyObject{
	mesh = "DBN_assets/PP/right.obj",
	--mesh = "DBN_assets/PP/bottom.obj",
	renderqueue = RQ_OVERLAY + 100,
	shader = "DBN_assets/PP/RevealAlpha.shader",
}--]]
--[[
makeDummyObject{
	mesh = "DBN_assets/PP/right.obj",
	--mesh = "DBN_assets/PP/bottom.obj",
	renderqueue = RQ_OVERLAY + 100,
	shader = "DBN_assets/PP/RevealStencil.shader",
}--]]
--[[
makeDummyObject{ -- used to figure out the renderqueue of some other object in the scene by repeatedly changing this object's queue to perform a manual binary search
	mesh = "DBN_assets/PP/right.obj",
	--mesh = "DBN_assets/PP/bottom.obj",
	renderqueue = 3899,
	shader = "DBN_assets/PP/Solid.shader",
	shadercolors = { _Color = greyscale(0.5) },
}--]]

	
	---------------------------------------------------------------------------------------------------------------------------
	-- Player model
	---------------------------------------------------------------------------------------------------------------------------

	-- can't control what alpha the rooster sets but this looks alright
	local roostercolor = greyscale(fif(options.glow_foreground, 0.01, 0.05))

	if options.isjumping then
		local rot = {20.49113, 0, 0}
		local debug_scale = fif(options.debug_camera, 10, 1)
		local player = {
			cameramode = "first_jumpthird",

			camfirst = {
				pos = {0, 2.7, -3.50475},
				rot = rot,
				strafefactor = 1,
			},
			camthird = { --lerps out towards pos2 when the song is less intense
				pos  = { 0, 2.7 * debug_scale, -3.50475 * debug_scale },
				rot  = rot,
				strafefactor = 0.75,
				pos2 = { 0, 2.8 * debug_scale, -3.50475 * debug_scale },
				rot2 = rot,
				strafefactorFar = 1,
			},
			
		}
		if not options.invert then
			player.surfer = {
				roostercolor = roostercolor,
				arms = {
					shader = "RimLightHatchedSurfer",
					shadercolors = {
						_Color = { colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=6, param="_Threshold", paramMin=2, paramMax=2 },
						_RimColor = {0,63,192},
					},
					texture = "Audiosurf_assets/FullLeftArm_1024_wAO.png",
				},
				board = {
					shader = ifhifi("RimLightHatchedSurferExternal","VertexColorUnlitTinted"),
					renderqueue = 4000 - 1, -- TODO: investigate why this is so high
					shadercolors = {
						_Color = { colorsource="highway", scaletype="intensity", minscaler=5, maxscaler=5 },
						_RimColor = black,
					},
					shadersettings = {
						_Threshold = 11,
					},
					texture = "Audiosurf_assets/board_internalOutline.png",
				},
				body = {
					shader = "RimLightHatchedSurferExternal",
					renderqueue = 3000,
					shadercolors = {
						_Color = { colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=3 },
						_RimColor = black,
					},
					shadersettings = {
						_Threshold = 1.7,
					},
					texture = "Audiosurf_assets/robot_HighContrast.png",
				}
			}
		else
			player.surfer = {
				arms = {
					shader = "DBN_assets/UnlitTintedInvert.shader",
					shadercolors = {
						_Color = "highway",
					},
					recalculateNormals = false,
					texture = "Audiosurf_assets/FullLeftArm_1024_wAO_Neg.png",
				},
				board = {
					shader = "DBN_assets/UnlitTintedInvert.shader",
					shadercolors = {
						_Color = { colorsource="highway", scaletype="intensity", minscaler=0.5, maxscaler=0.5 },
					},
					texture = "Audiosurf_assets/board_internalOutline.png",
				},
				leg = {
					shader = "VertexColorUnlitTinted",
					shadercolors = {
						_Color = "highway",
					},
					recalculateNormals = false,
					-- don't have leg.png
					--texture = "White.png",
				},
				body = {
					shader = "DBN_assets/UnlitTintedInvert.shader",
					shadercolors = {
						_Color = "highway",
					},
					texture = "Audiosurf_assets/iRobot_AO.png",
				}
			}
		end
		SetPlayer(player)
	else
		local ship_material = BuildMaterial{
			renderqueue = 2000 - 3,
			shader = "3rd_assets/MatCap_Glow" --[[.. options.mirrored_suffix]] .. options.glowing_suffix,
			shadersettings = {
				_GlowBrightness = 0,
			},
			shadercolors={
				_Color = { colorsource="highway", scaletype="intensity", minscaler=2/3, maxscaler=2/3 },
				_GlowColor = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1 },
			},
			textures = {
				_MatCap = "Audiosurf_assets/matcapchrome.jpg",
				_Glow = "Audiosurf_assets/glowBWj1_DBN_edit.png",
			},
		}
		local ship_shadersettings = {}
		local ship_material_update = {
			material = ship_material,
			shadersettings = ship_shadersettings,
		}
		local UpdateShaderSettings = UpdateShaderSettings
		onUpdate(function(delta_time, location, strafe, jump_height, intensity)
			if delta_time == 0 then return end
			ship_shadersettings._GlowBrightness = intensity * 7/4 + 1/4
			UpdateShaderSettings(ship_material_update)
		end)

		local player = {
			--showsurfer = false,
			--showboard = false,
			cameramode = "third",

			camfirst={
				pos={0,1.84,-0.8},
				rot={20,0,0}},
			camthird={
				pos=fif(options.debug_camera, {0,40,-10}, {0,2,-0.5}),
				rot={30,0,0},
				strafefactorFar = .75,
				pos2={0,5,-4},
				rot2={30,0,0},
				strafefactorFar = 1,
				transitionspeed = 1,
				puzzleoffset=-0.65,
				puzzleoffset2=-1.5},

			vehicle={
				min_hover_height= 0.11,
				max_hover_height = 1.4,
				use_water_rooster = false,
				smooth_tilting = false,
				smooth_tilting_speed = 10,
				smooth_tilting_max_offset = -20,

				-- TODO: make the reflection as a child object instead?
				mesh = "Audiosurf_assets/racingship_scaled75_scaled0.75_left0.023_back0.4.obj",
				--pos = {0, 0, 0},
				--scale = {1, 1, 1},
				material = ship_material,
				--[[
				mesh = "DBN_assets/Particle.obj",
				material = BuildMaterial{
					shader = "DBN_assets/Particle.shader",
					shadersettings = {
						_Scale = 1,
					},
					shadercolors={
						_Color = {1,1,1,0},
					},
					texture = "DBN_assets/Particle.png",
				}, --]]
				reflect = false,
				layer = 13,

				thrusters = {
					crossSectionShape = makeCylinderThruster(0.5, ifquality(12,7,7)),
					perShapeNodeColorScalers = clonesArray(1,12), -- mandatory. can have more entries than the number of cross section points
					shader = "DBN_assets/FG/Thruster.shader",
					renderqueue = 3034+1,
					layer = options.glowing_layer,
					colorscaler = {
						close = fif(options.glow_foreground, 0.25, 2.5),
						far = 0,
					},
					sizefalloff = 1,
					minrenderedsize = 0, --0.05,
					extrusions = 25,
					stretch = -0.1191,
					updateseconds = 0.025,
					instances = {
						{ pos={ 0.00, 0.3675, -1.515}, scale={0.405, 0.450, 0.450} },
						{ pos={ 0.21, 0.2100, -1.530}, scale={0.225, 0.225, 0.495} },
						{ pos={-0.21, 0.2100, -1.530}, scale={0.225, 0.225, 0.495} },
					},
				},
			},
		}
		--[[
		player.camthird.pos={0,20,0}
		player.camthird.rot={90,0,0}
		player.camthird.pos2=player.camthird.pos
		player.camthird.rot2=player.camthird.rot
		--]]
		
		--[[if options.mirror then
			-- because we can't give different materials to different instances and the grid renderqueue is locked
			local instances = player.vehicle.thrusters.instances
			local thrusters = shallowCopy(player.vehicle.thrusters)
			player.vehicle.childobjects = {{ layer=L_MAIN, thrusters=thrusters }}

			thrusters.shader = "DBN_assets/FG/Thruster" .. options.mirrored_suffix .. ".shader"
			thrusters.renderqueue = RQ_MIRRORED_TRANS

			local new_instances = {}
			thrusters.instances = new_instances

			for i,instance in ipairs(instances) do
				table.insert(new_instances, {
					pos=vec3.mul(instance.pos, {1,-1,1}),
					rot=instance.rot,
					scale=instance.scale,
				})
			end
		end]]
		SetPlayer(player)
	end

end

do
	local track, track_vec3s, node_count, sqrt, floor = track, track_vec3s, #track, math.sqrt, math.floor
	local max_node_xz_len2, max_node_xz_len = 0
	do
		local a,b
		b = track_vec3s[1]
		for i=2,node_count do
			a, b = b, track_vec3s[i]
			local len_squared = (a[1] - b[1])^2 + (a[3] - b[3])^2
			if max_node_xz_len2 < len_squared then max_node_xz_len2 = len_squared end
		end
		max_node_xz_len = sqrt(max_node_xz_len2)
	end
	local node_side = node_bounds.side

	function collide_XZCircle_Track(x,z,radius)
		radius=radius+node_side
		local i, count, forcex,forcez = 1, 0, 0,0
		while i <= node_count do
			local pos = track_vec3s[i]
			local dX,dZ = x - pos[1], z - pos[3] -- delta X Z
			local dist = sqrt(dX^2 + dZ^2)
			if dist < radius then
				count=count+1
				-- have the node exert force on the struct
				if dist ~= 0 then
					forcex=forcex + dX/dist -- normalize
					forcez=forcez + dZ/dist
				end
				i=i+1
			else -- skip nodes that have no chance of hitting the circle even if the track is on a direct collision course
				dist = floor((dist-radius) / max_node_xz_len) -- reuse variable
				if dist > 0 then i=i+dist else i=i+1 end
			end
		end

		-- the direction to move away from each colliding node is normalized and then all those are added together
		-- if all the nodes are on the same side of the circle then the length of the vector will equal "count"
		return count, forcex, forcez
	end
end
do
	local CreateObject = CreateObject -- for speed
	local fif, vec2, vec3, sin, cos, random, table = fif, vec2, vec3, math.sin, math.cos, math.random, table
	local track, track_vec3s, track_bounds = track, track_vec3s, track_bounds
	local debugging = options.debug

	---------------------------------------------------------------------------------------------------------------------------
	-- Calculate the scene dimensions
	---------------------------------------------------------------------------------------------------------------------------
	local scene_radius, scene_inner_radius, scene_floor, scene_ceiling, scene_height, scene_center
	local scene_floor_thickness, scene_ceiling_thickness
	do
		local level_height = track_bounds.node_max[2] - track_bounds.node_min[2]
		-- treating the square like it's textured as a circle; the corners are not important
		local radius = track_bounds.node_center_horizontal_radius * 2 * 4 -- 4x the size of the level
		
		-- distance from the bottom of the level to the ground
		local distance = math.max(1000, level_height / 10)
		
		-- now make the ground look that big from the top of the level rather than from the bottom
		-- calculate the line from the center of the bottom to the edge of the ground, where the center of the bottom is {0,0} and the edge is {distance,radius}
		local m = (radius)/distance
		-- now find the point on that line moved down as far as the level is tall
		scene_radius = m * (distance + level_height)
		
		scene_inner_radius = radius
		scene_floor = track_bounds.node_min[2] - distance
		scene_ceiling = track_bounds.node_max[2] + 1000 -- will probably change this when I actually start working on the ceiling
		scene_height = scene_ceiling - scene_floor
		scene_center = track_bounds.node_center
		
		scene_floor_thickness = 500000
		scene_ceiling_thickness = 5000
	end

	function makeAnimatedRingMesh(points)
	local points2 = points * 2
	local addQuadIndices = addQuadIndices

	-- pre-calculate a unit circle
	local template_vertextable, template_uvtable = {}, {}
	do
		local math, insert = math, table.insert
		local tau = math.tau
		for i=1,points do
			local fraction = (points - i)/points -- clockwise
			local angle = tau * fraction
			local x, z = math.cos(angle), math.sin(angle)
			template_vertextable[i         ] = {x, nil, z}
			template_vertextable[i + points] = {x, nil, z}
			template_uvtable[i         ] = {fraction, 1}
			template_uvtable[i + points] = {fraction, 0}
			-- TODO BUG: the last polygon has a backwards higher resolution texture. there needs to be duplicate vertices at the UV wrap point for a proper seam.
		end
	end

	-- the tables that will be sent to BuildMesh
	local vertextable, uvtable, colortable, indextable = {}, {}, {}, {}
	local mesh_dirty = true -- indextable has changed and needs to be re-sent

	local rings_count = 0
	local rings_allocated = {} -- an array of booleans for whether an existing ring is allocated (true) or removed via removeRing() (false)
	local rings_allocated_count = 0

	-- returns ring_id, color
	-- if supplied, color will be used rather than creating a new table. rings created this way must never be removed
	local function makeRing(outer, inner, slot, color)
		-- reuse removed rings
		if rings_count > rings_allocated_count then
			for i=1,rings_count do
				if rings_allocated[i] == false then
					rings_allocated[i] = true
					rings_allocated_count = rings_allocated_count + 1
					local offset = (i-1) * points2
					for i=1,points2 do
						local scale
						if i <= points then scale = outer
						else scale = inner end

						local vert, template = vertextable[i + offset], template_vertextable[i]
						vert[1], vert[2], vert[3] = template[1] * scale, slot, template[3] * scale
						--uvtable[i + offset][2] = template_uvtable[i][2] + slot
					end
					return offset, colortable[offset+1]
				end
			end
			error("corrupted ring state (allocated count doesn't match actual number allocated)")
		end

		-- make new ring
		rings_count = rings_count + 1
		rings_allocated_count = rings_allocated_count + 1
		rings_allocated[rings_count] = true
		mesh_dirty = true
		--if print ~= nil then print("rings_count increased to " .. rings_count) end

		color = color or {1,1,1,1}
		local offset = #vertextable

		for i=1,points2 do
			local vert = template_vertextable[i]
			--local uv = template_uvtable[i]
			local offset_i = i + offset
			local scale
			if i <= points then scale = outer
			else scale = inner end
			vertextable[offset_i] = {vert[1] * scale, slot, vert[3] * scale}
			uvtable[offset_i] = template_uvtable[i] --{uv[1], uv[2] + slot}
			colortable[offset_i] = color
		end
		for i=offset,offset+points-2 do
			-- indextable is 0-based
			addQuadIndices(indextable, i, i+1, i+points, i+points+1)
		end
		addQuadIndices(indextable, offset+points-1, offset, offset+points*2-1, offset+points)
		return offset, color
	end

	local function updateRing(ring_id, inner, outer)
		for i=1,points2 do
			local scale
			if i <= points then scale = outer
			else scale = inner end
			local template, vert = template_vertextable[i], vertextable[i + ring_id]
			vert[1], vert[3] = template[1] * scale, template[3] * scale
		end
	end

	local function updateMainRing(ring_id, outer)
		for i=1,points do
			local template, vert = template_vertextable[i], vertextable[i + ring_id]
			vert[1], vert[3] = template[1] * outer, template[3] * outer
		end
	end

	local function removeRing(ring_id)
		updateRing(ring_id, 0, 0)
		local i = ring_id / points2 + 1
		rings_allocated[i] = false
		rings_allocated_count = rings_allocated_count - 1
	end

	local mesh = {vertextable = vertextable, uvtable = uvtable, colortable = colortable, indextable = indextable}
	local BuildMesh = BuildMesh

	local function buildUpdates()
		if mesh_dirty then
			mesh.indextable = indextable
			BuildMesh(mesh)
			mesh.indextable = nil
			mesh_dirty = false
		else
			-- sending the indextable when it hasn't changed is a big performance waste
			BuildMesh(mesh)
		end
	end

	-- only call this once
	local function buildInitial()
		local mesh_built = BuildMesh(mesh)
		mesh.mesh = mesh_built
		mesh.indextable = nil
		mesh_dirty = false
		return mesh_built, buildUpdates
	end

	local ringDebugStats
	if debugging then
		function ringDebugStats() return rings_count, rings_allocated_count end
	end

	return makeRing, updateRing, updateMainRing, removeRing, buildInitial, ringDebugStats
end

function makeCircleVisualizer(o)
	local BuildMesh, GetHighwayColor, min, max, insert, remove, pairs = BuildMesh, GetHighwayColor, math.min, math.max, table.insert, table.remove, pairs
	local makeRing, updateRing, updateMainRing, removeRing, buildRings, ringDebugStats = makeAnimatedRingMesh(o.vertices)

	local bin_count, inner_radius, radius_scale = o.bin_count, o.inner_radius, o.radius_scale
	local start_health, fade_health = o.start_health, o.fade_health
	local speed_exponent, speed_scale, speed_add = o.speed_exponent, o.speed_scale, o.speed_add
	local wave_size_min, wave_brightness = o.wave_size_min, o.wave_brightness
	local wave_size_min_flip = 1 - wave_size_min
	o = nil

	

	-- power: 0-1 measure of the wave's intensity
	-- dist:  how far from the center the wave is, without "inner_radius" and "radius_scale" applied
	local w_id, w_power, w_dist, w_health, w_color = {}, {}, {}, {}, {}
	local function newWave(offset, power, dist, color)
		local id, c = makeRing(((dist) * radius_scale + inner_radius), ((dist + power) * radius_scale + inner_radius), offset)
		c[1], c[2], c[3], c[4] = color[1], color[2], color[3], wave_brightness
		insert(w_id, id)
		insert(w_power, power)
		insert(w_dist, dist)
		insert(w_health, start_health)
		insert(w_color, c)
	end

	local main, main_color = {}, {1,1,1,1}
	local velocities, previous = {}, {}

	for i=1,bin_count do
		main[i] = makeRing(inner_radius, inner_radius, i-1, main_color)
		velocities[i] = 0
		previous[i] = 0
	end

	-- slope should be a number from 0-1
	local function update(bins, delta_time, speed)
		local color = GetHighwayColor()
		main_color[1], main_color[2], main_color[3] = color[1], color[2], color[3]

		-- simulate existing waves
		for i,id in pairs(w_id) do
			local delta = delta_time * speed
			local health = w_health[i] - delta
			--local health = w_health[i] - delta_time * 1/2
			if health <= 0 then
				removeRing(id)
				remove(w_id, i)
				remove(w_power, i)
				remove(w_dist, i)
				remove(w_health, i)
				remove(w_color, i)
			else
				local power = w_power[i]
				local dist = w_dist[i] + delta*(power ^ speed_exponent * speed_scale + speed_add)
				w_health[i], w_dist[i] = health, dist
				if health < fade_health then w_color[i][4] = health / fade_health * wave_brightness end
				updateRing(id, ((dist) * radius_scale + inner_radius), ((dist + power) * radius_scale + inner_radius))
			end
		end

		-- the more waves that currently exist, the less probable it is that a new one will be thrown
		-- temporary measure until AS2 performance and memory problems are fixed
		local wave_size_min = (max(#w_id-bin_count,4)-4) / 40 * wave_size_min_flip + wave_size_min

		for i=1,bin_count do
			-- update the main waves
			local bin = bins[i]
			updateMainRing(main[i], ((bin) * radius_scale + inner_radius))

			-- throw new waves
			local velocity, prev = velocities[i], previous[i]
			previous[i] = bin
			local delta = bin - prev
			local wave_size = max(-delta, velocity) --min(-delta, velocity)
			if wave_size > wave_size_min then
				newWave(i-1, min(wave_size, 1), bin, color) -- we should be doing this min() in the master function that does the spectrum filtering, but the data is useful for the dynamic wave_size_min. should move this if/when that is ever fixed
				velocity = 0 --velocity - wave_size
			end
			velocities[i] = max(0,max(0, velocity - 10*delta_time) + delta)
		end
		buildRings()
	end
	local mesh
	mesh, buildRings = buildRings()
	return mesh, update, ringDebugStats

	
end

if debugging then
	local graph_mesh, background_mesh, updateGraph = makeBarGraph{
		count = 4,
		size = {1/16, 1/4},
		border = 1/4 * 0.05,
		offset = {15/16 - 1/4 * 0.1, 0 + 1/4 * 1.1},
		bar_scale = 0.9,
		uv_mode = "slide",
		uv_antistretch = "top"
	}
	CreateObject{
		railoffset = 0,
		floatonwaterwaves = false,
		gameobject={
			mesh = graph_mesh,
			texture = "graph/bars_horizontal_flip.png",
			texturewrap = "clamp",
			shader = "DBN_assets/HUD/VertexColorTinted.shader",
			renderqueue=4500,
			layer = 13,
		}
	}
	CreateObject{
		railoffset = 0,
		floatonwaterwaves = false,
		gameobject={
			mesh = background_mesh,
			shader = "DBN_assets/HUD/VertexColorTinted.shader",
			renderqueue=4500 - 1,
			layer = 13,
			shadercolors = { _Color=black },
		}
	}
	local spectrumLinear = spectrumLinear
	local new_spectrum = {0,0,0,0}
	local max = math.max
	onUpdateSpectrum(function(spectrum)
		for i=1,4 do
			local x = (i-1) * 4 + 1
			--new_spectrum[i] = (spectrum[x] + spectrum[x+1] + spectrum[x+2] + spectrum[x+3]) / (4*4)
			new_spectrum[i] = max(spectrum[x], spectrum[x+1], spectrum[x+2], spectrum[x+3]) / 2
		end
		--spectrumLinear(new_spectrum)
		updateGraph(new_spectrum)
	end)
end

local hex_mesh
do
	local updateGraph
	if debugging then
		local graph_mesh, background_mesh
		graph_mesh, background_mesh, updateGraph = makeBarGraph{
			count = 11,
			size = {1/8, 1/4},
			border = 1/4 * 0.05,
			offset = {7/8 - 1/4 * 0.1, -28.78/32 + 1/4 * 1.1},
			bar_scale = 0.9,
			uv_mode = "slide",
			uv_antistretch = "top"
		}
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject={
				mesh = graph_mesh,
				texture = "graph/bars_horizontal_flip.png",
				texturewrap = "clamp",
				shader = "DBN_assets/HUD/VertexColorTinted.shader",
				renderqueue=4500,
				layer = 13,
			}
		}
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject={
				mesh = background_mesh,
				shader = "DBN_assets/HUD/VertexColorTinted.shader",
				renderqueue=4500 - 1,
				layer = 13,
				shadercolors = { _Color=black },
			}
		}
	end

	local opt = options.struct_hex
	local floor = math.floor
	local updateHexVis, ringDebugStats
	hex_mesh, updateHexVis, ringDebugStats = makeCircleVisualizer{
		vertices =        opt.vertices,
		bin_count = 8,
		radius_scale = 13,
		inner_radius = 2,
		start_health =    opt.start_health, -- waves lose 1 HP per second when the speed scale is 1
		fade_health =     opt.fade_health, -- the point when waves start fading out
		wave_brightness = opt.wave_brightness,
		wave_size_min =   opt.wave_size_min,
		-- power ^ exponent * scale + add
		speed_exponent =  opt.speed_exponent,
		speed_scale =     opt.speed_scale,
		speed_add =       opt.speed_add,
	}
	--[[
		vertices = 6,
		bin_count = 8,
		radius_scale = 13,
		inner_radius = 2,
		start_health = 1, -- waves lose 1 HP per second when the speed scale is 1
		fade_health = 0.5, -- the point when waves start fading out
		wave_brightness = 1/4,
		wave_size_min = 1/10,
		-- power ^ exponent * scale + add
		speed_exponent = 4,
		speed_scale = 6,
		speed_add = 1,
	}--]]
	if opt.upscale == "uphill" then
		opt = function(slope) return 2 - slope end
	elseif opt.upscale == "downhill" then
		opt = function(slope) return 1 + slope end
	elseif opt.upscale == "always" then
		opt = function() return 2 end
	else
		opt = function() return 1 end
	end
	onUpdateSpectrum(function(spectrum, delta_time, track_location)
		local new_spectrum = {}
		local slope = track[floor(track_location)].color.a
		local scale = opt(slope) / 2
		for i=1,8 do new_spectrum[i] = (spectrum[i*2] + spectrum[i*2-1]) * scale end
		updateHexVis(new_spectrum, delta_time, slope * 7/4 + 1/4)
		if debugging then
			new_spectrum[9] = slope
			local rings_count, rings_allocated_count = ringDebugStats()
			new_spectrum[10] = (rings_allocated_count-8) / 40
			new_spectrum[11] = (rings_count-8) / 40
			updateGraph(new_spectrum)
		end
	end)
end

--[[
function makeCylinderVisualizer(points, count, inner, outer, far_outer, farthest_outer)
	local makeRing, updateRing, updateMainRing, removeRing, buildRings = makeAnimatedRingMesh(points)

	local main, main_color = {}, {1,1,1,1}

	local wave_ids, wave_sizes, wave_dists, wave_colors = {}, {}, {}, {}
	local velocities, previous = {}, {}

	for i=1,count do
		main[i] = makeRing(inner, inner, i-1, main_color)
		velocities[i] = 0
		previous[i] = 0
	end

	local range = outer - inner
	local falloff_inner, falloff_outer = (far_outer - inner) / range, (farthest_outer - inner) / range
	local falloff_range = falloff_outer - falloff_inner
	local BuildMesh, GetHighwayColor, min, max, insert, remove, pairs = BuildMesh, GetHighwayColor, math.min, math.max, table.insert, table.remove, pairs
	local speed_min, speed_max = 1, 15
	local speed_range = speed_max - speed_min

	local function update(bins, delta_time)
		local color = GetHighwayColor()
		main_color[1], main_color[2], main_color[3] = color[1], color[2], color[3]

		-- simulate existing waves
		for i,id in pairs(wave_ids) do
			local size, dist = wave_sizes[i], wave_dists[i]
			local falloff = (dist - falloff_inner) / falloff_range
			if falloff >= 1 then
				removeRing(id)
				remove(wave_ids, i)
				remove(wave_sizes, i)
				remove(wave_dists, i)
				remove(wave_colors, i)
			else
				if falloff > 0 then
					falloff = 1-falloff
					size = size * max(falloff, 0.25)
					wave_colors[i][4] = falloff * 0.25
				end
				dist = dist + delta_time * (size * speed_range + speed_min)
				updateRing(id, dist * range + inner, (dist - size) * range + inner)
				wave_dists[i] = dist
			end
		end

		-- temporary measure until performance and memory problems are fixed
		local wave_size_min = #wave_ids / 50 * 0.9 + 0.1

		for i=1,count do
			-- update the main waves
			local bin = bins[i]
			updateMainRing(main[i], bin * range + inner)

			-- throw new waves
			local velocity, prev = velocities[i], previous[i]
			previous[i] = bin
			local delta = bin - prev
			local wave_size = max(-delta, velocity) --min(-delta, velocity)
			--if (wave_size > 0.1) then
			if (wave_size > wave_size_min) then
				local id, wave_color = makeRing(prev * range + inner, (prev - wave_size) * range + inner, i-1)
				wave_color[1], wave_color[2], wave_color[3], wave_color[4] = color[1], color[2], color[3], 0.25
				insert(wave_ids, id)
				insert(wave_sizes, wave_size)
				insert(wave_dists, prev)
				insert(wave_colors, wave_color)
				velocity = 0 --velocity - wave_size
			end
			velocities[i] = max(0,max(0, velocity - 10*delta_time) + delta)
		end
		buildRings()
	end
	local mesh
	mesh, buildRings = buildRings()
	return mesh, update
end

-- old version:
--hex_mesh, updateHexVis = makeCylinderVisualizer(6, 8, 2, 15, 21, 32)
--]]
	
	-- EndCookie
	--[[
	CreateObject{
		name = "EndCookie",
		tracknode = "end",
		gameobject = {
			transform = {
				pos = {0,0,126},
				scale = {55,99,900},
			},
			mesh = "Audiosurf_assets/danishCookie_boxes.obj",
			--shader = "RimLightHatched",
			--shader = "VertexColorUnlitTinted",
			shader = "assets/Solid.shader",
			shadercolors = {
				--_Color = {0,0,0}
				--_Color = "highwaysmooth"
				--_Color = {colorsource="highway", scaletype="intensity", minscaler=6, maxscaler=1}
				_Color = greyscale(1,1),
				--_RimColor = "highwaysmooth"--{1,1,1}
			},
			layer = L_MAIN,
		}
	} --]]

	local exit_pos = track_vec3s[#track_vec3s]

	if debugging then -- incomplete
		do --------- cloudjet ---------
			local material = BuildMaterial{
				shader  = "DBN_assets/BG/exit/cloudjet.shader",
				texture = "DBN_assets/BG/exit/cloudjet.png",
				shadercolors = {
					--_Color = {colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1}, --, param="_Intensity", paramMin=0, paramMax=1}
					_Color = options.color_downhill,
				},
			}
			local shadersettings = {}
			local material_update = {
				material = material,
				shadersettings = shadersettings,
			}
			local intensity_time = 0
			local UpdateShaderSettings = UpdateShaderSettings
			onUpdate(function(delta_time, location, strafe, jump_height, intensity)
				if delta_time == 0 then return end
				intensity_time = intensity_time - delta_time * (1 + intensity*3)
				shadersettings._ScrollTime = intensity_time * (2/8)
				shadersettings._ScrollTime2 = intensity_time * (3/8)
				shadersettings._Intensity = 1/4 + 3/4 * intensity
				UpdateShaderSettings(material_update)
			end)
			CreateObject{
				gameobject = {
					transform = {
						pos = exit_pos,
						scale = {100,2000,100},
						--scale = {50,50,50},
						rot = {0,180,0},
					},
					--mesh = "DBN_assets/BG/exit/cloudjet_debug.obj",
					mesh = "DBN_assets/BG/exit/cloudjet.obj",
					material = material,
					layer = 13,
				}
			}
		end

		do --------- helix ---------
			local scale = 20
			--scale = 1

			local material_table = {
				shader  = "DBN_assets/ParticleVortex.shader",
				textures = {
					_MainTex     = "DBN_assets/BG/exit/helix_particle.png",
					_CylinderTex = "DBN_assets/BG/exit/helix.png",
				},
				shadercolors = {
					_Color = options.color_downhill,
				},
				shadersettings = {
					_Scale           = scale/8,
					_SpinSpeed       = math.tau/48,
					_ScaleCylinderU  = 1,
					_ScaleCylinderV  = 1/scale * 1/2, -- include scale to compensate for unity engine faggotry (see below)
					_ScrollCylinderU = 1/4,
					_ScrollCylinderV = -1/4,
				},
			}
			local obj = {
				gameobject = {
					transform = {
						pos = exit_pos,
						scale = {scale,scale,scale},
						rot = {0,0,0},
					},
					mesh = BuildMesh{mesh="DBN_assets/BG/exit/helix.obj"},
					material = BuildMaterial(material_table),
					layer = 13,
				},
				--railoffset = 0,
				--railoffset = 100, -- comment out "pos = exit_pos,"
			}
			local transform = obj.gameobject.transform
			local rot   = transform.rot
			local scale_vec = transform.scale

			local function make()
				rot[1] = 0
				CreateObject(obj)
				rot[1] = 180
				CreateObject(obj)
			end
			local function setRadius(x)
				x = 1.01
				scale_vec[1], scale_vec[3] = scale * x, scale * x
			end

			-- unity engine bakes scaling into the mesh if their scaling along the xyz axes isn't uniform, which interferes with the shader's UV generation
			-- never use exactly 1.0 radius so it can at least be consistent
			setRadius(1.01)
			make()

			rot[2] = 190
			setRadius(0.8)
			make()

			material_table.shadersettings._ScrollCylinderV = -1/3
			material_table.shadersettings._TimeOffset = 2
			obj.gameobject.material = BuildMaterial(material_table)

			rot[2] = -45
			setRadius(1.01)
			make()

			rot[2] = 180
			setRadius(0.7)
			make()
		end
	end

	do --------- visualizer_hack ---------
		local bbfix = 100000 -- scale up the model by this much then scale it back down in the vertex shader.
		                     -- this increases the size of the bounding box to compensate for it being completely wrong
		local o = {
			gameobject = {
				mesh = hex_mesh,
				-- BuildMaterial so the material/shader/texture will be shared between the two objects. (improves performance and memory)
				material = BuildMaterial{
					-- this shader takes the standard horizontally expanding waves and distorts it so they travel upward along a vertical shaft instead
					-- for now it's hardcoded to work based on the hex visualizer dimensions
					shader  = "DBN_assets/BG/exit/visualizer_hack.shader",
					texture = "DBN_assets/BG/hex/wave.png",
					shadercolors = {
						_Color = Inherit(options.color_downhill, {nil,nil,nil,0}),
						_Scale = {1 / bbfix, 100/30 / bbfix, 1 / bbfix, 1}, -- not a color
					}
				},
				layer = 13,
				transform = {
					pos = exit_pos,
					rot = {0,0,0},
					scale = vec3.mulSingle({30,30,30}, bbfix),
					-- scale must be the same for all 3 axes or Unity will scale the mesh before sending it to the gpu, which ruins the special shader
					-- when it's the same, unity does scaling in the transformation matrix and unity_Scale.w
					-- http://forum.unity3d.com/threads/handle-uniform-non-uniform-scale-in-a-shader.115675/
				},
			}
		}
		CreateObject(o)
		o.gameobject.transform.rot[1] = 180 -- upside-down
		CreateObject(o)
	end

	--skywires
	if options.skywires > 0 then
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject = {
				name = "scriptSkyWires",
				mesh = "Audiosurf_assets/skywires.obj",
				renderqueue = 1002,
				layer = options.glowing_layer,
				shader = "DBN_assets/BG/Skywire" .. ifinvert("", "Add") .. ".shader",
				shadercolors = {
					--_Color = "highway" --{255,0,0}
					_Color = greyscale(ifinvert(1 - options.skywires, options.skywires),0.5) -- everything else in the scene is glowing the highway colors so this will give it some contrast. also it makes the special effects on them more visible
				},
				shadersettings = {
					_RollSpeed = math.tau / -100, -- 1 revolution every 100 seconds, clockwise
				},
				--scale = {1,1,1},
				lookat = "end"
			}
		}
	end

	
	
	-- the background should be a bit brighter than black so the
	-- completely black buildings and the cliff stand out a bit
	
	-- background color breaks all glow effects
	if not options.glow then
		SetBackgroundColor(ifinvert({243,243,243}, {12,12,12}))
	end
	
	-- Scene top-down map overlay
	local markPoint, finalizeMarks
if debugging then
	local insert = table.insert
	
	-- returns a function(x, y) that returns those coordinates mapped into a rectangle as big as "size", with its bottom left corner centered at "offset"
	-- min/max are the possible range of inputs that function can receive. (results are not clamped)
	-- if square_aspect is true, min/max is expanded to be a square (letterboxing)
	-- also returns a set of shadersettings that can be passed to plotting shaders
	local function makePlotter(size, offset, z, min, max, square_aspect)
		do -- i've been led to believe that closure local variable garbage collection happens on a per-block basis. the returned function won't need the above arguments
			local scale_x, scale_y
			do
				local range = vec2.sub(max, min)
				if square_aspect then
					local diff = (range[1] - range[2]) / 2
					if diff >= 0 then -- x >= y
						min[2] = min[2] - diff
						max[2] = max[2] + diff
						range[2] = range[1]
					else -- x < y
						min[1] = min[1] + diff
						max[1] = max[1] - diff
						range[1] = range[2]
					end
				end
				scale_x = size[1] / range[1]
				scale_y = size[2] / range[2]
			end
			
			local min_x, min_y = min[1], min[2]
			local offs_x, offs_y = offset[1], offset[2]
			local z = z
			return function (x, y)
				return {
					(x - min_x) * scale_x + offs_x,
					(y - min_y) * scale_y + offs_y,
					z
				}
			end, {
				_MinX = min_x,
				_MinY = min_y,
				_ScaleX = scale_x,
				_ScaleY = scale_y,
				_OffsX = offs_x,
				_OffsY = offs_y,
				_ClipX = (max[1] - min_x) * scale_x + offs_x, -- this and _Offs* can be used to clip out-of-bounds pixels
				_ClipY = (max[2] - min_y) * scale_y + offs_y,
			}
		end
	end
	
	local function plotPath(plotter, points, rotations, thickness)
		local triangles, vertices = {}, {}
		local p, rot, right_x, right_y
		
		local addQuadIndices_real = addQuadIndices
		local function addQuadIndices()
			-- skipping the first quad this way rather than adding "if i ~= 1 then" to the loop
			addQuadIndices = addQuadIndices_real
		end
		local index = -1
		
		for i=1,#points do
			p, rot = points[i], rotations[i]
			right_x, right_y = thickness*cos(rot), thickness*sin(rot)
			
			insert(vertices, plotter(p[1] - right_x, p[2] - right_y)) -- left
			insert(vertices, plotter(p[1] + right_x, p[2] + right_y)) -- right
			index = index + 2
			
			addQuadIndices(triangles, index - 1, index, index - 3, index - 2)
		end
		return {
			vertextable = vertices,
			indextable = triangles,
		}
	end
	
	local function plotLine(plotter, mesh, a, b, thickness)
		local verts = mesh.vertextable
		local i = #verts - 1
		local left = vec2.setLen(vec2.left(vec2.sub(b, a)), thickness)
		local right = vec2.neg(left)
		insert(verts, plotter(unpack( vec2.add(b, left )) ))
		insert(verts, plotter(unpack( vec2.add(b, right)) ))
		insert(verts, plotter(unpack( vec2.add(a, left )) ))
		insert(verts, plotter(unpack( vec2.add(a, right)) ))
		addQuadIndices(mesh.indextable, i+1, i+2, i+3, i+4)
	end
	local function plotCircle(plotter, mesh, a, radius, points)
		local verts, tris = mesh.vertextable, mesh.indextable
		local index = #verts
		for i=index+1,index+points-2 do
			insert(tris, index)
			insert(tris, i)
			insert(tris, i+1)
		end
		
		local tau = math.tau
		for i=points-1,0,-1 do
			local angle = tau * (i/points)
			insert(verts, plotter(a[1] + radius * math.cos(angle), a[2] + radius * math.sin(angle)))
		end
	end
	
	local red, green = {1, 0, 0}, {0, 1, 0}
	
	local worldPlot, plot_settings = makePlotter({0.5, 0.5}, {-1, -0.5}, 0, {track_bounds.node_min[1], track_bounds.node_min[3]}, {track_bounds.node_max[1], track_bounds.node_max[3]}, true)
	
	local rad = math.rad
	local points, rots, colors = {}, {}, {}
	local n, p
	for i=1,#track,20 do
		n = track[i]
		p = track_vec3s[i]
		insert(points, {p[1], p[3]})
		insert(rots, -rad(n.pan))
		
		insert(colors, n.color)
		insert(colors, n.color)
		--insert(colors, greyscale(n.trafficstrength))
		--insert(colors, greyscale(n.trafficstrength))
		--insert(colors, greyscale(n.color.a))
		--insert(colors, greyscale(n.color.a))
	end
	local plot_table = plotPath(worldPlot, points, rots, options.track_width * 4)
	
	plot_table.colortable = colors
	
	--print("plot_table:\n" .. tableToPrettyString(plot_table))
	--print("plot_settings:\n" .. tableToPrettyString(plot_settings))
	
	CreateObject{
		railoffset = 0,
		floatonwaterwaves = false,
		gameobject = {
			mesh = BuildMesh(plot_table),
			shader = "DBN_assets/HUD/Plot_VertexColorTinted.shader",
			--shadercolors = { _Color = white },
			layer = 13,
			renderqueue = 4500 + 1,
			shadersettings = plot_settings,
		}
	}
	if options.debug_map_camera then
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject = {
				mesh = "DBN_assets/HUD/VisualizeCamera.obj",
				shader = "DBN_assets/HUD/VisualizeCamera.shader",
				--shadercolors = { _Color=white },
				shadersettings = Inherit(plot_settings, { _Scale=0.9995 }),
				layer = 13,
				renderqueue = 4500 + 2,
			}
		}
	else
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject = {
				mesh = "DBN_assets/HUD/VisualizeCameraPos.obj",
				shader = "DBN_assets/HUD/VisualizeCameraPos.shader",
				--shadercolors = { _Color=white },
				shadersettings = Inherit(plot_settings, { _Scale=512 }),
				layer = 13,
				renderqueue = 4500 + 2,
			}
		}
	end
	
	local marks = {
		vertextable = {},
		indextable = {},
		colortable = {},
	}
	
	function markPoint(pos_2d, radius, color)
		plotCircle(worldPlot, marks, pos_2d, radius, 16)
		for i=1,16 do insert(marks.colortable, color) end
	end
	function finalizeMarks()
		plotLine(worldPlot, marks, {track_bounds.node_min[1], 0}, {track_bounds.node_max[1], 0}, 10)
		for i=1,4 do insert(marks.colortable, red) end
		plotLine(worldPlot, marks, {0, track_bounds.node_min[3]}, {0, track_bounds.node_max[3]}, 10)
		for i=1,4 do insert(marks.colortable, green) end
		
		print("marks:\n" .. tableToPrettyString(marks))
		CreateObject{
			railoffset=0,
			floatonwaterwaves = false,
			gameobject={
				mesh=BuildMesh(marks),
				shader="DBN_assets/HUD/Plot_VertexColorTinted.shader",
				layer = 13,
				renderqueue=4500,
				shadersettings = plot_settings,
			}
		}
	end
end

	
	
	-- distant ground texture
	do
	
	local brightness_divisor = 16 / 6
	local quad
	local colors = {}
	--local matrix = {}
	local rotations = {
		makeZRotationMatrix(0, {}),
		makeZRotationMatrix(1/4 * math.tau, {}),
		makeZRotationMatrix(2/4 * math.tau, {}),
		makeZRotationMatrix(3/4 * math.tau, {}),
	}
	local offset_max = 0.1
	local offset_range = offset_max * 2
	for i=1,16 do
		local offset_u, offset_v = random() * offset_range - offset_max, random() * offset_range - offset_max
		--local offset_u, offset_v = 0, 0
		local q = buildXQuad(0.5 + offset_v, -0.5 + offset_v, 0.5 + offset_u, -0.5 + offset_u, -i)
		--makeZRotationMatrix(random() * math.tau, matrix) -- rotate by a random amount from 0 to 360 degrees
		--makeZRotationMatrix(pick(0,1/4,2/4,3/4) * math.tau, matrix) -- rotate by 0, 90, 180, or 270 degrees
		local matrix = rotations[(i-1) % 4 + 1]
		for v=1,4 do
			local vector = q.vertextable[v]
			matrixMultXY(vector, matrix, vector)
		end
		if i == 1 then
			quad = q
			quad.colortable = {}
		else
			buildMerged(quad, q)
		end
		local color = {1,1,1}
		for c=1,4 do table.insert(quad.colortable, color) end
		table.insert(colors, color)
	end
	local quad_mesh = BuildMesh(quad)
	quad.mesh = quad_mesh
	--print("floor mesh:\n" .. tableToPrettyString(quad))
	
	
	local updateGraph
	if debugging then
		local graph_mesh, background_mesh
		graph_mesh, background_mesh, updateGraph = makeBarGraph{
			count = 16,
			size = {1/4, 1/4},
			border = 1/4 * 0.05,
			offset = {3/4 - 1/4 * 0.1, -28.78/32},
			bar_scale = 0.9,
			uv_mode = "slide",
			uv_antistretch = "top"
		}
		CreateObject{
			railoffset = 0, -- gotta keep it in front of the camera so it isn't culled
			floatonwaterwaves = false,
			gameobject={
				mesh = graph_mesh,
				--texture = "graph/bars_vertical.png",
				--texture = "graph/bars_vertical_top.png",
				--texture = "graph/bars_horizontal.png",
				texture = "graph/bars_horizontal_flip.png",
				--texture = "graph/bars_both.png",
				--texture = "graph/bars_both_flip.png",
				texturewrap = "clamp", -- prevents artifacts at the edge of textures not designed to be tiled
				shader = "DBN_assets/HUD/VertexColorTinted.shader",
				renderqueue=4500,
				layer = 13,
			}
		}
		CreateObject{
			railoffset = 0,
			floatonwaterwaves = false,
			gameobject={
				mesh = background_mesh,
				shader = "DBN_assets/HUD/VertexColorTinted.shader",
				renderqueue=4500 - 1,
				layer = 13,
				shadercolors = { _Color=black },
			}
		}
	end
	
	
	--local spectLogDynamic, destroy = spectLogDynamic, destroy
	local GetSongCompletionPercentage, GetSpectrum, BuildMesh = GetSongCompletionPercentage, GetSpectrum, BuildMesh
	local spectrumNormScaleClamp, spectrumNormSmooth = spectrumNormScaleClamp, spectrumNormSmooth
	--local spectrumNormSmooth = spectrumNormSmoothPeaks_Generate(0, 0.125, 1, 1/15) -- min, min_range, max_range, fall_rate
	local spectrumNormSmooth = spectrumNormSmoothPeaks_Generate(0, 0.02, 1, 1/15) -- min, min_range, max_range, fall_rate
	do
		local real_spectrumNormSmooth = spectrumNormSmooth
		function spectrumNormSmooth(spectrum, delta_time)
			local percent = GetSongCompletionPercentage()
			if percent >= 0 then
				spectrumNormSmooth = real_spectrumNormSmooth
				return real_spectrumNormSmooth(spectrum, delta_time)
			end
			if percent >= -0.02 then -- -0.0075
				percent = percent / -0.02 * 0.25
			else
				percent = 0.25
			end
			for i=1,16 do spectrum[i] = percent end
			return 0.02
		end
	end
	local spectrum = {}
	onUpdateSpectrum(function(spectrum_orig, delta_time)
		for i=1,16 do spectrum[i] = spectrum_orig[i] end
		--spectrumLog(spectrum)
		--spectrumNormDynamic(spectrum, 0.1, 1)
		--spectrumNormRemap(spectrum, 0.75, 1)
		--spectrumNormClamp(spectrum)
		
		--spectrumDecibel(spectrum)
		--spectrumNormDynamic(spectrum, 25, 0)
		
		-- using the raw spectrum like this is the equivalent of squaring the linear scale
		-- it has a strong difference between lows and highs, which is perfect for the floor intensity
		-- this has less noisy variation than doing logarithmic dynamic range so we avoid killing epileptics
		spectrumNormScaleClamp(spectrum, 0.25, 0)
		local range = spectrumNormSmooth(spectrum, delta_time)
		
		--spectrumLinear(spectrum)
		--spectrumNormRemap(spectrum, 0, 0.5)
		
		for i=1,16 do
			local c = colors[i]
			local v = spectrum[i] --/ brightness_divisor
			c[1], c[2], c[3] = v,v,v
		end
		BuildMesh(quad)
		
		--spectrum[16] = range
		--spectrum[15] = delta_time * 50
		if debugging then updateGraph(spectrum) end
	end)
	

	
	local transform = {
		
		--pos={track_bounds.node_center[1], track_bounds.node_min[2] - 1000, track_bounds.node_center[3]},
		pos={track_bounds.node_center[1], scene_floor, track_bounds.node_center[3]},
		scale={scene_radius * 2, scene_radius * 2, scene_floor_thickness / 16},
		
		rot={-90,random() * 360,0},
	}
	local layer = 13 -- don't glow
	
	CreateObject{
		name="Ground City",
		gameobject={
			mesh="quad.obj", -- 1x1 square
			transform=transform,
			renderqueue=999,
			layer=13,
			texture="DBN_assets/BG/city.png",
			texturewrap = "clamp",
			shader=ifinvert("DBN_assets/UnlitTintedMultInvert.shader", "DBN_assets/BG/SkySceneAddSmooth.shader"),
			shadercolors={
				_Color = {colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=ifinvert(0.5, 6)},
			},
		}
	}
	
	CreateObject{
		name="Ground Details",
		gameobject={
			mesh=quad_mesh, --"quad.obj", -- 1x1 square
			transform=transform,
			renderqueue=999,
			layer=13,
			texture="DBN_assets/BG/city.png",
			texturewrap = "clamp",
			shader=ifinvert("DBN_assets/UnlitTintedMultInvert.shader", "DBN_assets/BG/SkySceneAddSmooth.shader"),
			shadercolors={
				--_Color = {colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=ifinvert(0.5, 6)},
				_Color = {colorsource="highway", scaletype="intensity", minscaler=6, maxscaler=6},
				--_Color = {colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1},
				_ColorScale = greyscale(1/brightness_divisor),
			},
		}
	}
	
	
	-- a giant black sheet to cover the entire bottom half of the skybox, forming a horizon
	diameter = 1000000000
	transform.scale = {diameter,diameter,1}
	CreateObject{
		name="Ground base",
		gameobject={
			mesh="quad.obj", -- 1x1 square
			transform=transform,
			renderqueue=999 - 1,
			layer=fif(options.glow, 18, 13), -- gotta get behind the skywires
			shader="DBN_assets/BG/SkyScene.shader",
			shadercolors={ _Color = background },
		}
	}
end
	
	--[[
	CreateObject{
		railoffset=0,
		floatonwaterwaves = false,
		gameobject={
			mesh=BuildMesh{mesh="submeshtest.obj", submeshesWhenCombining=true},
			transform={
				pos = {4, 1, 0},
			},
			materials = {
				{
					shader="VertexColorUnlitTinted",
					shadercolors={ _Color = {0,0.5,1} },
				},{
					shader="VertexColorUnlitTinted",
					shadercolors={ _Color = {1,0.5,0} },
				}
			}
		}
	}
	--]]

	
	if options.enable_structures then
		---------------------------------------------------------------------------------------------------------------------------
		-- Define the properties of the individual structures
		---------------------------------------------------------------------------------------------------------------------------
		
local structs
do
	local surface1 = {
		shader = "DBN_assets/BG/struct_simple" .. options.glowing_suffix,
		layer = 11,
		textures = {
			_DetailTex = "DBN_assets/BG/surface4.png",
			_FlowTex = "DBN_assets/BG/flow3.png",
		},
		texturewrap = "clamp",
		shadercolors = {
			-- BREF's "colors" setting lets us specify per-object shader settings which we will be exploiting
			-- since it only works on _Color we will send actual color settings through _HighwayColor instead
			_HighwayColor = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1, param="_Intensity", paramMin=1, paramMax=2.25 }
		},
		shadersettings = {
			_SpinSpeed = 0,

			_ScaleDetailU = 2,
			_ScaleDetailV = 2,
			
			_ScaleFlowU = 4,
			_ScaleFlowV = 4,
			
			--[[
			_ScrollFlowU = 0.001,
			_ScrollFlowV = 0.005,
			
			_ScrollFlow2U = -0.001,
			_ScrollFlow2V = 0.008,
			--]]
			--[
			
			_ScrollFlowU = 0.005 * 2,
			_ScrollFlowV = 0.005 * 2,
			
			_ScrollFlow2U = -0.001 * 2,
			_ScrollFlow2V = 0.008 * 2,
			
			--]]
		},
	}
	
	structs = {
		test_track_marker = {
			object = {
				mesh = "cylinder.obj",
				scale = {1,100,1},
				shader = "DBN_assets/BG/struct_simple" .. options.glowing_suffix,
				textures = { _LightmapTex = "White.png" },
				shadercolors = { _HighwayColor = {0, 1, 0, 0} },
				texture = "White.png",
				layer = 13,
			},
			side_radius = options.track_width * 16,
			debug_color = {0, 1, 0},
			near_chance = 0,
			far_chance = 0,
		},
		test_track_marker2 = {
			object = {
				mesh = "cylinder.obj",
				scale = {1,100,1},
				shader = "DBN_assets/BG/struct_simple" .. options.glowing_suffix,
				textures = { _LightmapTex = "White.png" },
				shadercolors = { _HighwayColor = {1, 0, 0, 0} },
				texture = "White.png",
				layer = 13,
			},
			side_radius = options.track_width * 16,
			debug_color = {1, 0, 0},
			near_chance = 0,
			far_chance = 0,
		},
		flare = {
			object = Inherit(surface1, {
				mesh = "DBN_assets/BG/flare/body.obj",
				scale = {50,50,50},
				textures = {
					_LightmapTex = "DBN_assets/BG/flare/lightmap.png",
				},
			}),
			
			top = 2.05 * 50,
			bottom = -11 * 50,
			side_radius = vec2.len{1.8, 1.8} * 50,
			debug_color = {0.5, 0.5, 0.5, 0.5},
			
			stretch = {
				mid = "DBN_assets/BG/flare/mid.obj",
				base = "DBN_assets/BG/flare/base.obj",
				attach = -1, -- 1 == ceiling, -1 == floor
				scale = 50,
			},
			
			-- structures with a higher chance get more entries in the lottery
			near_chance = 2,
			far_chance = 8,
			
			--[=[
			thrusters = {
				--shader = "DBN_assets/thruster_jet.shader",
				shader = "DBN_assets/thruster_beam.shader",
				--[[
				crossSectionShape={
					{ 0.58713,  1.78340, 0},
					{ 0.58713, -1.78340, 0},
					{ 0.00000, -2.21872, 0},
					{-0.58713, -1.78340, 0},
					{-0.58713,  1.78340, 0},
					{ 0.00000,  2.26669, 0},
				},
				perShapeNodeColorScalers = clonesArray(1,6), -- can't be omitted
				--]]
				crossSectionShape = makeCylinderThruster(1, 8),
				perShapeNodeColorScalers = clonesArray(1,8), -- can't be omitted
				extrusions = 32,
				stretch = -1/32,
				updateseconds = 0.025,
				sizefalloff = 0,
				layer = 11,
				instances={
					{pos={0, 3.1*50, 0},rot={90, 0, 0},scale={150, 150, 1}},
				}
			},
			
			-- attached thrusters should be defined with a height of 1
			-- each number in this array corresponds to an instance, and uses 1 or -1 like stretch.attach
			thrusters_attach = {1}, --]=]
		},
		hex = {
			object = Inherit(surface1, {
				mesh = "DBN_assets/BG/hex/body.obj",
				scale = {25,25,25},
				textures = {
					_LightmapTex = "DBN_assets/BG/hex/lightmap.png",
				},
				shadersettings = {
					_SpinSpeed = math.tau/64,
				},
			}),

			-- the name of another structure to place wherever this one is placed
			-- parent+includes group collisions are handled properly
			includes = {"hex_visualizer"},
			
			top = 30.6527 * 25,
			bottom = -30.6527 * 25,
			side_radius = 28 * 25,
			side_radius_inner = 14 * 25,
			debug_color = {0.5, 0.5, 0.5, 0.5},
			
			near_chance = 3,
			far_chance = 1,
			
			--[ [
			thrusters = {
				material = BuildMaterial{
					shader = "DBN_assets/thruster_beam.shader",
					shadersettings = { _Alpha = 1/32 },
				},
				crossSectionShape=makeCylinderThruster(0.9*2, 8),
				perShapeNodeColorScalers = clonesArray(1,8),
				extrusions = 4,
				stretch = -1/4,
				updateseconds = 0.025,
				layer = 11,
				instances={
					{pos={0, 0, 0},rot={ 90, 0, 0},scale={25, 25, 1}},
					{pos={0, 0, 0},rot={-90, 0, 0},scale={25, 25, 1}},
				}
			},
			thrusters_attach = {1, -1}, --]]
		},
		hex_visualizer = {
			object = {
				mesh = hex_mesh,
				scale = {25,50*0.6,25},
				texture = "DBN_assets/BG/hex/wave.png",
				texturewrap = "clamp",
				shader = "DBN_assets/BG/ring_visualizer.shader",
				layer = 13,
				shadercolors = {
					_Colour = greyscale(1,0),
				},
				shadersettings = {
					_SpinSpeed = math.tau/64,
				},
			},
			
			-- if dimensions are omitted then no collision checks are performed
			near_chance = 0,
			far_chance = 0,
		},
		pyramid = {
			object = Inherit(surface1, {
				mesh = "DBN_assets/BG/pyramid/body.obj",
				shader = "DBN_assets/BG/struct" .. options.glowing_suffix,
				scale = {400,400,400},
				textures = {
					_SurfacemapTex = "DBN_assets/BG/pyramid/body.png",
				},
				shadercolors = {
					_HighwayColor = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1, param="_Power", paramMin=1, paramMax=2 }
				},
				shadersettings = {
					_ScaleFlowU = 1,
					_ScaleFlowV = 2,

					
					_ScrollFlowU = 0.005 * 0.5,
					_ScrollFlowV = 0.005 * 0.5,
					
					_ScrollFlow2U = -0.001 * 0.5,
					_ScrollFlow2V = 0.008 * 0.5,
					
				},
			}),
			
			top = 1.25 * 400,
			bottom = -0.06674 * 400,
			side_radius = 1.41421 * 400,
			debug_color = {0.5, 0.5, 0.5, 0.5},
			
			near_chance = 2,
			far_chance = 1,

			onMaterialBuilt = function(mat)
				local UpdateShaderSettings = UpdateShaderSettings
				mat = {
					material = mat,
					shadersettings = {},
				}
				onUpdate(function(delta_time, location, strafe, jump_height, intensity)
					if delta_time == 0 then return end
					mat.shadersettings._Intensity = intensity * 2
					UpdateShaderSettings(mat)
				end)
			end,
			
			--[[thrusters = {
				shader = "DBN_assets/thruster_jet.shader",
				crossSectionShape=makeCylinderThruster(1, 8),
				perShapeNodeColorScalers = clonesArray(1,8),
				extrusions = 16,
				stretch = -1/16,
				updateseconds = 0.025,
				layer = 11,
				instances={
					{pos={0, 1.2 * 400, 0},rot={-90, 0, 0},scale={100, 100, 1}},
				}
			},
			thrusters_attach = {-1},]]
		},
	}
end

-- collisions as axis-aligned vertical cylinders
do
	local track, track_vec3s, node_count, node_skip, sqrt = track, track_vec3s, #track, 20, math.sqrt
	for st,s in pairs(structs) do
		local top, bottom, side_radius_sq, side_radius_inner = s.top, s.bottom, s.side_radius, s.side_radius_inner
		local outer_top, outer_bottom, side_radius_range -- put all variables in the same stack
		if side_radius_sq and top and bottom then -- no "continue" is bs
			side_radius_sq = (side_radius_sq + node_bounds.side)^2
			outer_top    = top    + node_bounds.up
			outer_bottom = bottom - node_bounds.down
			if s.stretch then
				if s.stretch.attach < 0
				then bottom = nil
				else top    = nil  end
			end
			for i,attach in ipairs(s.thrusters_attach or {}) do
				if     attach < 0 then bottom = nil
				elseif attach > 0 then top    = nil  end
			end


			if top == nil and bottom == nil then
				-- infinite struct height, force is always purely horizontal
				function s.collider(x,y,z)
					local count, forcex, forcez = 0,0,0
					local pos, dX,dZ, dist_squared, len -- pre-allocate and reuse the loop's temporary variables
					for i=1,node_count,node_skip do
						pos = track_vec3s[i]
						dX,dZ = x - pos[1], z - pos[3] -- delta X Z
						dist_squared = dX^2 + dZ^2
						if dist_squared < side_radius_sq then
							count = count + 1
							-- have the node exert force on the struct
							len = sqrt(dist_squared)
							if len ~= 0 then
								forcex = forcex + dX/len
								forcez = forcez + dZ/len
							end
						end
					end
					return count, forcex, 0, forcez
				end


			else
				if top    == nil then outer_top   , top    =  math.huge,  math.huge end
				if bottom == nil then outer_bottom, bottom = -math.huge, -math.huge end

				if side_radius_inner == nil then
					function s.collider(x,y,z)
						local top, bottom, outer_top, outer_bottom = y + top, y + bottom, y + outer_top, y + outer_bottom
						local count, forcex, forcey, forcez = 0,0,0,0
						local pos, pY, dX,dZ, dist_squared, len -- pre-allocate and reuse the loop's temporary variables
						for i=1,node_count,node_skip do
							pos = track_vec3s[i]
							pY = pos[2] -- position Y
							if pY > outer_bottom and pY < outer_top then
								dX,dZ = x - pos[1], z - pos[3] -- delta X Z
								dist_squared = dX^2 + dZ^2
								if dist_squared < side_radius_sq then
									count = count + 1
									-- have the node exert force on the struct
									-- for this, the node center is a point and the struct is a vertical line segment
									-- reuse pY rather than allocating another variable. just pretend it's named dY now
									if     pY < bottom then pY = y - bottom
									elseif pY > top    then pY = y - top
									else pY = 0  end
									len = sqrt(dist_squared + pY^2)
									if len ~= 0 then
										forcex = forcex + dX/len
										forcey = forcey + pY/len
										forcez = forcez + dZ/len
									end
								end
							end
						end
						return count, forcex, forcey, forcez
					end


				else
					side_radius_range = s.side_radius - side_radius_inner
					function s.collider(x,y,z)
						local top, bottom, outer_top, outer_bottom = y + top, y + bottom, y + outer_top, y + outer_bottom
						local count, forcex, forcey, forcez = 0,0,0,0
						local pos, pY, dX,dZ, dist_squared, side_radius_sq, len -- pre-allocate and reuse the loop's temporary variables
						for i=1,node_count,node_skip do
							pos = track_vec3s[i]
							pY = pos[2] -- position Y
							if pY > outer_bottom and pY < outer_top then
								dX,dZ = x - pos[1], z - pos[3] -- delta X Z
								dist_squared = dX^2 + dZ^2
								side_radius_sq = (side_radius_range * track[i].color.a + side_radius_inner)^2 -- see GetTrack section in init.luainc
								if dist_squared < side_radius_sq then
									count = count + 1
									-- have the node exert force on the struct
									-- for this, the node center is a point and the struct is a vertical line segment
									-- reuse pY rather than allocating another variable. just pretend it's named dY now
									if     pY < bottom then pY = y - bottom
									elseif pY > top    then pY = y - top
									else pY = 0  end
									len = sqrt(dist_squared + pY^2)
									if len ~= 0 then
										forcex = forcex + dX/len
										forcey = forcey + pY/len
										forcez = forcez + dZ/len
									end
								end
							end
						end
						return count, forcex, forcey, forcez
					end


				end -- else side_radius_inner == nil
			end -- else top == nil and bottom == nil
		end -- if side_radius_sq
	end -- for pairs(structs)
end

-- encapsulate all code for actually creating the structs into these functions
local canPlaceStruct, placeStruct, finalizeStructs
do
	-- thrusters
	local placeThruster, finalizeThrusters
	do
		local thruster_instances = {}
		for st,s in pairs(structs) do thruster_instances[st] = {} end -- TODO: try removing ",s"

		function placeThruster(s, st, pos, rot)

			local instances = s.thrusters.instances
			local instance, scale, inst_pos
			for i=1,#instances do
				instance = instances[i]
				scale = instance.scale
				inst_pos = vec3.add(pos, instance.pos)
				if s.thrusters_attach[i] ~= 0 then
					local dist
					if s.thrusters_attach[i] > 0 then
						dist = scene_ceiling + scene_ceiling_thickness * 5 - inst_pos[2]
					else
						dist = inst_pos[2] - scene_floor
					end
					scale = {scale[1], scale[2], dist * scale[3]}
				end
				table.insert(thruster_instances[st], {pos=inst_pos, rot=vec3.add(rot, instance.rot), scale=scale})
			end
		end
		function finalizeThrusters(debug_output)
			if debugging then table.insert(debug_output, "thruster_instances = {") end
			
			for st,s in pairs(structs) do
				if #thruster_instances[st] > 0 then
					local thrusters = s.thrusters
					s.thrusters = nil -- clear it out for safety because we're modifying the original
					thrusters.instances = thruster_instances[st]
					
					CreateObject{
						gameobject = {
							thrusters=thrusters,
							layer = 13,
						}
					}
					if debugging then table.insert(debug_output, tableToPrettyString(thruster_instances[st]) .. ",") end
				end
			end

			if debugging then table.insert(debug_output, "}") end
		end
	end
	
	local center = {scene_center[1], scene_center[3]}
	function canPlaceStruct(st, pos)
		--print("placeStruct(" .. st .. ", " .. tableToString(pos) .. ")")
		local s = structs[st]
		if s.top == nil then return true end

		--scene_radius, scene_floor, scene_ceiling, scene_height, scene_center
		local bottom_dist = pos[2] + s.bottom - scene_floor
		local top_dist = scene_ceiling - (pos[2] + s.top)
		if bottom_dist < 0 or top_dist < 0 then return false end
		if vec2.dist(center, {pos[1], pos[3]}) > scene_radius then return false end
	
		-- TODO: collision checks
		-- considered basic visibility checks by generating a normal for the direction each node is facing and checking which side of the normal it's on
		-- but thats a ton of Lua number crunching added to the loading screen for a probably small benefit
		-- edit: that new placeStructNearNode function will be used for a lot of the positions and it guarantees this kind of visibility, making this even less important

		local collider = s.collider
		if collider then
			local count = collider(pos[1], pos[2], pos[3])
			if count ~= 0 then return false end
		end

		if s.includes then
			for k,v in ipairs(s.includes) do
				if not canPlaceStruct(v, pos) then return false end
			end
		end
		return true
	end

	--@include "scene_struct_spawn_individual.luainc"
	--@include "scene_struct_spawn_batch.luainc"
	--@include "scene_struct_spawn_prototyped.luainc"
	--[[
	CreateObject + BuildMaterial/BuildMesh spawn method
	Pull artist mesh/material settings out of object settings and feed them through BuildMesh/BuildMaterial, then do simple CreateObject spawning.
	pros:
		enables draw call batching
		individual objects can have arbitrary transformations
	cons:
		has to modify artist settings and so must be updated to allow use of new API features
		doesn't support standard mesh morphing
]]

local spawnStruct, finalizeSpawnedStructs
do
	local BuildMaterial = BuildMaterial
	local objects = {}
	for st,s in pairs(structs) do
		local go = s.object
		go.transform = {
			pos = nil,
			rot = {0, 0, 0},
			scale = shallowCopy(go.scale),
		}

		-- build all unbuilt materials
		if go.material == nil then
			if go.materials ~= nil then
				local materials = go.materials
				for k,v in ipairs(materials) do
					if type(v) == "table" then
						materials[k] = BuildMaterial(v)
					end
				end
				if s.onMaterialBuilt then s.onMaterialBuilt(go.materials) end
			else
				local material = {
					shader         = go.shader,
					diffuseinair   = go.diffuseinair,
					renderqueue    = go.renderqueue,
					shadersettings = go.shadersettings,
					shadercolors   = go.shadercolors,
					texturewrap    = go.texturewrap,
					usemipmaps     = go.usemipmaps,
					texturefilter  = go.texturefilter,
					texture        = go.texture,
					textures       = go.textures,
				}
				if next(material) ~= nil then -- table is not empty
					go.material = BuildMaterial(material)
					if s.onMaterialBuilt then s.onMaterialBuilt(go.material) end
					for key in pairs(material) do go[key] = nil end
				end
			end
		end

		-- build all unbuilt meshes
		if type(go.mesh) == "string" then
			if go.meshmorph ~= nil then error("This struct spawner cannot handle mesh morphing.") end
			local mesh = {
				mesh                   = go.mesh,
				calculateNormals       = go.recalculateNormals,
				calculateTangents      = go.calculateTangents,
				submeshesWhenCombining = go.submeshesWhenCombining,
			}
			if mesh.calculateNormals == nil then mesh.calculateNormals = go.calculateNormals end
			if next(mesh) ~= nil then
				for key in pairs(mesh) do go[key] = nil end
				go.recalculateNormals = nil
				go.mesh = BuildMesh(mesh)
			end
		end

		objects[st] = go
		if s.stretch then
			objects[st .. "#mid"] = Inherit(go, { mesh = s.stretch.mid })
			objects[st .. "#base"] = Inherit(go, { mesh = s.stretch.base })
		end
	end

	function spawnStruct(st, s, pos, y_rot, y_scale)
		local go = objects[st]
		
		go.transform.pos = pos
		go.transform.rot[2] = y_rot
		go.transform.scale[2] = y_scale * go.scale[2]

		CreateObject{
			gameobject=go
		}
	end

	function finalizeSpawnedStructs(debug_output)
		-- nothing to do here except maybe output a list built up by all the spawnStruct calls
	end
end


	function placeStruct(st, pos)
		--print("placeStruct(" .. st .. ", " .. tableToString(pos) .. ")")
		local s = structs[st]

		if debugging and s.debug_color then markPoint({pos[1], pos[3]}, s.side_radius, s.debug_color) end
		
		local rot = random() * 360

		spawnStruct(st, s, pos, rot, 1)
		
		local stretch = s.stretch
		if stretch then
			local distance, y
			if stretch.attach < 0 then
				distance = (pos[2] - scene_floor) + (s.bottom + stretch.scale)
				y = scene_floor - s.bottom
			else
				distance = (scene_ceiling - pos[2]) - (s.top - stretch.scale)
				y = scene_ceiling - s.top
			end
			
			local scale = s.object.scale or {1,1,1}
			spawnStruct(st .. "#mid", s, pos, rot, distance / scale[2])
			spawnStruct(st .. "#base", s, {pos[1], y, pos[3]}, rot, 1)
		end

		if s.thrusters then placeThruster(s, st, pos, {0, math.deg(rot), 0}) end

		if s.includes then
			for k,v in ipairs(s.includes) do placeStruct(v, pos) end
		end
	end
	


	function finalizeStructs()
	
		-- where doing it man
		-- where MAKING THIS HAPEN

		local debug_output
		if debugging then debug_output = {} end
		
		finalizeSpawnedStructs(debug_output)
		finalizeThrusters(debug_output)
		
		if debugging then
			print(table.concat(debug_output, "\n"))
			finalizeMarks()
		end
		
		placeStruct, finalizeStructs = nil, nil -- can only do this once
	end
end

-- function to choose a structure randomly
local pickStructNear, pickStructFar
do
	local function makePicker(chance_key)
		local lottery = {}
		for st,s in pairs(structs) do
			for i=1,s[chance_key] do table.insert(lottery, st) end
		end
		if #lottery == 0 then return nil end
		return function()
			return lottery[random(1,#lottery)]
		end
	end
	pickStructNear, pickStructFar = makePicker("near_chance"), makePicker("far_chance")
end
		
		---------------------------------------------------------------------------------------------------------------------------
		-- Decide where the structures will go
		---------------------------------------------------------------------------------------------------------------------------
		do
	local placeStructNearNode
	do
		local rotation = {}
		local track_width = options.track_width
		function placeStructNearNode(st, node)
			-- place a struct in the upper half of its forward view
			makeRotationMatrix(node.pan, node.tilt, node.roll, rotation)
			local pos = vec3.fromApi(node.pos)
			local offset, offset_pos
			repeat
				offset = {
					gaussRand(0, 400),
					gaussRandHalf(0, 500),
					gaussRandLowerHalf(0, 1000),
				}
				offset[1] = offset[1] + sign(offset[1]) * (track_width + structs[st].side_radius)
				--offset = {0, 0, 2000}
				matrixMult(offset, rotation, offset)
				offset_pos = vec3.add(pos, offset)
			until canPlaceStruct(st, offset_pos)
			placeStruct(st, offset_pos)
		end
	end
	local placeStructDistant
	do
		local vertical_min = track_bounds.node_min[2]
		local vertical_range = (track_bounds.node_max[2] - track_bounds.node_min[2]) * 0.75
		function placeStructDistant(st)
			local pos
			repeat
				pos = {
					scene_center[1] + gaussRandRange(0, scene_inner_radius, -scene_radius, scene_radius),
					vertical_min + random() * vertical_range,
					scene_center[3] + gaussRandRange(0, scene_inner_radius, -scene_radius, scene_radius),
				}
			until canPlaceStruct(st, pos)
			placeStruct(st, pos)
		end
	end

	
	if debugging then print("NODE COUNT: " .. #track) end
	-- 5273: 10 near, 20 far

	if pickStructNear then
		for i=1,math.ceil(#track/500) do
			local st = pickStructNear()
			placeStructNearNode(st, track[random(#track)])
		end
	end
	if pickStructFar then
		for i=1,math.ceil(#track/250) do
			local st = pickStructFar()
			placeStructDistant(st)
		end
	end
	--placeStruct(pickStruct(), vec3.fromApi(track[math.floor(#track * 3/10)].pos))
	--placeStruct(pickStruct(), vec3.fromApi(track[#track].pos))
	
	--[[for i=1,50 do
		local st = pickStruct()
		placeStructNearNode(st, track[40])
	end]]
	
	--[[
		analyze the sine wave-shaped track
		get the delta rotation for each node and split up the track at points where the delta changes from positive to negative and vice versa
		in each segment, find the node with the greatest delta
		the split nodes are the middle of the sine wave and the max delta nodes are the high/low points
	]]
	do
		-- some extra nodes with negative time are added to the start of the track. skip them.
		local first_real_node
		for i=1,#track do
			if track[i].seconds >= 0 then
				first_real_node = i
				break
			end
		end
		if first_real_node == nil then error("negative length song") end
		if first_real_node == 1 then first_real_node = 2 end


		--local debug = {}
		local node, i_start, pan, pan_prev, pan_delta, sign, sign_start

		i_start, pan, pan_prev = first_real_node-1, track[first_real_node].pan, track[first_real_node-1].pan
		if pan_prev - pan >= 0 then sign_start = 1 else sign_start = -1 end

		local pan_delta_max, pan_delta_max_i = -1, i_start

		for i=first_real_node,#track do
			node = track[i]
			pan = node.pan

			if not node.funkyrot then
				pan_delta = pan - pan_prev
				if pan_delta >= 0 then
					sign = 1
				else
					sign = -1
					pan_delta = -pan_delta
				end
				if sign ~= sign_start then
					if debugging then
						placeStruct("test_track_marker2", track_vec3s[i])
						if vec3.distSquared(track_vec3s[i], track_vec3s[pan_delta_max_i]) > 10^2 then
							placeStruct("test_track_marker", track_vec3s[pan_delta_max_i])
						end
					end
					pan_delta_max, pan_delta_max_i, i_start, sign_start = pan_delta, i, i, sign
				elseif pan_delta > pan_delta_max then
					pan_delta_max, pan_delta_max_i = pan_delta, i
				end

				--debug[i] = time_delta --pan_delta

				--dist = dist + 
			end
			pan_prev = pan
		end
		if vec3.dist(track_vec3s[#track], track_vec3s[pan_delta_max_i]) > 10 then
			if debugging then placeStruct("test_track_marker", track_vec3s[pan_delta_max_i]) end
		end
		--print("!~ " .. tableToPrettyString(debug))
	end



	local potentials = {}
	local function placePotential(st, pos)
		table.insert(potentials, {st=st, pos=pos})
	end
end

		finalizeStructs()
	end
	
end




-- create symmetrical rails
-- call this with the arguments needed to create one side's rail and it will make the opposite side too
-- it expects vectors in {0,0} format. {x=0,y=0} format is not supported. (replace "[1]" with ".x" in the code if you prefer that style)
-- "nondestructive" is an optional argument. you can still call the function like createSymRails{ ... } if you don't need it
-- pass "true" to "nondestructive" if you are passing a table that you are going to reuse
function createSymRails(table, nondestructive)

	-- first, optimize the rails to share a single material
	if table.shader then -- without a shader it's guaranteed to be using some sort of pre-made material already
		if nondestructive and not (table.allowfullmaterialoptions and table.material) then
			table = shallowCopy(table)
		end
		if not table.allowfullmaterialoptions and table.shader then
			table.allowfullmaterialoptions = true
			if not table.renderqueue then table.renderqueue = 2000 end
			if not table.shadercolors then table.shadercolors = {} end
			if not table.shadercolors._Color then table.shadercolors._Color = table.color or {1,1,1,1} end
			table.material = nil -- this would have been ignored
		end
		if not table.material then
			table.material = BuildMaterial(table)
			-- can leave the options in place, they will be ignored now that material is set
			for i,key in ipairs({'shader','diffuseinair','renderqueue','shadersettings','shadercolors','texturewrap','usemipmaps','texturefilter','texture','textures'}) do
				--table[key] = nil
			end
		end
	end

	CreateRail(table)

	if nondestructive then
		table = shallowCopy(table)
		table.positionOffset = shallowCopy(table.positionOffset)
	end
	local flip = fif(nondestructive, copyFlipArray, flipArray)
	table.positionOffset[1] = -table.positionOffset[1]
	table.crossSectionShape = flip(table.crossSectionShape)
	for i=1,#table.crossSectionShape do
		table.crossSectionShape[i][1] = -table.crossSectionShape[i][1]
	end
	if table.perShapeNodeColorScalers then table.perShapeNodeColorScalers = flip(table.perShapeNodeColorScalers) end
	if table.perShapeNodeColors       then table.perShapeNodeColors       = flip(table.perShapeNodeColors) end
	if table.perShapeNodeNormals      then table.perShapeNodeNormals      = flip(table.perShapeNodeNormals) end
	
	CreateRail(table)
end

-- makes a square crossSectionShape
function makeBox(half_width)
	local L = -half_width
	local H = half_width
	return {
		{L,L},
		{L,H},
		{H,H},
		{H,L},
	}
end

-- makes a circular crossSectionShape
-- "points" is the number of vertices (level of detail) and should probably vary depending on the quality setting
function makeCylinder(radius, points, inside_out)
	local min, max, step
	if inside_out then min, max, step = 0, points-1, 1
	else min, max, step = points-1, 0, -1 end
	
	local shape = {}
	local tau = math.tau
	for i=min,max,step do
		local angle = tau * (i/points)
		table.insert(shape, {radius * math.cos(angle), radius * math.sin(angle)})
	end
	return shape
end

--[[
	as2 ignores fullfuture=true if the rail would have over 63500 vertices (which is a good idea. even if it didn't, this function would be good practice)
	this function generates a value for CreateRail{stretch} which causes fullfuture to always work
	crossSectionShape_count should be the number of points in crossSectionShape (e.g. crossSectionShape={{0,0},{1,1}} has 2 points)
	stretch=X tells it to only create a vertex every X nodes. effectively, we degrade the rail's quality in response to the song's length
	it may be useful to multiply the result by some GetQualityLevel()-dependent number. especially if you're using more than one fullfuture rail
	Dylan: it'd be nice if there was a way to have the quality increase for sections of the track close to the player
]]
function fullfutureStretch(crossSectionShape_count)
	return math.ceil(#track * crossSectionShape_count / 63500)
end

---------------------------------------------------------------------------------------------------------------------------
-- Lane dividers and shoulder lines
---------------------------------------------------------------------------------------------------------------------------
if not options.track_surface and (#options.lane_dividers ~= 0 or #options.shoulder_lines ~= 0) then
	local rail
	if options.water then
		rail = {
			positionOffset = {nil, 0.1},
			crossSectionShape = {
				{-0.07,0},
				{ 0.07,0},
			},
			wrapnodeshape = options.trackless,
			nodeskip = 2,
			colorMode = "static",
			layer = options.glowing_layer,
			allowfullmaterialoptions = true,
			material = BuildMaterial{
				shader = "assets/Solid.shader",
				shadercolors = { _Color = fif(options.glow_foreground, colorAlpha(foreground, 0), foreground) },
				renderqueue = 2000,
			},
		}
	else
		rail = {
			positionOffset = {nil, 0},
			crossSectionShape = {
				{-1.00, 0},
				{ 1.00, 0},
			},
			wrapnodeshape = false,
			colorMode = "aurora",
			layer = 13,
			allowfullmaterialoptions = true,
			material = BuildMaterial{
				shader = "DBN_assets/FG/LaneMarker.shader",
				texture = "DBN_assets/FG/LaneMarker-azure.png",
				shadercolors = {
					_Color = {colorsource="highway", scaletype="intensity", minscaler=0.33, maxscaler=0.33},
				},
				shadersettings = {
					_AuroraScale=2,
				},
				renderqueue = 3000,
			}
		}
	end
	
	for i=1,#options.lane_dividers do
		rail.positionOffset[1] = options.lane_dividers[i]
		CreateRail(rail)
	end
	
	rail.nodeskip = nil
	for i=1,#options.shoulder_lines do
		rail.positionOffset[1] = options.shoulder_lines[i]
		CreateRail(rail)
	end
end

---------------------------------------------------------------------------------------------------------------------------
-- Track surface
---------------------------------------------------------------------------------------------------------------------------
if options.track_surface then
	local track_width = options.track_width

	createSymRails{
		positionOffset = {track_width, 0},
		crossSectionShape = {
			{0,    0},
			{0,    0.25},
			{0.25, 0.25},
			{0.25, -1},
		},
		wrapnodeshape = false,
		renderqueue = 2000,
		layer = 13,
		shader = "VertexColorUnlitTinted",
		colorMode = "highway",
		color = greyscale(1/8, 0),
	}
	local surface_rail = {
		positionOffset = {0, 0},
		crossSectionShape = {
			{nil, 0},
			{nil, 0},
		},
		wrapnodeshape = false,
		colorMode = "highway",
		alphafromintensity = true,
		layer = 13,
		allowfullmaterialoptions = true,
		material = { -- will BuildMaterial this further down
			shader = "DBN_assets/FG/TrackSurface.shader",
			renderqueue = 2000,
			shadersettings = {
				_HighwayScale = 1,
				_IntensityScale = 1/2,
			},
			shadercolors = {
				_Color = greyscale(1/5, 0),
			},
		},
	}
	local divider_rail = {
		positionOffset = {0, 0},
		crossSectionShape = {
			-- should have a width of 1. the shape will be scaled horizontally for each rail
			{-0.5,  0},
			{-0.5, -1},
			{ 0.5, -1},
			{ 0.5,  0},
		},
		wrapnodeshape = false,
		colorMode = "highway",
		alphafromintensity = true,
		layer = 13,
		allowfullmaterialoptions = true,
		material = { -- will BuildMaterial this further down
			shader = "DBN_assets/FG/TrackLaneDivider" .. options.glowing_suffix,
			renderqueue = 2000,
			shadersettings = {
				_HighwayScale = 1,
				_IntensityScale = 2,
				_AuroraScale = 2,
			},
			texture = "DBN_assets/FG/TrackLaneDivider.png",
			texturewrap = "clamp",
		},
		textureMode = "wraparoundcontinuous",
		texturemirroreddowntrack = false,
	}

	if options.mirror then
		local rail = Inherit(surface_rail, {
			crossSectionShape = {
				{-track_width, 0},
				{ track_width, 0},
			},
			material = {
				shader = "DBN_assets/FG/TrackSurface_MirrorPost.shader",
				renderqueue = 2000 - 1,
				textures = {
					--_MainTex = "../stadium/roadsideUnlit.png", --"White.png",
					_SurfaceTex = "DBN_assets/FG/TrackSurface_Mirror.png",
				},
				shadercolors = {
					_SpecularColor = greyscale(1/3, 1),
				},
			},
			texturemirroreddowntrack = false,
			--textureTrackLength = 9001, -- this setting does absolutely nothing
		})
		rail.material = BuildMaterial(rail.material)
		CreateRail(rail)

		CreateRail{
			positionOffset = {0, 0},
			crossSectionShape = {
				{-track_width, 0},
				{-track_width, -1.5},
				{ track_width, -1.5},
				{ track_width, 0},
			},
			wrapnodeshape = true,
			layer = 13,
			shader = "assets/DepthMask.shader",
			renderqueue = 2000 - 5,
			colorMode = "static",
		}
		surface_rail = {
			positionOffset = {0, 0},
			crossSectionShape = {
				{nil, 0},
				{nil, 0},
			},
			wrapnodeshape = false,
			colorMode = "static",
			layer = 13,
			allowfullmaterialoptions = true,
			material = {
				shader = "DBN_assets/FG/TrackSurface_Mirror.shader",
				renderqueue = 2000 - 4,
			},
		}
	end

	local lines = {} -- each entry is {left_edge, right_edge}

	-- build a list of lines from the lane dividers and shoulder lines
	do
		local lane_halfwidth, shoulder_halfwidth = options.lane_width / 8 / 2, options.lane_width / 6 / 2

		for i,divider in ipairs(options.lane_dividers) do
			table.insert(lines, {divider - lane_halfwidth, divider + lane_halfwidth})
		end
		for i,shoulder in ipairs(options.shoulder_lines) do
			table.insert(lines, {shoulder - shoulder_halfwidth, shoulder + shoulder_halfwidth})
		end
	end

	-- delete lines that go outside the track
	for i,line in pairs(lines) do
		if line[1] <= -track_width or line[2] >= track_width then
			table.remove(lines, i)
		end
	end

	-- sort the lines from left to right by their left edge
	table.sort(lines, function(a,b) return a[1] < b[1] end)

	-- merge overlapping lines
	do
		-- "for i=1,#lines" would only check #lines the first time
		local i = 2
		while i < #lines do
			local line_a, line_b = lines[i], lines[i-1]
			if line_b[2] > line_a[1] then
				-- widen A and discard B
				line_b[2] = math.max(line_a[2], line_b[2]) -- in case B is completely inside A
				table.remove(lines, i)
			else
				i = i + 1
			end
		end
	end

	surface_rail.material = BuildMaterial(surface_rail.material)

	local divider_shape, surface_shape = divider_rail.crossSectionShape, surface_rail.crossSectionShape
	if #lines == 0 then
		surface_shape[1][1] = -track_width
		surface_shape[2][1] = track_width
		CreateRail(surface_rail)
	else
		divider_rail.material = BuildMaterial(divider_rail.material)

		surface_shape[1][1], surface_shape[2][1] = -track_width, lines[1][1]
		CreateRail(surface_rail)
		surface_shape[1][1], surface_shape[2][1] = lines[#lines][2], track_width
		CreateRail(surface_rail)
		for i=2,#lines do
			surface_shape[1][1], surface_shape[2][1] = lines[i-1][2], lines[i][1]
			CreateRail(surface_rail)
		end

		local divider_shape_x = {}
		for i=1,#divider_shape do divider_shape_x[i] = divider_shape[i][1] end

		for i,line in ipairs(lines) do
			local scale, offset = line[2] - line[1], (line[1] + line[2]) / 2
			for i=1,#divider_shape do divider_shape[i][1] = divider_shape_x[i] * scale + offset end
			CreateRail(divider_rail)
		end
	end
end

---------------------------------------------------------------------------------------------------------------------------
-- Tunnel
---------------------------------------------------------------------------------------------------------------------------
if options.tunnel then
	local tunnel_color_scale = 3
	local track_corner_distance = math.sqrt(options.track_width^2 + options.track_height^2)
	local points = ifquality(64,24,16)
	
	if options.quality == 1 then
		-- low quality, no shape changing
		CreateRail{
			positionOffset = {0, 0},
			crossSectionShape = makeCylinder(track_corner_distance * 3, points, true),
			perShapeNodeColorScalers = clonesArray(tunnel_color_scale, points),
			wrapnodeshape = true,
			colorMode = "aurora",
			shader = "VertexColorUnlitTinted",
			layer = 13,
			renderqueue = 2000,
		}
	else
		-- medium and high quality
		CreateRail{
			positionOffset = {0, 0},
			crossSectionShape = makeCylinder(track_corner_distance * 5, points, true),
			wrapnodeshape = true,
			colorMode = "aurora",
			--fullfuture = true, -- show the whole track from any distance
			shader = "DBN_assets/Tunnel.shader",
			calculatenormals = true,
			color = greyscale(-tunnel_color_scale), -- the function greyscale(x) returns {x,x,x}. it's just something to make things easier. you can also do greyscale(x,y) for {x,x,x,y}
			shadercolors = {
				_Scale = greyscale(-track_corner_distance * 3, 1),
			},
			layer = 13,
			renderqueue = 3000,
		}
		--[[
			_Scale isn't actually a color, it's just a collection of 4 decimal numbers
			it is used to specify the distance that each point should be able to move at 100% intensity
			if you specify a tunnel with a radius of 4 and a scale of 3, the radius can get as small as 1
			the first 3 will scale it along the x y z axes. if you specify negative numbers they will be treated as positive
			doing this because shadercolors can't get any higher than 1 or they get treated as 0-255 colors. and you can't go higher than 255 either
			LOOPHOLE: they CAN go less than -1 without AS2's 255 code triggering. so you can put in negative numbers as big as you want and the shader will math.abs() it after it passes through AS2
			the 4th number will multiply all 3 axes. it DOESN'T do the negative number thing. this lets you specify -1 to make it move outwards instead of inwards
			I found that having them move outwards tends to just hide bright spots since the intense parts tend to be in spikes
			
			color also accepts negative numbers so there's no need to do the perShapeNodeColorScalers hack. perShapeNodeColorScalers would scale the alpha too which would mess up the shader
			
			see the top of the .shader files for more information. (they are text files)
		]]
	end
end

---------------------------------------------------------------------------------------------------------------------------
-- Water transparency limiter
---------------------------------------------------------------------------------------------------------------------------
if options.water_transparency_limiter then
	-- create a black sheet under the water to limit how much you can see through it
	local d = -1.8 -- cliff surface is -2
	local w = options.water_width
	local crossSectionShape
	if options.water_transparency_limiter == 2 then
		crossSectionShape={
			{-w, d},
			{w, d},
		}
	else
		crossSectionShape={
			{-w, 0},
			{-w, d},
			{w, d},
			{w, 0},
		}
	end
	CreateRail{
		positionOffset={0, 0},
		crossSectionShape=crossSectionShape,
		wrapnodeshape = false,
		colorMode="static",
		color = background,
		shader="VertexColorUnlitTinted",
		layer = 13,
		renderqueue = 2000,
	}
end


---------------------------------------------------------------------------------------------------------------------------
-- Aurora rails
---------------------------------------------------------------------------------------------------------------------------
local function createAuroraRail(x, y, w, h, extent, fullfuture)
	w, h = w/2, h/2
	-- solid portion
	createSymRails{
		positionOffset = {x, y},
		crossSectionShape = {
			{-w, h},
			{ w, h},
			{ w,-h},
			{-w,-h},
		},
		colorMode = "highway",
		shader = "DBN_Assets/FG/ColoredRail" .. options.glowing_suffix,
		color = options.glowing_common_tint,
		layer = options.glowing_layer,
		renderqueue = 2000,
	}
	-- outward glow
	createSymRails{
		positionOffset = {x+w, y},
		crossSectionShape = { {0,0}, {extent,0}, },
		perShapeNodeColorScalers = {1,0},
		fullfuture = true,
		stretch = fullfutureStretch(2),
		colorMode = "aurora",
		color = white,
		shader = ifinvert("assets/VertexColorUnlitTintedAlpha.shader", "assets/VertexColorUnlitTintedAdd.shader"),
		layer = 13,
		renderqueue = 3000,
	}
end

if options.track_aurora_ceiling then
	createAuroraRail(
		options.track_width + 17.5, options.track_height + 31.5,
		4, 1,
		ifquality(66, 22, 11),
		not options.track_aurora_edge  )
end
if options.track_aurora_edge then
	createAuroraRail(
		options.track_width + 16, 0,
		2, 1/2,
		ifquality(66, 22, 11),
		true  )
end

scriptCompleted()
end -- THE_ENTIRE_SCRIPT

do
	local status, traceback = xpcall(THE_ENTIRE_SCRIPT, debug.traceback)
	THE_ENTIRE_SCRIPT = nil
	if not status then
		error("\n\nSkin script error. Traceback:\n" .. tostring(traceback) .. "\n\n\nYou can report this bug to the skin author by sending a screenshot of this message (press F12) or copying the contents of output_log.txt")
	end
end

print([[Azure 194 loaded]])

-- erase all global variables
-- any needed by the Update function should be saved in a closure regardless
do
	local g, updateFunction, collectgarbage, printFunction, errorFunction = _G, Update, collectgarbage, fif(options.debug, print, nil), error
	for key in pairs(g) do g[key] = nil end
	Update, print, error = updateFunction, printFunction, errorFunction
	-- it'd be a waste to force debug code to keep an extra variable on the stack for print when it won't be used in release mode
	-- error because it presumably doesn't get called 99% of the time and so forgetting to keep it on the stack won't be noticed until the rare error message fails to display potentially useful information
	collectgarbage()
end
