// Copyright 2014 DeathByNukes

Shader "FG/TrackSurface_Mirror" {
Properties {
	_MainTex ("Base (RGB)", 2D) = "white" {}
	_SurfaceTex ("Specularity (R) Base Multiplier (G)", 2D) = "white" {}
	_Color ("Tint", Color) = (1,1,1,1)
	_SpecularColor ("Specularity Tint", Color) = (1,1,1,1)

	_HighwayScale ("Highway Color Scale", float) = 1
	_IntensityScale ("Intensity Color Scale", float) = 1
}

SubShader {
	GrabPass { }

	Pass {
		Stencil {
			Ref 1
			ReadMask 1
			Comp Equal
		}
		Program "vp" {
// Vertex combos: 1
//   opengl - ALU: 10 to 10
//   d3d9 - ALU: 11 to 11
//   d3d11 - ALU: 7 to 7, TEX: 0 to 0, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
"!!ARBvp1.0
# 10 ALU
PARAM c[5] = { { 1, 0.02, 0, 0.5 },
		state.matrix.mvp };
TEMP R0;
TEMP R1;
DP4 R0.w, vertex.position, c[4];
DP4 R0.z, vertex.position, c[3];
DP4 R0.x, vertex.position, c[1];
DP4 R0.y, vertex.position, c[2];
MUL R1.xyz, R0.xyww, c[0].w;
MOV result.position, R0;
MOV result.color, vertex.color;
MOV result.texcoord[1].zw, R0;
ADD result.texcoord[1].xy, R1, R1.z;
MUL result.texcoord[0].xy, vertex.texcoord[0], c[0];
END
# 10 instructions, 2 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
Matrix 0 [glstate_matrix_mvp]
"vs_2_0
; 11 ALU
def c4, 1.00000000, 0.02000000, 0.50000000, 0
dcl_position0 v0
dcl_texcoord0 v1
dcl_color0 v2
dp4 r0.w, v0, c3
dp4 r0.z, v0, c2
dp4 r0.x, v0, c0
dp4 r0.y, v0, c1
mul r1.xyz, r0.xyww, c4.z
mov r1.y, -r1
add oT1.xy, r1, r1.z
mov oPos, r0
mov oD0, v2
mov oT1.zw, r0
mul oT0.xy, v1, c4
"
}

SubProgram "d3d11 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
ConstBuffer "UnityPerDraw" 336 // 64 used size, 6 vars
Matrix 0 [glstate_matrix_mvp] 4
BindCB "UnityPerDraw" 0
// 11 instructions, 2 temp regs, 0 temp arrays:
// ALU 7 float, 0 int, 0 uint
// TEX 0 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"vs_4_0
eefiecedhnfafoeebkejkdemkicjbajllmcgokhaabaaaaaaomacaaaaadaaaaaa
cmaaaaaajmaaaaaaciabaaaaejfdeheogiaaaaaaadaaaaaaaiaaaaaafaaaaaaa
aaaaaaaaaaaaaaaaadaaaaaaaaaaaaaaapapaaaafjaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaabaaaaaaadadaaaagcaaaaaaaaaaaaaaaaaaaaaaadaaaaaaacaaaaaa
apapaaaafaepfdejfeejepeoaafeeffiedepepfceeaaedepemepfcaaepfdeheo
ieaaaaaaaeaaaaaaaiaaaaaagiaaaaaaaaaaaaaaabaaaaaaadaaaaaaaaaaaaaa
apaaaaaaheaaaaaaaaaaaaaaaaaaaaaaadaaaaaaabaaaaaaadamaaaahnaaaaaa
aaaaaaaaaaaaaaaaadaaaaaaacaaaaaaapaaaaaaheaaaaaaabaaaaaaaaaaaaaa
adaaaaaaadaaaaaaapaaaaaafdfgfpfaepfdejfeejepeoaafeeffiedepepfcee
aaedepemepfcaaklfdeieefclmabaaaaeaaaabaagpaaaaaafjaaaaaeegiocaaa
aaaaaaaaaeaaaaaafpaaaaadpcbabaaaaaaaaaaafpaaaaaddcbabaaaabaaaaaa
fpaaaaadpcbabaaaacaaaaaaghaaaaaepccabaaaaaaaaaaaabaaaaaagfaaaaad
dccabaaaabaaaaaagfaaaaadpccabaaaacaaaaaagfaaaaadpccabaaaadaaaaaa
giaaaaacacaaaaaadiaaaaaipcaabaaaaaaaaaaafgbfbaaaaaaaaaaaegiocaaa
aaaaaaaaabaaaaaadcaaaaakpcaabaaaaaaaaaaaegiocaaaaaaaaaaaaaaaaaaa
agbabaaaaaaaaaaaegaobaaaaaaaaaaadcaaaaakpcaabaaaaaaaaaaaegiocaaa
aaaaaaaaacaaaaaakgbkbaaaaaaaaaaaegaobaaaaaaaaaaadcaaaaakpcaabaaa
aaaaaaaaegiocaaaaaaaaaaaadaaaaaapgbpbaaaaaaaaaaaegaobaaaaaaaaaaa
dgaaaaafpccabaaaaaaaaaaaegaobaaaaaaaaaaadiaaaaakdccabaaaabaaaaaa
egbabaaaabaaaaaaaceaaaaaaaaaiadpaknhkddmaaaaaaaaaaaaaaaadgaaaaaf
pccabaaaacaaaaaaegbobaaaacaaaaaadiaaaaakhcaabaaaabaaaaaamgabbaaa
aaaaaaaaaceaaaaaaaaaaadpaaaaaadpaaaaaalpaaaaaaaadgaaaaafmccabaaa
adaaaaaakgaobaaaaaaaaaaaaaaaaaahdccabaaaadaaaaaafgafbaaaabaaaaaa
igaabaaaabaaaaaadoaaaaab"
}

}
Program "fp" {
// Fragment combos: 1
//   opengl - ALU: 11 to 11, TEX: 3 to 3
//   d3d9 - ALU: 9 to 9, TEX: 3 to 3
//   d3d11 - ALU: 8 to 8, TEX: 3 to 3, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Vector 0 [_Color]
Vector 1 [_SpecularColor]
Float 2 [_HighwayScale]
Float 3 [_IntensityScale]
SetTexture 0 [_GrabTexture] 2D
SetTexture 1 [_SurfaceTex] 2D
SetTexture 2 [_MainTex] 2D
"!!ARBfp1.0
# 11 ALU, 3 TEX
PARAM c[4] = { program.local[0..3] };
TEMP R0;
TEMP R1;
TEMP R2;
TEMP R3;
TEX R3.xy, fragment.texcoord[0], texture[1], 2D;
TEX R0, fragment.texcoord[0], texture[2], 2D;
TXP R1, fragment.texcoord[1], texture[0], 2D;
MUL R2, R3.x, c[1];
MUL R0, R0, fragment.color.primary;
MUL R0, R0, c[0];
MOV R3.x, c[2];
MAD R3.x, R0.w, c[3], R3;
MUL R1, R1, R2;
MUL R0, R0, R3.x;
MAD result.color, R0, R3.y, R1;
END
# 11 instructions, 4 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Vector 0 [_Color]
Vector 1 [_SpecularColor]
Float 2 [_HighwayScale]
Float 3 [_IntensityScale]
SetTexture 0 [_GrabTexture] 2D
SetTexture 1 [_SurfaceTex] 2D
SetTexture 2 [_MainTex] 2D
"ps_2_0
; 9 ALU, 3 TEX
dcl_2d s0
dcl_2d s1
dcl_2d s2
dcl t0.xy
dcl v0
dcl t1
texldp r1, t1, s0
texld r0, t0, s2
texld r4, t0, s1
mul r2, r0, v0
mul r2, r2, c0
mov_pp r0.x, c2
mad_pp r0.x, r2.w, c3, r0
mul_pp r3, r4.x, c1
mul_pp r1, r1, r3
mul_pp r0, r2, r0.x
mad_pp r0, r0, r4.y, r1
mov_pp oC0, r0
"
}

SubProgram "d3d11 " {
Keywords { }
ConstBuffer "$Globals" 64 // 56 used size, 5 vars
Vector 16 [_Color] 4
Vector 32 [_SpecularColor] 4
Float 48 [_HighwayScale]
Float 52 [_IntensityScale]
BindCB "$Globals" 0
SetTexture 0 [_GrabTexture] 2D 2
SetTexture 1 [_SurfaceTex] 2D 1
SetTexture 2 [_MainTex] 2D 0
// 12 instructions, 4 temp regs, 0 temp arrays:
// ALU 8 float, 0 int, 0 uint
// TEX 3 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"ps_4_0
eefieceddcipfhgpcinbnjmalibdeebenocipfeiabaaaaaaaiadaaaaadaaaaaa
cmaaaaaaliaaaaaaomaaaaaaejfdeheoieaaaaaaaeaaaaaaaiaaaaaagiaaaaaa
aaaaaaaaabaaaaaaadaaaaaaaaaaaaaaapaaaaaaheaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaabaaaaaaadadaaaahnaaaaaaaaaaaaaaaaaaaaaaadaaaaaaacaaaaaa
apapaaaaheaaaaaaabaaaaaaaaaaaaaaadaaaaaaadaaaaaaapalaaaafdfgfpfa
epfdejfeejepeoaafeeffiedepepfceeaaedepemepfcaaklepfdeheocmaaaaaa
abaaaaaaaiaaaaaacaaaaaaaaaaaaaaaaaaaaaaaadaaaaaaaaaaaaaaapaaaaaa
fdfgfpfegbhcghgfheaaklklfdeieefcbeacaaaaeaaaaaaaifaaaaaafjaaaaae
egiocaaaaaaaaaaaaeaaaaaafkaaaaadaagabaaaaaaaaaaafkaaaaadaagabaaa
abaaaaaafkaaaaadaagabaaaacaaaaaafibiaaaeaahabaaaaaaaaaaaffffaaaa
fibiaaaeaahabaaaabaaaaaaffffaaaafibiaaaeaahabaaaacaaaaaaffffaaaa
gcbaaaaddcbabaaaabaaaaaagcbaaaadpcbabaaaacaaaaaagcbaaaadlcbabaaa
adaaaaaagfaaaaadpccabaaaaaaaaaaagiaaaaacaeaaaaaaefaaaaajpcaabaaa
aaaaaaaaegbabaaaabaaaaaaeghobaaaacaaaaaaaagabaaaaaaaaaaadiaaaaah
pcaabaaaaaaaaaaaegaobaaaaaaaaaaaegbobaaaacaaaaaadiaaaaaipcaabaaa
aaaaaaaaegaobaaaaaaaaaaaegiocaaaaaaaaaaaabaaaaaadcaaaaalbcaabaaa
abaaaaaadkaabaaaaaaaaaaabkiacaaaaaaaaaaaadaaaaaaakiacaaaaaaaaaaa
adaaaaaadiaaaaahpcaabaaaaaaaaaaaegaobaaaaaaaaaaaagaabaaaabaaaaaa
aoaaaaahdcaabaaaabaaaaaaegbabaaaadaaaaaapgbpbaaaadaaaaaaefaaaaaj
pcaabaaaabaaaaaaegaabaaaabaaaaaaeghobaaaaaaaaaaaaagabaaaacaaaaaa
efaaaaajpcaabaaaacaaaaaaegbabaaaabaaaaaaeghobaaaabaaaaaaaagabaaa
abaaaaaadiaaaaaipcaabaaaadaaaaaaagaabaaaacaaaaaaegiocaaaaaaaaaaa
acaaaaaadiaaaaahpcaabaaaabaaaaaaegaobaaaabaaaaaaegaobaaaadaaaaaa
dcaaaaajpccabaaaaaaaaaaaegaobaaaaaaaaaaafgafbaaaacaaaaaaegaobaaa
abaaaaaadoaaaaab"
}

}

#LINE 73

	}
}
}
