﻿// 2014 DeathByNukes

// With this shader, position in world space will be directly translated to position on the screen.
// X and Y are coordinates on the screen where 0,0 is the center and 1,1 is the top right. X is horizontal and Y is vertical.
// Z is ignored. Use renderqueue to layer things.

Shader "HUD/VertexColorTinted" {
Properties {
	_MainTex ("Base (RGB)", 2D) = "white" {}
	_Color ("Tint", Color) = (1,1,1,1)

	_OffsX ("Screen boundaries", Float) = -1
	_OffsY ("Screen boundaries", Float) = -1
	_ClipX ("Screen boundaries", Float) = 1
	_ClipY ("Screen boundaries", Float) = 1
}

SubShader {
	Tags {"Queue" = "Overlay+750" }
	// If you aren't using any shaders that ignore the depth buffer you can put the HUD at the very beginning of the queue instead.
	// It always writes the highest Z value so nothing should be able to go through it.
	// This will increase the speed because fragments of the scene under the HUD will never be drawn.
	
	Pass {
		Fog { Mode Off }
		//ZWrite Off
		//Blend SrcAlpha OneMinusSrcAlpha // Alpha
		//AlphaTest Greater 0.01
		Cull Off // back faces
		ZTest Always
		Program "vp" {
// Vertex combos: 1
//   opengl - ALU: 7 to 7
//   d3d9 - ALU: 7 to 7
//   d3d11 - ALU: 2 to 2, TEX: 0 to 0, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
Vector 1 [_MainTex_ST]
"!!ARBvp1.0
# 7 ALU
PARAM c[2] = { { -1, 1 },
		program.local[1] };
TEMP R0;
RCP R0.x, vertex.position.w;
MUL R0.xy, vertex.position, R0.x;
MOV R0.zw, c[0].xyxy;
MOV result.position, R0;
MOV result.color, vertex.color;
MAD result.texcoord[0].xy, vertex.texcoord[0], c[1], c[1].zwzw;
MUL result.texcoord[1].xy, R0, c[0].y;
END
# 7 instructions, 1 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
Vector 0 [_MainTex_ST]
"vs_2_0
; 7 ALU
def c1, 0.00000000, 1.00000000, 0, 0
dcl_position0 v0
dcl_texcoord0 v1
dcl_color0 v2
rcp r0.x, v0.w
mul r0.xy, v0, r0.x
mov r0.zw, c1.xyxy
mov oPos, r0
mov oD0, v2
mad oT0.xy, v1, c0, c0.zwzw
mul oT1.xy, r0, c1.y
"
}

SubProgram "d3d11 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
ConstBuffer "$Globals" 64 // 48 used size, 7 vars
Vector 32 [_MainTex_ST] 4
BindCB "$Globals" 0
// 7 instructions, 1 temp regs, 0 temp arrays:
// ALU 2 float, 0 int, 0 uint
// TEX 0 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"vs_4_0
eefiecednjpicdgnolkmfpngoddofkfeobolafchabaaaaaafaacaaaaadaaaaaa
cmaaaaaajmaaaaaaciabaaaaejfdeheogiaaaaaaadaaaaaaaiaaaaaafaaaaaaa
aaaaaaaaaaaaaaaaadaaaaaaaaaaaaaaapalaaaafjaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaabaaaaaaadadaaaagcaaaaaaaaaaaaaaaaaaaaaaadaaaaaaacaaaaaa
apapaaaafaepfdejfeejepeoaafeeffiedepepfceeaaedepemepfcaaepfdeheo
ieaaaaaaaeaaaaaaaiaaaaaagiaaaaaaaaaaaaaaabaaaaaaadaaaaaaaaaaaaaa
apaaaaaaheaaaaaaaaaaaaaaaaaaaaaaadaaaaaaabaaaaaaadamaaaaheaaaaaa
abaaaaaaaaaaaaaaadaaaaaaabaaaaaaamadaaaahnaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaacaaaaaaapaaaaaafdfgfpfaepfdejfeejepeoaafeeffiedepepfcee
aaedepemepfcaaklfdeieefccaabaaaaeaaaabaaeiaaaaaafjaaaaaeegiocaaa
aaaaaaaaadaaaaaafpaaaaadlcbabaaaaaaaaaaafpaaaaaddcbabaaaabaaaaaa
fpaaaaadpcbabaaaacaaaaaaghaaaaaepccabaaaaaaaaaaaabaaaaaagfaaaaad
dccabaaaabaaaaaagfaaaaadmccabaaaabaaaaaagfaaaaadpccabaaaacaaaaaa
giaaaaacabaaaaaadgaaaaaimccabaaaaaaaaaaaaceaaaaaaaaaaaaaaaaaaaaa
aaaaaaaaaaaaiadpaoaaaaahdcaabaaaaaaaaaaaegbabaaaaaaaaaaapgbpbaaa
aaaaaaaadgaaaaafdccabaaaaaaaaaaaegaabaaaaaaaaaaadgaaaaafmccabaaa
abaaaaaaagaebaaaaaaaaaaadcaaaaaldccabaaaabaaaaaaegbabaaaabaaaaaa
egiacaaaaaaaaaaaacaaaaaaogikcaaaaaaaaaaaacaaaaaadgaaaaafpccabaaa
acaaaaaaegbobaaaacaaaaaadoaaaaab"
}

}
Program "fp" {
// Fragment combos: 1
//   opengl - ALU: 11 to 11, TEX: 1 to 1
//   d3d9 - ALU: 15 to 15, TEX: 2 to 2
//   d3d11 - ALU: 7 to 7, TEX: 1 to 1, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Float 0 [_OffsX]
Float 1 [_OffsY]
Float 2 [_ClipX]
Float 3 [_ClipY]
Vector 4 [_Color]
SetTexture 0 [_MainTex] 2D
"!!ARBfp1.0
# 11 ALU, 1 TEX
PARAM c[5] = { program.local[0..4] };
TEMP R0;
TEMP R1;
SLT R0.y, c[2].x, fragment.texcoord[1].x;
SLT R0.x, fragment.texcoord[1], c[0];
ADD_SAT R0.z, R0.x, R0.y;
SLT R0.x, fragment.texcoord[1].y, c[1];
SLT R0.y, c[3].x, fragment.texcoord[1];
ADD_SAT R0.x, R0.z, R0;
ADD_SAT R1.x, R0, R0.y;
TEX R0, fragment.texcoord[0], texture[0], 2D;
KIL -R1.x;
MUL R0, R0, fragment.color.primary;
MUL result.color, R0, c[4];
END
# 11 instructions, 2 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Float 0 [_OffsX]
Float 1 [_OffsY]
Float 2 [_ClipX]
Float 3 [_ClipY]
Vector 4 [_Color]
SetTexture 0 [_MainTex] 2D
"ps_2_0
; 15 ALU, 2 TEX
dcl_2d s0
def c5, 0.00000000, 1.00000000, 0, 0
dcl t0.xy
dcl v0
dcl t1.xy
add r0.x, -t1, c2
add r1.x, t1, -c0
add r2.x, -t1.y, c3
cmp r1.x, r1, c5, c5.y
cmp r0.x, r0, c5, c5.y
add_pp_sat r0.x, r1, r0
add r1.x, t1.y, -c1
cmp r1.x, r1, c5, c5.y
add_pp_sat r0.x, r0, r1
cmp r2.x, r2, c5, c5.y
add_pp_sat r0.x, r0, r2
mov_pp r1, -r0.x
texld r0, t0, s0
texkill r1.xyzw
mul r0, r0, v0
mul r0, r0, c4
mov_pp oC0, r0
"
}

SubProgram "d3d11 " {
Keywords { }
ConstBuffer "$Globals" 64 // 64 used size, 7 vars
Float 16 [_OffsX]
Float 20 [_OffsY]
Float 24 [_ClipX]
Float 28 [_ClipY]
Vector 48 [_Color] 4
BindCB "$Globals" 0
SetTexture 0 [_MainTex] 2D 0
// 10 instructions, 1 temp regs, 0 temp arrays:
// ALU 4 float, 0 int, 3 uint
// TEX 1 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"ps_4_0
eefiecedfalcgmfhibnpeanacjjlamohiooljomcabaaaaaageacaaaaadaaaaaa
cmaaaaaaliaaaaaaomaaaaaaejfdeheoieaaaaaaaeaaaaaaaiaaaaaagiaaaaaa
aaaaaaaaabaaaaaaadaaaaaaaaaaaaaaapaaaaaaheaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaabaaaaaaadadaaaaheaaaaaaabaaaaaaaaaaaaaaadaaaaaaabaaaaaa
amamaaaahnaaaaaaaaaaaaaaaaaaaaaaadaaaaaaacaaaaaaapapaaaafdfgfpfa
epfdejfeejepeoaafeeffiedepepfceeaaedepemepfcaaklepfdeheocmaaaaaa
abaaaaaaaiaaaaaacaaaaaaaaaaaaaaaaaaaaaaaadaaaaaaaaaaaaaaapaaaaaa
fdfgfpfegbhcghgfheaaklklfdeieefchaabaaaaeaaaaaaafmaaaaaafjaaaaae
egiocaaaaaaaaaaaaeaaaaaafkaaaaadaagabaaaaaaaaaaafibiaaaeaahabaaa
aaaaaaaaffffaaaagcbaaaaddcbabaaaabaaaaaagcbaaaadmcbabaaaabaaaaaa
gcbaaaadpcbabaaaacaaaaaagfaaaaadpccabaaaaaaaaaaagiaaaaacabaaaaaa
dbaaaaaidcaabaaaaaaaaaaaogbkbaaaabaaaaaaegiacaaaaaaaaaaaabaaaaaa
dbaaaaaimcaabaaaaaaaaaaakgiocaaaaaaaaaaaabaaaaaakgbobaaaabaaaaaa
dmaaaaahbcaabaaaaaaaaaaackaabaaaaaaaaaaaakaabaaaaaaaaaaadmaaaaah
bcaabaaaaaaaaaaabkaabaaaaaaaaaaaakaabaaaaaaaaaaadmaaaaahbcaabaaa
aaaaaaaadkaabaaaaaaaaaaaakaabaaaaaaaaaaaanaaaeadakaabaaaaaaaaaaa
efaaaaajpcaabaaaaaaaaaaaegbabaaaabaaaaaaeghobaaaaaaaaaaaaagabaaa
aaaaaaaadiaaaaahpcaabaaaaaaaaaaaegaobaaaaaaaaaaaegbobaaaacaaaaaa
diaaaaaipccabaaaaaaaaaaaegaobaaaaaaaaaaaegiocaaaaaaaaaaaadaaaaaa
doaaaaab"
}

}

#LINE 75

	}
}
}
