﻿// 2014 DeathByNukes

// With this shader, position in projection space will be translated to world space then to position on the screen.
// This does a particularly absurd mathematical operation (4x4 matrix inversion) on the GPU per-vertex and may eat a lot of FPS.
// In addition to revealing the bounds of projection space this can also demonstrate the fact that projection depth is nonlinear. (google "depth buffer precision")

Shader "HUD/VisualizeCamera" {
Properties {
	_MainTex ("Base (RGB)", 2D) = "white" {}
	_Color ("Tint", Color) = (1,1,1,1)
	_Scale ("Z Scale", float) = 1
	
	// other things not listed here
}

SubShader {
	Tags {"Queue" = "Overlay+750" }
	// If you aren't using any shaders that ignore the depth buffer you can put the HUD at the very beginning of the queue instead.
	// It always writes the highest Z value so nothing should be able to go through it.
	// This will increase the speed because fragments of the scene under the HUD will never be drawn.
	
	Pass {
		Fog { Mode Off }
		//ZWrite Off
		//Blend SrcAlpha OneMinusSrcAlpha // Alpha
		//AlphaTest Greater 0.01
		Cull Off // back faces
		ZTest Always
		Program "vp" {
// Vertex combos: 1
//   opengl - ALU: 73 to 73
//   d3d9 - ALU: 94 to 94
//   d3d11 - ALU: 60 to 60, TEX: 0 to 0, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
Matrix 1 [unity_MatrixVP]
Float 5 [_OffsX]
Float 6 [_OffsY]
Float 7 [_MinX]
Float 8 [_MinY]
Float 9 [_ScaleX]
Float 10 [_ScaleY]
Vector 11 [_MainTex_ST]
Float 12 [_Scale]
"!!ARBvp1.0
# 73 ALU
PARAM c[13] = { { -1, 1, 2 },
		program.local[1..12] };
TEMP R0;
TEMP R1;
TEMP R2;
TEMP R3;
TEMP R4;
TEMP R5;
TEMP R6;
MOV R2, c[4];
MUL R0, R2.wxyz, c[3].yzwx;
MAD R0, R2.yzwx, c[3].wxyz, -R0;
MUL R3, R0, c[2].zwxy;
MUL R1, R2.zwxy, c[3].wxyz;
MAD R1, R2.wxyz, c[3].zwxy, -R1;
MAD R1, R1, c[2].yzwx, R3;
MUL R0, R2.yzwx, c[3].zwxy;
MAD R0, R2.zwxy, c[3].yzwx, -R0;
MAD R0, R0, c[2].wxyz, R1;
MUL R0, R0, c[1];
DP4 R0.x, R0, c[0].yxyx;
MUL R1.xyz, R2.ywxw, c[3].wxyw;
MAD R1.xyz, R2.wxyw, c[3].ywxw, -R1;
RCP R4.w, R0.x;
DP3 R0.y, R1, c[1].xyww;
DP3 R0.x, R1, c[2].xyww;
MOV R1, c[3];
MUL R4.xyz, R1.ywxw, c[2].wxyw;
MAD R4.xyz, R1.wxyw, c[2].ywxw, -R4;
DP3 R0.w, R4, c[1].xyww;
MUL R4.xyz, R2.yzxw, c[3].zxyw;
MUL R6.xyz, R1.yzxw, c[2].zxyw;
MUL R3.xyz, R2.ywxw, c[2].wxyw;
MAD R3.xyz, R2.wxyw, c[2].ywxw, -R3;
MAD R5.xyz, R2.zxyw, c[3].yzxw, -R4;
MAD R6.xyz, R1.zxyw, c[2].yzxw, -R6;
DP3 R0.z, R3, c[1].xyww;
MOV R0.y, -R0;
MOV R0.w, -R0;
MUL R3, R0, R4.w;
MAD R0.z, vertex.position, c[0], -c[0].y;
MOV R0.xyw, vertex.position;
MUL R0.z, R0, c[12].x;
DP4 R6.w, R3, R0;
MUL R3.xyz, R2.zwyw, c[3].wyzw;
MAD R4.xyz, R2.wyzw, c[3].zwyw, -R3;
DP3 R3.w, R5, c[2];
MOV R3.x, -R3.w;
DP3 R3.y, R5, c[1];
MUL R5.xyz, R2.yzxw, c[2].zxyw;
MAD R5.xyz, R2.zxyw, c[2].yzxw, -R5;
DP3 R1.x, R5, c[1];
MOV R3.z, -R1.x;
DP3 R1.x, R4, c[1].yzww;
DP3 R3.w, R6, c[1];
DP3 R5.x, R4, c[2].yzww;
MUL R4.xyz, R1.zwyw, c[2].wyzw;
MAD R4.xyz, R1.wyzw, c[2].zwyw, -R4;
MOV R5.y, -R1.x;
DP3 R1.w, R4, c[1].yzww;
MUL R3, R4.w, R3;
MUL R1.xyz, R2.zwyw, c[2].wyzw;
MAD R1.xyz, R2.wyzw, c[2].zwyw, -R1;
DP4 R2.x, R0, R3;
MOV R5.w, -R1;
DP3 R5.z, R1, c[1].yzww;
MUL R1, R4.w, R5;
DP4 R6.z, R0, R1;
RCP R0.z, R2.x;
MOV R0.y, c[8].x;
MOV R0.x, c[7];
MAD R1.xy, R6.zwzw, R0.z, -R0;
MOV R0.z, c[9].x;
MOV R0.w, c[10].x;
MOV R0.y, c[6].x;
MOV R0.x, c[5];
MAD R0.xy, R1, R0.zwzw, R0;
MOV result.position.xy, R0;
MOV result.texcoord[1].xy, R0;
MOV result.color, vertex.color;
MOV result.position.zw, c[0].xyxy;
MAD result.texcoord[0].xy, vertex.texcoord[0], c[11], c[11].zwzw;
END
# 73 instructions, 7 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
Matrix 0 [unity_MatrixVP]
Float 4 [_OffsX]
Float 5 [_OffsY]
Float 6 [_MinX]
Float 7 [_MinY]
Float 8 [_ScaleX]
Float 9 [_ScaleY]
Vector 10 [_MainTex_ST]
Float 11 [_Scale]
"vs_2_0
; 94 ALU
dcl_position0 v0
dcl_texcoord0 v1
dcl_color0 v2
def c12, 1.00000000, -1.00000000, 0.00000000, 0
mov r1, c2
mul r2, c3.wxyz, r1.yzwx
mov r1, c2
mad r2, c3.yzwx, r1.wxyz, -r2
mov r0, c2
mul r1, c3.zwxy, r0.wxyz
mov r0, c2
mad r1, c3.wxyz, r0.zwxy, -r1
mul r2, r2, c1.zwxy
mad r2, r1, c1.yzwx, r2
mov r0, c2
mul r1, c3.yzwx, r0.zwxy
mov r0, c2
mad r0, c3.zwxy, r0.yzwx, -r1
mad r0, r0, c1.wxyz, r2
mul r1, r0, c0
dp4 r0.w, r1, c12.xyxy
mov r0.xyz, c2.xyww
mul r1.xyz, c3.ywxw, r0.zxyw
mov r0.xyz, c2.xyww
mad r0.xyz, c3.wxyw, r0.yzxw, -r1
rcp r4.w, r0.w
dp3 r0.w, r0, c0.xyww
dp3 r3.x, r0, c1.xyww
mov r1.xyz, c1.xyww
mul r2.xyz, c2.ywxw, r1.zxyw
mov r1.xyz, c1.xyww
mov r3.y, -r0.w
mad r2.xyz, c2.wxyw, r1.yzxw, -r2
dp3 r0.w, r2, c0.xyww
mov r0.xyz, c1.xyww
mul r1.xyz, c3.ywxw, r0.zxyw
mov r0.xyz, c1.xyww
mad r0.xyz, c3.wxyw, r0.yzxw, -r1
mov r2.xyz, c2
mov r1.xyw, v0
dp3 r3.z, r0, c0.xyww
mov r3.w, -r0
mul r0, r3, r4.w
mul r1.z, v0, c11.x
dp4 r2.w, r0, r1
mul r2.xyz, c3.yzxw, r2.zxyw
mov r0.xyz, c2
mad r3.xyz, c3.zxyw, r0.yzxw, -r2
mov r0.xyz, c2.yzww
mul r2.xyz, c3.zwyw, r0.zxyw
mov r0.xyz, c2.yzww
mad r2.xyz, c3.wyzw, r0.yzxw, -r2
dp3 r0.w, r3, c1
dp3 r0.y, r3, c0
mov r3.xyz, c1
mul r5.xyz, c3.yzxw, r3.zxyw
mov r3.xyz, c1
mad r3.xyz, c3.zxyw, r3.yzxw, -r5
dp3 r0.z, r3, c0
mov r4.xyz, c1
mul r5.xyz, c2.yzxw, r4.zxyw
dp3 r3.y, r2, c0.yzww
dp3 r3.x, r2, c1.yzww
mov r4.xyz, c1
mov r0.x, -r0.w
mad r4.xyz, c2.zxyw, r4.yzxw, -r5
dp3 r0.w, r4, c0
mov r2.xyz, c1.yzww
mul r4.xyz, c2.zwyw, r2.zxyw
mov r2.xyz, c1.yzww
mad r4.xyz, c2.wyzw, r2.yzxw, -r4
dp3 r3.z, r4, c0.yzww
mov r2.xyz, c1.yzww
mul r4.xyz, c3.zwyw, r2.zxyw
mov r0.z, -r0
mul r0, r4.w, r0
dp4 r0.x, r1, r0
rcp r0.z, r0.x
mov r2.xyz, c1.yzww
mad r2.xyz, c3.wyzw, r2.yzxw, -r4
mov r3.w, -r3.z
dp3 r3.z, r2, c0.yzww
mov r3.y, -r3
mul r3, r4.w, r3
mov r0.y, c7.x
mov r0.x, c6
dp4 r2.z, r1, r3
mad r1.xy, r2.zwzw, r0.z, -r0
mov r0.z, c8.x
mov r0.w, c9.x
mov r0.y, c5.x
mov r0.x, c4
mad r0.xy, r1, r0.zwzw, r0
mov oPos.xy, r0
mov oT1.xy, r0
mov oD0, v2
mov oPos.zw, c12.xyzx
mad oT0.xy, v1, c10, c10.zwzw
"
}

SubProgram "d3d11 " {
Keywords { }
Bind "vertex" Vertex
Bind "texcoord" TexCoord0
Bind "color" Color
ConstBuffer "$Globals" 96 // 84 used size, 12 vars
Float 16 [_OffsX]
Float 20 [_OffsY]
Float 32 [_MinX]
Float 36 [_MinY]
Float 40 [_ScaleX]
Float 44 [_ScaleY]
Vector 48 [_MainTex_ST] 4
Float 80 [_Scale]
ConstBuffer "UnityPerFrame" 208 // 208 used size, 4 vars
Matrix 144 [unity_MatrixVP] 4
BindCB "$Globals" 0
BindCB "UnityPerFrame" 1
// 67 instructions, 4 temp regs, 0 temp arrays:
// ALU 60 float, 0 int, 0 uint
// TEX 0 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"vs_4_0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"
}

}
Program "fp" {
// Fragment combos: 1
//   opengl - ALU: 11 to 11, TEX: 1 to 1
//   d3d9 - ALU: 15 to 15, TEX: 2 to 2
//   d3d11 - ALU: 7 to 7, TEX: 1 to 1, FLOW: 1 to 1
SubProgram "opengl " {
Keywords { }
Float 0 [_OffsX]
Float 1 [_OffsY]
Float 2 [_ClipX]
Float 3 [_ClipY]
Vector 4 [_Color]
SetTexture 0 [_MainTex] 2D
"!!ARBfp1.0
# 11 ALU, 1 TEX
PARAM c[5] = { program.local[0..4] };
TEMP R0;
TEMP R1;
SLT R0.y, c[2].x, fragment.texcoord[1].x;
SLT R0.x, fragment.texcoord[1], c[0];
ADD_SAT R0.z, R0.x, R0.y;
SLT R0.x, fragment.texcoord[1].y, c[1];
SLT R0.y, c[3].x, fragment.texcoord[1];
ADD_SAT R0.x, R0.z, R0;
ADD_SAT R1.x, R0, R0.y;
TEX R0, fragment.texcoord[0], texture[0], 2D;
KIL -R1.x;
MUL R0, R0, fragment.color.primary;
MUL result.color, R0, c[4];
END
# 11 instructions, 2 R-regs
"
}

SubProgram "d3d9 " {
Keywords { }
Float 0 [_OffsX]
Float 1 [_OffsY]
Float 2 [_ClipX]
Float 3 [_ClipY]
Vector 4 [_Color]
SetTexture 0 [_MainTex] 2D
"ps_2_0
; 15 ALU, 2 TEX
dcl_2d s0
def c5, 0.00000000, 1.00000000, 0, 0
dcl t0.xy
dcl v0
dcl t1.xy
add r0.x, -t1, c2
add r1.x, t1, -c0
add r2.x, -t1.y, c3
cmp r1.x, r1, c5, c5.y
cmp r0.x, r0, c5, c5.y
add_pp_sat r0.x, r1, r0
add r1.x, t1.y, -c1
cmp r1.x, r1, c5, c5.y
add_pp_sat r0.x, r0, r1
cmp r2.x, r2, c5, c5.y
add_pp_sat r0.x, r0, r2
mov_pp r1, -r0.x
texld r0, t0, s0
texkill r1.xyzw
mul r0, r0, v0
mul r0, r0, c4
mov_pp oC0, r0
"
}

SubProgram "d3d11 " {
Keywords { }
ConstBuffer "$Globals" 96 // 80 used size, 12 vars
Float 16 [_OffsX]
Float 20 [_OffsY]
Float 24 [_ClipX]
Float 28 [_ClipY]
Vector 64 [_Color] 4
BindCB "$Globals" 0
SetTexture 0 [_MainTex] 2D 0
// 10 instructions, 1 temp regs, 0 temp arrays:
// ALU 4 float, 0 int, 3 uint
// TEX 1 (0 load, 0 comp, 0 bias, 0 grad)
// FLOW 1 static, 0 dynamic
"ps_4_0
eefiecedhogkgpndlakekdcgplfbgheceigbaljmabaaaaaageacaaaaadaaaaaa
cmaaaaaaliaaaaaaomaaaaaaejfdeheoieaaaaaaaeaaaaaaaiaaaaaagiaaaaaa
aaaaaaaaabaaaaaaadaaaaaaaaaaaaaaapaaaaaaheaaaaaaaaaaaaaaaaaaaaaa
adaaaaaaabaaaaaaadadaaaaheaaaaaaabaaaaaaaaaaaaaaadaaaaaaabaaaaaa
amamaaaahnaaaaaaaaaaaaaaaaaaaaaaadaaaaaaacaaaaaaapapaaaafdfgfpfa
epfdejfeejepeoaafeeffiedepepfceeaaedepemepfcaaklepfdeheocmaaaaaa
abaaaaaaaiaaaaaacaaaaaaaaaaaaaaaaaaaaaaaadaaaaaaaaaaaaaaapaaaaaa
fdfgfpfegbhcghgfheaaklklfdeieefchaabaaaaeaaaaaaafmaaaaaafjaaaaae
egiocaaaaaaaaaaaafaaaaaafkaaaaadaagabaaaaaaaaaaafibiaaaeaahabaaa
aaaaaaaaffffaaaagcbaaaaddcbabaaaabaaaaaagcbaaaadmcbabaaaabaaaaaa
gcbaaaadpcbabaaaacaaaaaagfaaaaadpccabaaaaaaaaaaagiaaaaacabaaaaaa
dbaaaaaidcaabaaaaaaaaaaaogbkbaaaabaaaaaaegiacaaaaaaaaaaaabaaaaaa
dbaaaaaimcaabaaaaaaaaaaakgiocaaaaaaaaaaaabaaaaaakgbobaaaabaaaaaa
dmaaaaahbcaabaaaaaaaaaaackaabaaaaaaaaaaaakaabaaaaaaaaaaadmaaaaah
bcaabaaaaaaaaaaabkaabaaaaaaaaaaaakaabaaaaaaaaaaadmaaaaahbcaabaaa
aaaaaaaadkaabaaaaaaaaaaaakaabaaaaaaaaaaaanaaaeadakaabaaaaaaaaaaa
efaaaaajpcaabaaaaaaaaaaaegbabaaaabaaaaaaeghobaaaaaaaaaaaaagabaaa
aaaaaaaadiaaaaahpcaabaaaaaaaaaaaegaobaaaaaaaaaaaegbobaaaacaaaaaa
diaaaaaipccabaaaaaaaaaaaegaobaaaaaaaaaaaegiocaaaaaaaaaaaaeaaaaaa
doaaaaab"
}

}

#LINE 111

	}
}
}
